@@ -22,7 +22,7 @@ sdistName = os.environ['_TOIL_SDIST_NAME']
 
 # Make sure to install packages into the pip for the version of Python we are
 # building for.
-pip = 'pip{}'.format(sys.version_info[0])
+pip = 'python{}.{} -m pip'.format(sys.version_info[0], sys.version_info[1])
 
 
 dependencies = ' '.join(['libffi-dev',  # For client side encryption for extras with PyNACL
@@ -125,7 +125,7 @@ print(heredoc('''
     RUN {pip} install --upgrade virtualenv==15.0.3
 
     # Install s3am (--never-download prevents silent upgrades to pip, wheel and setuptools)
-    RUN virtualenv --python python3 --never-download /home/s3am \
+    RUN virtualenv --python python3.6 --never-download /home/s3am \
         && /home/s3am/bin/pip install s3am==2.0 \
         && ln -s /home/s3am/bin/s3am /usr/local/bin/
 
