@@ -1,3 +1,4 @@
+
 # Copyright (C) 2015-2018 Regents of the University of California
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -47,7 +48,6 @@ except ImportError:
 
 log = logging.getLogger(__name__)
 
-
 def which(cmd, mode=os.F_OK | os.X_OK, path=None):
     """
     Copy-pasted in from python3.6's shutil.which().
@@ -136,7 +136,8 @@ def resolveEntryPoint(entryPoint):
     Returns the path to the given entry point (see setup.py) that *should* work on a worker. The
     return value may be an absolute or a relative path.
     """
-    if inVirtualEnv():
+
+    if os.environ.get("TOIL_CHECK_ENV", None) == 'True' and inVirtualEnv():
         path = os.path.join(os.path.dirname(sys.executable), entryPoint)
         # Inside a virtualenv we try to use absolute paths to the entrypoints.
         if os.path.isfile(path):
@@ -146,17 +147,10 @@ def resolveEntryPoint(entryPoint):
             # a virtualenv located at the same path on each worker as well.
             assert os.access(path, os.X_OK)
             return path
-        else:
-            # For virtualenv's that have the toil package directory on their sys.path but whose
-            # bin directory lacks the Toil entrypoints, i.e. where Toil is included via
-            # --system-site-packages, we rely on PATH just as if we weren't in a virtualenv.
-            return entryPoint
-    else:
-        # Outside a virtualenv it is hard to predict where the entry points got installed. It is
-        # the responsibility of the user to ensure that they are present on PATH and point to the
-        # correct version of Toil. This is still better than an absolute path because it gives
-        # the user control over Toil's location on both leader and workers.
-        return entryPoint
+    # Otherwise, we aren't in a virtualenv, or we're in a virtualenv but Toil
+    # came in via --system-site-packages, or we think the virtualenv might not
+    # exist on the workers.
+    return entryPoint
 
 
 @memoize
@@ -416,26 +410,20 @@ def logProcessContext(config):
     from toil.version import version
     log.info("Running Toil version %s.", version)
     log.debug("Configuration: %s", config.__dict__)
-    
 
-def _monkey_patch_boto():
-    """
-    Boto 2 can't automatically assume roles. We want to replace its Provider
-    class that manages credentials with one that uses the Boto 3 configuration
-    and can assume roles.
-    """
-    
+try:
     from boto import provider
     from botocore.session import Session
-    from botocore.credentials import create_credential_resolver, RefreshableCredentials
+    from botocore.credentials import create_credential_resolver, RefreshableCredentials, JSONFileCache 
 
-    # We cache the final credentials so that we don't send multiple processes to
-    # simultaneously bang on the EC2 metadata server or ask for MFA pins from the
-    # user.
     cache_path = '~/.cache/aws/cached_temporary_credentials'
     datetime_format = "%Y-%m-%dT%H:%M:%SZ"  # incidentally the same as the format used by AWS
     log = logging.getLogger(__name__)
 
+   # But in addition to our manual cache, we also are going to turn on boto3's
+   # new built-in caching layer.
+
+     
     def datetime_to_str(dt):
         """
         Convert a naive (implicitly UTC) datetime object into a string, explicitly UTC.
@@ -459,17 +447,25 @@ def _monkey_patch_boto():
         """
         return datetime.strptime(s, datetime_format)
 
+
     class BotoCredentialAdapter(provider.Provider):
         """
         Adapter to allow Boto 2 to use AWS credentials obtained via Boto 3's
         credential finding logic. This allows for automatic role assumption
         respecting the Boto 3 config files, even when parts of the app still use
         Boto 2.
-        
+
         This class also handles cacheing credentials in multi-process environments
         to avoid loads of processes swamping the EC2 metadata service.
         """
-        
+
+
+        """
+        Create a new BotoCredentialAdapter.
+        """
+        # TODO: We take kwargs because new boto2 versions have an 'anon'
+        # argument and we want to be future proof
+
         def __init__(self, name, access_key=None, secret_key=None,
             security_token=None, profile_name=None, **kwargs):
             """
@@ -481,7 +477,9 @@ def _monkey_patch_boto():
             if (name == 'aws' or name is None) and access_key is None and not kwargs.get('anon', False):
                 # We are on AWS and we don't have credentials passed along and we aren't anonymous.
                 # We will backend into a boto3 resolver for getting credentials.
-                self._boto3_resolver = create_credential_resolver(Session(profile=profile_name))
+                # Make sure to enable boto3's own caching, so we can share that
+                # cash with pure boto3 code elsewhere in Toil.
+                self._boto3_resolver = create_credential_resolver(Session(profile=profile_name), cache=JSONFileCache())
             else:
                 # We will use the normal flow
                 self._boto3_resolver = None
@@ -490,13 +488,13 @@ def _monkey_patch_boto():
             super(BotoCredentialAdapter, self).__init__(name, access_key=access_key,
                 secret_key=secret_key, security_token=security_token,
                 profile_name=profile_name, **kwargs)
-            
+
         def get_credentials(self, access_key=None, secret_key=None, security_token=None, profile_name=None):
             """
             Make sure our credential fields are populated. Called by the base class
             constructor.
             """
-            
+
             if self._boto3_resolver is not None:
                 # Go get the credentials from the cache, or from boto3 if not cached.
                 # We need to be eager here; having the default None
@@ -507,28 +505,28 @@ def _monkey_patch_boto():
                 # Use the normal route; our credentials shouldn't expire.
                 super(BotoCredentialAdapter, self).get_credentials(access_key=access_key,
                     secret_key=secret_key, security_token=security_token, profile_name=profile_name)
-            
+
         def _populate_keys_from_metadata_server(self):
             """
             This override is misnamed; it's actually the only hook we have to catch
             _credential_expiry_time being too soon and refresh the credentials. We
             actually just go back and poke the cache to see if it feels like
             getting us new credentials.
-            
+
             Boto 2 hardcodes a refresh within 5 minutes of expiry:
             https://github.com/boto/boto/blob/591911db1029f2fbb8ba1842bfcc514159b37b32/boto/provider.py#L247
-            
+
             Boto 3 wants to refresh 15 or 10 minutes before expiry:
             https://github.com/boto/botocore/blob/8d3ea0e61473fba43774eb3c74e1b22995ee7370/botocore/credentials.py#L279
-            
+
             So if we ever want to refresh, Boto 3 wants to refresh too.
             """
-            
+
             # This should only happen if we have expiring credentials, which we should only get from boto3
             assert(self._boto3_resolver is not None)
-            
+
             self._obtain_credentials_from_cache_or_boto3()
-        
+
         def _obtain_credentials_from_boto3(self):
             """
             We know the current cached credentials are not good, and that we
@@ -536,7 +534,7 @@ def _monkey_patch_boto():
             (_access_key, _secret_key, _security_token,
             _credential_expiry_time) from Boto 3.
             """
-            
+
             # We get a Credentials object
             # <https://github.com/boto/botocore/blob/8d3ea0e61473fba43774eb3c74e1b22995ee7370/botocore/credentials.py#L227>
             # or a RefreshableCredentials, or None on failure.
@@ -551,10 +549,10 @@ def _monkey_patch_boto():
                         except:
                             resolvers = "(Resolvers unavailable)"
                         raise RuntimeError("Could not obtain AWS credentials from Boto3. Resolvers tried: " + resolvers)
-            
+
             # Make sure the credentials actually has some credentials if it is lazy
             creds.get_frozen_credentials()
-            
+
             # Get when the credentials will expire, if ever
             if isinstance(creds, RefreshableCredentials):
                 # Credentials may expire.
@@ -563,21 +561,22 @@ def _monkey_patch_boto():
             else:
                 # Credentials never expire
                 self._credential_expiry_time = None
-            
+
             # Then, atomically get all the credentials bits. They may be newer than we think they are, but never older.
             frozen = creds.get_frozen_credentials()
-            
+
             # Copy them into us
             self._access_key = frozen.access_key
             self._secret_key = frozen.secret_key
             self._security_token = frozen.token
-        
+
         def _obtain_credentials_from_cache_or_boto3(self):
             """
             Get the cached credentials, or retrieve them from Boto 3 and cache them
             (or wait for another cooperating process to do so) if they are missing
             or not fresh enough.
             """
+            cache_path = '~/.cache/aws/cached_temporary_credentials'
             path = os.path.expanduser(cache_path)
             tmp_path = path + '.tmp'
             while True:
@@ -661,13 +660,8 @@ def _monkey_patch_boto():
                     finally:
                         if fd is not None:
                             os.close(fd)
-                            
-    
-    # Now we have defined the adapter class. Patch the Boto module so it replaces the default Provider when Boto makes Providers.
     provider.Provider = BotoCredentialAdapter
-    
-# If Boto is around, try monkey-patching it as soon as anything in Toil loads
-try:
-    _monkey_patch_boto()
+
 except ImportError:
     pass
+
