@@ -60,7 +60,7 @@ class GridEngineBatchSystem(AbstractGridEngineBatchSystem):
         def killJob(self, jobID):
             subprocess.check_call(['qdel', self.getBatchSystemID(jobID)])
 
-        def prepareSubmission(self, cpu, memory, jobID, command):
+        def prepareSubmission(self, cpu, memory, jobID, command, jobName):
             return self.prepareQsub(cpu, memory, jobID) + [command]
 
         def submitJob(self, subLine):
@@ -127,7 +127,7 @@ class GridEngineBatchSystem(AbstractGridEngineBatchSystem):
 
             stdoutfile = self.boss.formatStdOutErrPath(jobID, 'gridengine', '$JOB_ID', 'std_output')
             stderrfile = self.boss.formatStdOutErrPath(jobID, 'gridengine', '$JOB_ID', 'std_error')
-            sbatch_line.extend(['-o', stdoutfile, '-e', stderrfile])
+            qsubline.extend(['-o', stdoutfile, '-e', stderrfile])
 
             return qsubline
 
