@@ -17,7 +17,7 @@
 collect_ignore = []
 
 try:
-    import mesos.interface
+    import pymesos
 except ImportError:
     collect_ignore.append("batchSystem.py")
     collect_ignore.append("executor.py")
