@@ -27,21 +27,14 @@ import sys
 import tempfile
 import time
 import uuid
-from toil import subprocess
 import requests
 from argparse import ArgumentParser
-
-try:
-    import cPickle as pickle
-except ImportError:
-    import pickle
-
-# Python 3 compatibility imports
 from six import iteritems
 
 from toil.lib.humanize import bytes2human
 from toil.lib.retry import retry
-
+from toil import subprocess
+from toil import pickle
 from toil import logProcessContext
 from toil.lib.bioio import addLoggingOptions, getLogLevelString, setLoggingFromOptions
 from toil.realtimeLogger import RealtimeLogger
@@ -49,7 +42,7 @@ from toil.batchSystems.options import addOptions as addBatchOptions
 from toil.batchSystems.options import setDefaultOptions as setDefaultBatchOptions
 from toil.batchSystems.options import setOptions as setBatchOptions
 from toil.provisioners import clusterFactory
-from toil.provisioners.aws import checkValidNodeTypes
+from toil.provisioners.aws import checkValidNodeTypes, zoneToRegion
 from toil import lookupEnvVar
 from toil.version import dockerRegistry, dockerTag
 
@@ -74,6 +67,7 @@ class Config(object):
         self.jobStore = None
         self.logLevel = getLogLevelString()
         self.workDir = None
+        self.noStdOutErr = False
         self.stats = False
 
         # Because the stats option needs the jobStore to persist past the end of the run,
@@ -126,17 +120,20 @@ class Config(object):
         # Misc
         self.disableCaching = True
         self.disableChaining = False
+        self.disableJobStoreChecksumVerification = False
         self.maxLogFileSize = 64000
         self.writeLogs = None
         self.writeLogsGzip = None
+        self.writeLogsFromAllJobs = False
         self.sseKey = None
-        self.cseKey = None
         self.servicePollingInterval = 60
         self.useAsync = True
         self.forceDockerAppliance = False
+        self.runCwlInternalJobsOnWorkers = False
 
         # Debug options
         self.debugWorker = False
+        self.disableWorkerOutputCapture = False
         self.badWorker = 0.0
         self.badWorkerFailInterval = 0.01
 
@@ -198,6 +195,7 @@ class Config(object):
             if not os.path.exists(self.workDir):
                 raise RuntimeError("The path provided to --workDir (%s) does not exist."
                                    % self.workDir)
+        setOption("noStdOutErr")
         setOption("stats")
         setOption("cleanWorkDir")
         setOption("clean")
@@ -267,25 +265,33 @@ class Config(object):
         setOption("maxJobDuration", int, iC(1))
         setOption("rescueJobsFrequency", int, iC(1))
 
-        #Misc
+        # Misc
         setOption("maxLocalJobs", int)
         setOption("disableCaching")
         setOption("disableChaining")
+        setOption("disableJobStoreChecksumVerification")
         setOption("maxLogFileSize", h2b, iC(1))
         setOption("writeLogs")
         setOption("writeLogsGzip")
+        setOption("writeLogsFromAllJobs")
+        setOption("runCwlInternalJobsOnWorkers")
+
+        assert not (self.writeLogs and self.writeLogsGzip), \
+            "Cannot use both --writeLogs and --writeLogsGzip at the same time."
+        assert not self.writeLogsFromAllJobs or self.writeLogs or self.writeLogsGzip, \
+            "To enable --writeLogsFromAllJobs, either --writeLogs or --writeLogsGzip must be set."
 
         def checkSse(sseKey):
             with open(sseKey) as f:
                 assert (len(f.readline().rstrip()) == 32)
 
         setOption("sseKey", checkFn=checkSse)
-        setOption("cseKey", checkFn=checkSse)
         setOption("servicePollingInterval", float, fC(0.0))
         setOption("forceDockerAppliance")
 
         # Debug options
         setOption("debugWorker")
+        setOption("disableWorkerOutputCapture")
         setOption("badWorker", float, fC(0.0, 1.0))
         setOption("badWorkerFailInterval", float, fC(0.0))
 
@@ -305,7 +311,6 @@ jobStoreLocatorHelp = ("A job store holds persistent information about the jobs
                        "aws:<region>:<prefix> where <region> is the name of an AWS region like "
                        "us-west-2 and <prefix> will be prepended to the names of any top-level "
                        "AWS resources in use by job store, e.g. S3 buckets.\n\n "
-                       "azure:<account>:<prefix>\n\n"
                        "google:<project_id>:<prefix> TODO: explain\n\n"
                        "For backwards compatibility, you may also specify ./foo (equivalent to "
                        "file:./foo or just file:foo) or /bar (equivalent to file:/bar).")
@@ -322,11 +327,15 @@ def _addOptions(addGroupFn, config):
                 help="The location of the job store for the workflow. " + jobStoreLocatorHelp)
     addOptionFn("--workDir", dest="workDir", default=None,
                 help="Absolute path to directory where temporary files generated during the Toil "
-                     "run should be placed. Temp files and folders will be placed in a directory "
-                     "toil-<workflowID> within workDir (The workflowID is generated by Toil and "
-                     "will be reported in the workflow logs. Default is determined by the "
+                     "run should be placed. Temp files and folders, as well as standard output "
+                     "and error from batch system jobs (unless --noStdOutErr), will be placed in a "
+                     "directory toil-<workflowID> within workDir. The workflowID is generated by "
+                     "Toil and will be reported in the workflow logs. Default is determined by the "
                      "variables (TMPDIR, TEMP, TMP) via mkdtemp. This directory needs to exist on "
-                     "all machines running jobs.")
+                     "all machines running jobs; if capturing standard output and error from batch "
+                     "system jobs is desired, it will generally need to be on a shared file system.")
+    addOptionFn("--noStdOutErr", dest="noStdOutErr", action="store_true", default=None,
+                help="Do not capture standard output and error from batch system jobs.")
     addOptionFn("--stats", dest="stats", action="store_true", default=None,
                 help="Records statistics about the toil workflow to be used by 'toil stats'.")
     addOptionFn("--clean", dest="clean", choices=['always', 'onError', 'never', 'onSuccess'],
@@ -376,9 +385,9 @@ def _addOptions(addGroupFn, config):
                              "in an autoscaled cluster, as well as parameters to control the "
                              "level of provisioning.")
 
-    addOptionFn("--provisioner", dest="provisioner", choices=['aws', 'azure', 'gce'],
+    addOptionFn("--provisioner", dest="provisioner", choices=['aws', 'gce'],
                 help="The provisioner for cluster auto-scaling. The currently supported choices are"
-                     "'azure', 'gce', or 'aws'. The default is %s." % config.provisioner)
+                     "'gce', or 'aws'. The default is %s." % config.provisioner)
 
     addOptionFn('--nodeTypes', default=None,
                 help="List of node types separated by commas. The syntax for each node type "
@@ -524,13 +533,19 @@ def _addOptions(addGroupFn, config):
     # Misc options
     #
     addOptionFn = addGroupFn("Toil Miscellaneous Options", "Miscellaneous Options")
-    addOptionFn('--disableCaching', dest='disableCaching', action='store_true', default=True,
+    addOptionFn('--disableCaching', dest='disableCaching',
+                type='bool', nargs='?', const=True, default=True,
                 help='Disables caching in the file store. This flag must be set to use '
                      'a batch system that does not support caching such as Grid Engine, Parasol, '
                      'LSF, or Slurm')
     addOptionFn('--disableChaining', dest='disableChaining', action='store_true', default=False,
                 help="Disables chaining of jobs (chaining uses one job's resource allocation "
                 "for its successor job if possible).")
+    addOptionFn("--disableJobStoreChecksumVerification", dest="disableJobStoreChecksumVerification",
+                default=False, action="store_true",
+                help=("Disables checksum verification for files transferred to/from the job store. "
+                      "Checksum verification is a safety check to ensure the data is not corrupted "
+                      "during transfer. Currently only supported for non-streaming AWS files."))
     addOptionFn("--maxLogFileSize", dest="maxLogFileSize", default=None,
                 help=("The maximum size of a job log file to keep (in bytes), log files "
                       "larger than this will be truncated to the last X bytes. Setting "
@@ -540,30 +555,35 @@ def _addOptions(addGroupFn, config):
     addOptionFn("--writeLogs", dest="writeLogs", nargs='?', action='store',
                 default=None, const=os.getcwd(),
                 help="Write worker logs received by the leader into their own files at the "
-                     "specified path. The current working directory will be used if a path is "
-                     "not specified explicitly. Note: By default "
-                     "only the logs of failed jobs are returned to leader. Set log level to "
-                     "'debug' to get logs back from successful jobs, and adjust 'maxLogFileSize' "
-                     "to control the truncation limit for worker logs.")
+                     "specified path. Any non-empty standard output and error from failed batch "
+                     "system jobs will also be written into files at this path. "
+                     "The current working directory will be used if a path is "
+                     "not specified explicitly. Note: By default only the logs of failed jobs are "
+                     "returned to leader. Set log level to 'debug' or enable "
+                     "'--writeLogsFromAllJobs' to get logs back from successful jobs, and adjust "
+                     "'maxLogFileSize' to control the truncation limit for worker logs.")
     addOptionFn("--writeLogsGzip", dest="writeLogsGzip", nargs='?', action='store',
                 default=None, const=os.getcwd(),
                 help="Identical to --writeLogs except the logs files are gzipped on the leader.")
+    addOptionFn("--writeLogsFromAllJobs", dest="writeLogsFromAllJobs", action='store_true',
+                default=False,
+                help="Whether to write logs from all jobs (including the successful ones) without "
+                     "necessarily setting the log level to 'debug'. Ensure that either --writeLogs "
+                     "or --writeLogsGzip is set if enabling this option.")
     addOptionFn("--realTimeLogging", dest="realTimeLogging", action="store_true", default=False,
                 help="Enable real-time logging from workers to masters")
 
     addOptionFn("--sseKey", dest="sseKey", default=None,
                 help="Path to file containing 32 character key to be used for server-side encryption on "
                      "awsJobStore or googleJobStore. SSE will not be used if this flag is not passed.")
-    addOptionFn("--cseKey", dest="cseKey", default=None,
-                help="Path to file containing 256-bit key to be used for client-side encryption on "
-                     "azureJobStore. By default, no encryption is used.")
     addOptionFn("--setEnv", '-e', metavar='NAME=VALUE or NAME',
                 dest="environment", default=[], action="append",
                 help="Set an environment variable early on in the worker. If VALUE is omitted, "
                      "it will be looked up in the current environment. Independently of this "
                      "option, the worker will try to emulate the leader's environment before "
-                     "running a job. Using this option, a variable can be injected into the "
-                     "worker process itself before it is started.")
+                     "running a job, except for some variables known to vary across systems. "
+                     "Using this option, a variable can be injected into the worker process "
+                     "itself before it is started.")
     addOptionFn("--servicePollingInterval", dest="servicePollingInterval", default=None,
                 help="Interval of time service jobs wait between polling for the existence"
                 " of the keep-alive flag (defailt=%s)" % config.servicePollingInterval)
@@ -579,6 +599,9 @@ def _addOptions(addGroupFn, config):
                 help="Experimental no forking mode for local debugging."
                      " Specifically, workers are not forked and"
                      " stderr/stdout are not redirected to the log.")
+    addOptionFn("--disableWorkerOutputCapture", default=False, action="store_true",
+                help="Let worker output go to worker's standard"
+                     " out/error instead of per-job logs.")
     addOptionFn("--badWorker", dest="badWorker", default=None,
                 help=(
                 "For testing purposes randomly kill 'badWorker' proportion of jobs using SIGKILL, default=%s" % config.badWorker))
@@ -598,14 +621,17 @@ def addOptions(parser, config=Config()):
         def addGroup(headingString, bodyString):
             return parser.add_argument_group(headingString, bodyString).add_argument
 
+        parser.register("type", "bool", lambda v: v.lower() == "true")  # Custom type for arg=True/False.
         _addOptions(addGroup, config)
     else:
         raise RuntimeError("Unanticipated class passed to addOptions(), %s. Expecting "
                            "argparse.ArgumentParser" % parser.__class__)
 
-def getNodeID(extraIDFiles=None):
+
+def getNodeID():
     """
     Return unique ID of the current node (host).
+
     Tries several methods until success. The returned ID should be identical across calls from different processes on
     the same node at least until the next OS reboot.
 
@@ -613,30 +639,20 @@ def getNodeID(extraIDFiles=None):
     called. However, this method should never be reached on a Linux system, because reading from
     /proc/sys/kernel/random/boot_id will be tried prior to that. If uuid.getnode() is reached, it will be called twice,
     and exception raised if the values are not identical.
-
-    :param list extraIDFiles: Optional list of additional file names to try reading node ID before trying default
-    methods. ID should be a single word (no spaces) on the first line of the file.
     """
-    if extraIDFiles is None:
-        extraIDFiles = []
-    idSourceFiles = extraIDFiles + ["/var/lib/dbus/machine-id", "/proc/sys/kernel/random/boot_id"]
-    for idSourceFile in idSourceFiles:
+    for idSourceFile in ["/var/lib/dbus/machine-id", "/proc/sys/kernel/random/boot_id"]:
         if os.path.exists(idSourceFile):
             try:
                 with open(idSourceFile, "r") as inp:
                     nodeID = inp.readline().strip()
             except EnvironmentError:
-                logger.warning((
-                               "Exception when trying to read ID file {}. Will try next method to get node ID"). \
-                               format(idSourceFile), exc_info=True)
+                logger.warning(("Exception when trying to read ID file {}. Will try next method to get node ID.").format(idSourceFile), exc_info=True)
             else:
                 if len(nodeID.split()) == 1:
                     logger.debug("Obtained node ID {} from file {}".format(nodeID, idSourceFile))
                     break
                 else:
-                    logger.warning((
-                                   "Node ID {} from file {} contains spaces. Will try next method to get node ID"). \
-                                   format(nodeID, idSourceFile))
+                    logger.warning(("Node ID {} from file {} contains spaces. Will try next method to get node ID.").format(nodeID, idSourceFile))
     else:
         nodeIDs = []
         for i_call in range(2):
@@ -843,13 +859,8 @@ class Toil(object):
             from toil.jobStores.fileJobStore import FileJobStore
             return FileJobStore(rest)
         elif name == 'aws':
-            from toil.lib.ec2Credentials import enable_metadata_credential_caching
             from toil.jobStores.aws.jobStore import AWSJobStore
-            enable_metadata_credential_caching()
             return AWSJobStore(rest)
-        elif name == 'azure':
-            from toil.jobStores.azureJobStore import AzureJobStore
-            return AzureJobStore(rest)
         elif name == 'google':
             from toil.jobStores.googleJobStore import GoogleJobStore
             return GoogleJobStore(rest)
@@ -1083,7 +1094,7 @@ class Toil(object):
         So far there is no reason to store any old pids.
         """
         with self._jobStore.writeSharedFileStream('pid.log') as f:
-            f.write(str(os.getpid()))
+            f.write(str(os.getpid()).encode('utf-8'))
 
 
 class ToilRestartException(Exception):
@@ -1099,10 +1110,16 @@ class ToilContextManagerException(Exception):
 
 class ToilMetrics:
     def __init__(self, provisioner=None):
+        clusterName = 'none'
+        region = 'us-west-2'
         if provisioner is not None:
-            self.clusterName = provisioner.clusterName
-        else:
-            self.clusterName = "none"
+            clusterName = provisioner.clusterName
+            if provisioner._zone is not None:
+                if provisioner.cloud == 'aws':
+                    # Remove AZ name
+                    region = zoneToRegion(provisioner._zone)
+                else:
+                    region = provisioner._zone
 
         registry = lookupEnvVar(name='docker registry',
                                 envName='TOIL_DOCKER_REGISTRY',
@@ -1112,7 +1129,7 @@ class ToilMetrics:
         self.grafanaImage = "%s/toil-grafana:%s" % (registry, dockerTag)
         self.prometheusImage = "%s/toil-prometheus:%s" % (registry, dockerTag)
 
-        self.startDashboard(clusterName=self.clusterName)
+        self.startDashboard(clusterName=clusterName, zone=region)
 
         # Always restart the mtail container, because metrics should start from scratch
         # for each workflow
@@ -1161,12 +1178,12 @@ class ToilMetrics:
     def _containerRunning(containerName):
         try:
             result = subprocess.check_output(["docker", "inspect", "-f",
-                                              "'{{.State.Running}}'", containerName]) == "true"
+                                              "'{{.State.Running}}'", containerName]).decode('utf-8') == "true"
         except subprocess.CalledProcessError:
             result = False
         return result
 
-    def startDashboard(self, clusterName):
+    def startDashboard(self, clusterName, zone):
         try:
             if not self._containerRunning("toil_prometheus"):
                 try:
@@ -1179,7 +1196,8 @@ class ToilMetrics:
                                        "-d",
                                        "-p", "9090:9090",
                                        self.prometheusImage,
-                                       clusterName])
+                                       clusterName,
+                                       zone])
 
             if not self._containerRunning("toil_grafana"):
                 try:
@@ -1318,11 +1336,11 @@ def getDirSizeRecursively(dirPath):
     This method will return the cumulative number of bytes occupied by the files
     on disk in the directory and its subdirectories.
 
-    This method will raise a 'subprocess.CalledProcessError' if it is unable to
-    access a folder or file because of insufficient permissions.  Therefore this
-    method should only be called on the jobStore, and will alert the user if some
-    portion is inaccessible.  Everything in the jobStore should have appropriate
-    permissions as there is no way to read the filesize without permissions.
+    If the method is unable to access a file or directory (due to insufficient
+    permissions, or due to the file or directory having been removed while this
+    function was attempting to traverse it), the error will be handled
+    internally, and a (possibly 0) lower bound on the size of the directory
+    will be returned.
 
     The environment variable 'BLOCKSIZE'='512' is set instead of the much cleaner
     --block-size=1 because Apple can't handle it.
@@ -1336,9 +1354,13 @@ def getDirSizeRecursively(dirPath):
     # The call: 'du -s /some/path' should give the number of 512-byte blocks
     # allocated with the environment variable: BLOCKSIZE='512' set, and we
     # multiply this by 512 to return the filesize in bytes.
-    return int(subprocess.check_output(['du', '-s', dirPath],
-                                       env=dict(os.environ, BLOCKSIZE='512')).split()[0].decode(
-        'ascii')) * 512
+    
+    try:
+        return int(subprocess.check_output(['du', '-s', dirPath],
+                                           env=dict(os.environ, BLOCKSIZE='512')).decode('utf-8').split()[0]) * 512
+    except subprocess.CalledProcessError:
+        # Something was inaccessible or went away
+        return 0
 
 
 def getFileSystemSize(dirPath):
