@@ -30,8 +30,10 @@ import logging
 import copy
 import functools
 import shutil
-from typing import Text, Mapping, MutableSequence
+from typing import Text, Mapping, MutableSequence, MutableMapping
 import hashlib
+import uuid
+import datetime
 
 # Python 3 compatibility imports
 from six import iteritems, string_types
@@ -51,8 +53,10 @@ import cwltool.builder
 import cwltool.resolver
 import cwltool.stdfsaccess
 import cwltool.command_line_tool
+import cwltool.provenance
 
 from toil.jobStores.abstractJobStore import NoSuchJobStoreException
+from toil.fileStores import FileID
 from cwltool.loghandler import _logger as cwllogger
 from cwltool.loghandler import defaultStreamHandler
 from cwltool.pathmapper import (PathMapper, adjustDirObjs, adjustFileObjs,
@@ -152,7 +156,10 @@ class DefaultWithSource(object):
     def resolve(self):
         """Determine the final input value."""
         if self.source:
-            result = self.source[1][self.source[0]]
+            if isinstance(self.source, tuple):
+                result = self.source[1][0][self.source[0]]
+            else:
+                result = self.source[1][self.source[0]]
             if result:
                 return result
         return self.default
@@ -166,12 +173,21 @@ def _resolve_indirect_inner(maybe_idict):
 
     if isinstance(maybe_idict, IndirectDict):
         result = {}
+        metadata = {}
         for key, value in list(maybe_idict.items()):
             if isinstance(value, (MergeInputs, DefaultWithSource)):
                 result[key] = value.resolve()
             else:
-                result[key] = value[1].get(value[0])
-        return result
+                if isinstance(value[1], tuple):
+                    if isinstance(value[1][0], tuple):
+                        result[key] = value[1][0][0].get(value[0])
+                        metadata[key] = value[1][0][1]
+                    else:
+                        result[key] = value[1][0].get(value[0])
+                        metadata[key] = value[1][1]
+                else:
+                    result[key] = value[1].get(value[0])
+        return result, metadata
     return maybe_idict
 
 
@@ -182,6 +198,9 @@ def resolve_indirect(pdict):
 
     inner = IndirectDict() if isinstance(pdict, IndirectDict) else {}
     needs_eval = False
+    if isinstance(pdict, tuple):
+        #TODO should this return metadata
+        return resolve_indirect(pdict[0])
     for k, value in iteritems(pdict):
         if isinstance(value, StepValueFrom):
             inner[k] = value.inner
@@ -191,12 +210,21 @@ def resolve_indirect(pdict):
     res = _resolve_indirect_inner(inner)
     if needs_eval:
         ev = {}
+        metadata = {}
         for k, value in iteritems(pdict):
             if isinstance(value, StepValueFrom):
-                ev[k] = value.do_eval(res, res[k])
+                if isinstance(res, tuple):
+                    ev[k] = value.do_eval(res[0], res[0][k])
+                    metadata[k] = res[1]
+                else:
+                    ev[k] = value.do_eval(res, res[k])
             else:
-                ev[k] = res[k]
-        return ev
+                if isinstance(res, tuple):
+                    ev[k] = res[0][k]
+                    metadata[k] = res[1]
+                else:
+                    ev[k] = res[k]
+        return ev, metadata
     return res
 
 
@@ -211,11 +239,10 @@ def simplify_list(maybe_list):
 
 
 class ToilPathMapper(PathMapper):
-    """ToilPathMapper keeps track of a file's symbolic identifier (the Toil
-    FileStore token), its local path on the host (the value returned by
-    readGlobalFile) and the the location of the file inside the Docker
-    container.
-
+    """
+    ToilPathMapper keeps track of a file's symbolic identifier (the Toil
+    FileID), its local path on the host (the value returned by readGlobalFile)
+    and the the location of the file inside the Docker container.
     """
 
     def __init__(self, referenced_files, basedir, stagedir,
@@ -297,7 +324,7 @@ class ToilFsAccess(cwltool.stdfsaccess.StdFsAccess):
 
     def _abs(self, path):
         if path.startswith("toilfs:"):
-            return self.file_store.readGlobalFile(path[7:])
+            return self.file_store.readGlobalFile(FileID.unpack(path[7:]))
         return super(ToilFsAccess, self)._abs(path)
 
 
@@ -306,7 +333,7 @@ def toil_get_file(file_store, index, existing, file_store_id):
 
     if not file_store_id.startswith("toilfs:"):
         return file_store.jobStore.getPublicUrl(file_store.jobStore.importFile(file_store_id))
-    src_path = file_store.readGlobalFile(file_store_id[7:])
+    src_path = file_store.readGlobalFile(FileID.unpack(file_store_id[7:]))
     index[src_path] = file_store_id
     existing[file_store_id] = src_path
     return schema_salad.ref_resolver.file_uri(src_path)
@@ -334,7 +361,7 @@ def write_file(writeFunc, index, existing, x):
             else:
                 rp = x
             try:
-                index[x] = "toilfs:" + writeFunc(rp)
+                index[x] = "toilfs:" + writeFunc(rp).pack()
                 existing[index[x]] = x
             except Exception as e:
                 cwllogger.error("Got exception '%s' while copying '%s'", e, x)
@@ -407,7 +434,7 @@ def toilStageFiles(file_store, cwljob, outdir, index, existing, export,
     jobfiles = list(_collectDirEntries(cwljob))
     pm = ToilPathMapper(
         jobfiles, "", outdir, separateDirs=False, stage_listing=True)
-    for f, p in pm.items():
+    for _, p in pm.items():
         if not p.staged:
             continue
 
@@ -422,14 +449,14 @@ def toilStageFiles(file_store, cwljob, outdir, index, existing, export,
                 destUrl = '/'.join(s.strip('/')
                                    for s in [destBucket, unstageTargetPath])
 
-                file_store.exportFile(p.resolved[7:], destUrl)
+                file_store.exportFile(FileID.unpack(p.resolved[7:]), destUrl)
 
             continue
 
         if not os.path.exists(os.path.dirname(p.target)):
             os.makedirs(os.path.dirname(p.target), 0o0755)
         if p.type == "File":
-            file_store.exportFile(p.resolved[7:], "file://" + p.target)
+            file_store.exportFile(FileID.unpack(p.resolved[7:]), "file://" + p.target)
         elif p.type == "Directory" and not os.path.exists(p.target):
             os.makedirs(p.target, 0o0755)
         elif p.type == "CreateFile":
@@ -460,47 +487,20 @@ class CWLJobWrapper(Job):
         self.runtime_context = runtime_context
 
     def run(self, file_store):
-        cwljob = resolve_indirect(self.cwljob)
+        resolved_cwljob = resolve_indirect(self.cwljob)
+        metadata = {}
+        if isinstance(resolved_cwljob, tuple):
+            cwljob = resolved_cwljob[0]
+            metadata = resolved_cwljob[1]
+        else:
+            cwljob = resolved_cwljob
         fill_in_defaults(
             self.cwltool.tool['inputs'], cwljob,
             self.runtime_context.make_fs_access(
                 self.runtime_context.basedir or ""))
         realjob = CWLJob(self.cwltool, cwljob, self.runtime_context)
         self.addChild(realjob)
-        return realjob.rv()
-
-def _makeNestedTempDir(top, seed, levels=2):
-    """
-    Gets a temporary directory in the hierarchy of directories under a given
-    top directory.
-
-    This exists to avoid placing too many temporary directories under a single
-    top in a flat structure, which can slow down metadata updates such as
-    deletes on the local file system.
-
-    The seed parameter allows for deterministic placement of the created
-    directory. The seed is hashed into hex digest and the directory structure
-    is created from the initial letters of the digest.
-
-    :param top : string, top directory for the hierarchy
-    :param seed : string, the hierarchy will be generated from this seed string
-    :rtype : string, path to temporary directory - will be created when
-        necessary.
-    """
-    # Valid chars for the creation of temporary directories
-    validDirs = hashlib.md5(six.b(str(seed))).hexdigest()
-    tempDir = top
-    for i in range(max(min(levels, len(validDirs)), 1)):
-        tempDir = os.path.join(tempDir, validDirs[i])
-        if not os.path.exists(tempDir):
-            try:
-                os.makedirs(tempDir)
-            except os.error:
-                if not os.path.exists(tempDir):
-                    # In the case that a collision occurs and
-                    # it is created while we wait then we ignore
-                    raise
-    return tempDir
+        return realjob.rv(), metadata
 
 
 class CWLJob(Job):
@@ -561,10 +561,14 @@ class CWLJob(Job):
         self.runtime_context = runtime_context
         self.step_inputs = step_inputs or self.cwltool.tool["inputs"]
         self.workdir = runtime_context.workdir
-        self.openTempDirs = []
 
     def run(self, file_store):
-        cwljob = resolve_indirect(self.cwljob)
+        resolved_cwljob = resolve_indirect(self.cwljob)
+        if isinstance(resolved_cwljob, tuple):
+            cwljob, metadata = resolved_cwljob
+        else:
+            cwljob = resolved_cwljob
+            metadata = {}
         fill_in_defaults(
             self.step_inputs, cwljob,
             self.runtime_context.make_fs_access(""))
@@ -578,18 +582,21 @@ class CWLJob(Job):
                 cwljob.pop(inp_id)
 
         # Exports temporary directory for batch systems that reset TMPDIR
-        os.environ["TMPDIR"] = os.path.realpath(
-            self.runtime_context.tmpdir or file_store.getLocalTempDir())
+        os.environ["TMPDIR"] = os.path.realpath(file_store.getLocalTempDir())
         outdir = os.path.join(file_store.getLocalTempDir(), "out")
         os.mkdir(outdir)
-        top_tmp_outdir = self.workdir or os.environ["TMPDIR"]
-        tmp_outdir_prefix = os.path.join(
-            _makeNestedTempDir(top=top_tmp_outdir, seed=outdir, levels=2),
-            "out_tmpdir")
-        self.openTempDirs.append(top_tmp_outdir)
+        # Just keep the temporary output prefix under the job's local temp dir,
+        # next to the outdir.
+        #
+        # If we maintain our own system of nested temp directories, we won't
+        # know when all the jobs using a higher-level directory are ready for
+        # it to be deleted. The local temp dir, under Toil's workDir, will be
+        # cleaned up by Toil.
+        tmp_outdir_prefix = os.path.join(file_store.getLocalTempDir(), "tmp-out")
 
         index = {}
         existing = {}
+        # Prepare the run instructions for cwltool
         runtime_context = self.runtime_context.copy()
         runtime_context.basedir = os.getcwd()
         runtime_context.outdir = outdir
@@ -599,9 +606,13 @@ class CWLJob(Job):
             ToilFsAccess, file_store=file_store)
         runtime_context.toil_get_file = functools.partial(
             toil_get_file, file_store, index, existing)
+
+        process_uuid = uuid.uuid4()
+        started_at = datetime.datetime.now()
         # Run the tool
         (output, status) = cwltool.executors.SingleJobExecutor().execute(
             self.cwltool, cwljob, runtime_context, cwllogger)
+        ended_at = datetime.datetime.now()
         if status != "success":
             raise cwltool.errors.WorkflowException(status)
 
@@ -613,7 +624,15 @@ class CWLJob(Job):
             uploadFile, functools.partial(writeGlobalFileWrapper, file_store),
             index, existing))
 
-        return output
+        metadata[process_uuid] = {
+            'started_at': started_at,
+            'ended_at': ended_at,
+            'job_order': cwljob,
+            'outputs': output,
+            'internal_name': self.jobName
+        }
+        return output, metadata
+
 
 def makeJob(tool, jobobj, step_inputs, runtime_context):
     """Create the correct Toil Job object for the CWL tool (workflow, job, or job
@@ -690,7 +709,11 @@ class CWLScatter(Job):
         return outputs
 
     def run(self, file_store):
-        cwljob = resolve_indirect(self.cwljob)
+        resolved_cwljob = resolve_indirect(self.cwljob)
+        if isinstance(resolved_cwljob, tuple):
+            cwljob, metadata = resolved_cwljob
+        else:
+            cwljob = resolved_cwljob
 
         if isinstance(self.step.tool["scatter"], string_types):
             scatter = [self.step.tool["scatter"]]
@@ -713,8 +736,8 @@ class CWLScatter(Job):
             def valueFromFunc(k, v):
                 if k in valueFrom:
                     return cwltool.expression.do_eval(
-                            valueFrom[k], shortio, self.step.requirements,
-                            None, None, {}, context=v)
+                        valueFrom[k], shortio, self.step.requirements,
+                        None, None, {}, context=v)
                 else:
                     return v
             return {k: valueFromFunc(k, v) for k, v in list(io.items())}
@@ -745,7 +768,7 @@ class CWLScatter(Job):
                     "Must provide scatterMethod to scatter over multiple"
                     " inputs.")
 
-        return outputs
+        return outputs, metadata
 
 
 class CWLGather(Job):
@@ -772,14 +795,27 @@ class CWLGather(Job):
             return obj.get(k)
         elif isinstance(obj, MutableSequence):
             cp = []
+            metadata = []
             for l in obj:
-                cp.append(self.extract(l, k))
-            return cp
+                if isinstance(l, tuple):
+                    cp.append(self.extract(l[0], k))
+                    metadata.append(self.extract(l[1], k))
+                else:
+                    result = self.extract(l, k)
+                    if isinstance(result, tuple):
+                        cp.append(result[0])
+                        metadata.append(result[1])
+                    else:
+                        cp.append(result)
+            return cp, metadata
+        elif isinstance(obj, tuple):
+            return self.extract(obj[0], k)
         else:
             return []
 
     def run(self, file_store):
         outobj = {}
+        metadata = {}
 
         def sn(n):
             if isinstance(n, Mapping):
@@ -788,9 +824,12 @@ class CWLGather(Job):
                 return shortname(n)
 
         for k in [sn(i) for i in self.step.tool["out"]]:
-            outobj[k] = self.extract(self.outputs, k)
-
-        return outobj
+            result = self.extract(self.outputs, k)
+            if isinstance(result, tuple):
+                outobj[k], metadata[k] = result
+            else:
+                outobj[k] = result
+        return outobj, metadata
 
 
 class SelfJob(object):
@@ -823,6 +862,19 @@ def remove_pickle_problems(obj):
     return obj
 
 
+def _link_merge_source(promises, in_out_obj, source_obj):
+    to_merge = [(shortname(s), promises[s].rv()) for s in aslist(source_obj)]
+    link_merge = in_out_obj.get("linkMerge", "merge_nested")
+    if link_merge == "merge_nested":
+        merged = MergeInputsNested(to_merge)
+    elif link_merge == "merge_flattened":
+        merged = MergeInputsFlattened(to_merge)
+    else:
+        raise validate.ValidationException(
+            "Unsupported linkMerge '%s'" % link_merge)
+    return merged
+
+
 class CWLWorkflow(Job):
     """Traverse a CWL workflow graph and create a Toil job graph with appropriate
     dependencies.
@@ -856,7 +908,7 @@ class CWLWorkflow(Job):
         while not alloutputs_fufilled:
             # Iteratively go over the workflow steps, scheduling jobs as their
             # dependencies can be fufilled by upstream workflow inputs or
-            # step outputs.  Loop exits when the workflow outputs
+            # step outputs. Loop exits when the workflow outputs
             # are satisfied.
 
             alloutputs_fufilled = True
@@ -875,30 +927,12 @@ class CWLWorkflow(Job):
                         for inp in step.tool["inputs"]:
                             key = shortname(inp["id"])
                             if "source" in inp:
+                                inpSource = inp["source"]
                                 if inp.get("linkMerge") \
                                         or len(aslist(inp["source"])) > 1:
-                                    linkMerge = inp.get(
-                                        "linkMerge", "merge_nested")
-                                    if linkMerge == "merge_nested":
-                                        jobobj[key] = (
-                                            MergeInputsNested(
-                                                [(shortname(s),
-                                                  promises[s].rv())
-                                                 for s in aslist(
-                                                     inp["source"])]))
-                                    elif linkMerge == "merge_flattened":
-                                        jobobj[key] = (
-                                            MergeInputsFlattened(
-                                                [(shortname(s),
-                                                  promises[s].rv())
-                                                 for s in aslist(
-                                                      inp["source"])]))
-                                    else:
-                                        raise validate.ValidationException(
-                                            "Unsupported linkMerge '%s'" %
-                                            linkMerge)
+                                    jobobj[key] =\
+                                        _link_merge_source(promises, inp, inpSource)
                                 else:
-                                    inpSource = inp["source"]
                                     if isinstance(inpSource, MutableSequence):
                                         # It seems that an input source with a
                                         # '#' in the name will be returned as a
@@ -913,8 +947,10 @@ class CWLWorkflow(Job):
                                         jobobj[key] = DefaultWithSource(
                                             d, jobobj[key])
                                     else:
-                                        if jobobj[key][1][
-                                                jobobj[key][0]] is None:
+                                        if (isinstance(jobobj[key][1], tuple) and
+                                                jobobj[key][1][0][jobobj[key][0]]
+                                                is None) or (jobobj[key][1][
+                                                    jobobj[key][0]] is None):
                                             d = copy.copy(inp["default"])
                                             jobobj[key] = (
                                                 "default", {"default": d})
@@ -952,14 +988,12 @@ class CWLWorkflow(Job):
                             for s in aslist(inp.get("source", [])):
                                 if (isinstance(
                                         promises[s], (CWLJobWrapper, CWLGather)
-                                              ) and
-                                        not promises[s].hasFollowOn(wfjob)):
+                                ) and not promises[s].hasFollowOn(wfjob)):
                                     promises[s].addFollowOn(wfjob)
                                     connected = True
                                 if (not isinstance(
                                         promises[s], (CWLJobWrapper, CWLGather)
-                                                  ) and
-                                        not promises[s].hasChild(wfjob)):
+                                ) and not promises[s].hasChild(wfjob)):
                                     promises[s].addChild(wfjob)
                                     connected = True
                         if not connected:
@@ -986,21 +1020,7 @@ class CWLWorkflow(Job):
         for out in self.cwlwf.tool["outputs"]:
             key = shortname(out["id"])
             if out.get("linkMerge") or len(aslist(out["outputSource"])) > 1:
-                link_merge = out.get("linkMerge", "merge_nested")
-                if link_merge == "merge_nested":
-                    outobj[key] = (
-                        MergeInputsNested(
-                            [(shortname(s), promises[s].rv())
-                             for s in aslist(out["outputSource"])]))
-                elif link_merge == "merge_flattened":
-                    outobj[key] = (
-                        MergeInputsFlattened([
-                            (shortname(s), promises[s].rv())
-                            for s in aslist(out["source"])]))
-                else:
-                    raise validate.ValidationException(
-                        "Unsupported linkMerge '{}'".format(link_merge))
-
+                outobj[key] = _link_merge_source(promises, out, out["outputSource"])
             else:
                 # A CommentedSeq of length one still appears here rarely -
                 # not clear why from the CWL code. When it does, it breaks
@@ -1027,16 +1047,6 @@ def visitSteps(t, op):
             op(s.tool)
             visitSteps(s.embedded_tool, op)
 
-
-def cleanTempDirs(job):
-    """Remove temporarly created directories."""
-    if job is CWLJob and job._succeeded:  # Only CWLJobs have this attribute.
-        for tempDir in job.openTempDirs:
-            if os.path.exists(tempDir):
-                shutil.rmtree(tempDir)
-        job.openTempDirs = []
-
-
 def main(args=None, stdout=sys.stdout):
     """Main method for toil-cwl-runner."""
     cwllogger.removeHandler(defaultStreamHandler)
@@ -1056,7 +1066,6 @@ def main(args=None, stdout=sys.stdout):
     parser.add_argument("--basedir", type=str)
     parser.add_argument("--outdir", type=str, default=os.getcwd())
     parser.add_argument("--version", action='version', version=baseVersion)
-
     dockergroup = parser.add_mutually_exclusive_group()
     dockergroup.add_argument(
         "--user-space-docker-cmd",
@@ -1065,12 +1074,16 @@ def main(args=None, stdout=sys.stdout):
     dockergroup.add_argument(
         "--singularity", action="store_true", default=False,
         help="[experimental] Use Singularity runtime for running containers. "
-        "Requires Singularity v2.3.2+ and Linux with kernel version v3.18+ or "
+        "Requires Singularity v2.6.1+ and Linux with kernel version v3.18+ or "
         "with overlayfs support backported.")
     dockergroup.add_argument(
         "--no-container", action="store_true", help="Do not execute jobs in a "
         "Docker container, even when `DockerRequirement` "
         "is specified under `hints`.")
+    dockergroup.add_argument(
+        "--leave-container", action="store_false", default=True,
+        help="Do not delete Docker container used by jobs after they exit",
+        dest="rm_container")
 
     parser.add_argument(
         "--preserve-environment", type=str, nargs='+',
@@ -1111,10 +1124,58 @@ def main(args=None, stdout=sys.stdout):
     parser.add_argument(
         "--no-read-only", action="store_true", default=False,
         help="Do not set root directory in the container as read-only")
-
+    parser.add_argument(
+        "--strict-memory-limit", action="store_true", help="When running with "
+        "software containers and the Docker engine, pass either the "
+        "calculated memory allocation from ResourceRequirements or the "
+        "default of 1 gigabyte to Docker's --memory option.")    
+    parser.add_argument(
+        "--relax-path-checks", action="store_true",
+        default=False, help="Relax requirements on path names to permit "
+        "spaces and hash characters.", dest="relax_path_checks")
+    parser.add_argument("--default-container",
+                        help="Specify a default docker container that will be "
+                        "used if the workflow fails to specify one.")
     if args is None:
         args = sys.argv[1:]
 
+    provgroup = parser.add_argument_group("Options for recording provenance "
+                                          "information of the execution")
+    provgroup.add_argument("--provenance",
+                           help="Save provenance to specified folder as a "
+                           "Research Object that captures and aggregates "
+                           "workflow execution and data products.",
+                           type=Text)
+
+    provgroup.add_argument("--enable-user-provenance", default=False,
+                           action="store_true",
+                           help="Record user account info as part of provenance.",
+                           dest="user_provenance")
+    provgroup.add_argument("--disable-user-provenance", default=False,
+                           action="store_false",
+                           help="Do not record user account info in provenance.",
+                           dest="user_provenance")
+    provgroup.add_argument("--enable-host-provenance", default=False,
+                           action="store_true",
+                           help="Record host info as part of provenance.",
+                           dest="host_provenance")
+    provgroup.add_argument("--disable-host-provenance", default=False,
+                           action="store_false",
+                           help="Do not record host info in provenance.",
+                           dest="host_provenance")
+    provgroup.add_argument(
+        "--orcid", help="Record user ORCID identifier as part of "
+        "provenance, e.g. https://orcid.org/0000-0002-1825-0097 "
+        "or 0000-0002-1825-0097. Alternatively the environment variable "
+        "ORCID may be set.", dest="orcid", default=os.environ.get("ORCID", ''),
+        type=Text)
+    provgroup.add_argument(
+        "--full-name", help="Record full name of user as part of provenance, "
+        "e.g. Josiah Carberry. You may need to use shell quotes to preserve "
+        "spaces. Alternatively the environment variable CWL_FULL_NAME may "
+        "be set.", dest="cwl_full_name", default=os.environ.get("CWL_FULL_NAME", ''),
+        type=Text)
+
     # Problem: we want to keep our job store somewhere auto-generated based on
     # our options, unless overridden by... an option. So we will need to parse
     # options twice, because we need to feed the parser the job store.
@@ -1149,7 +1210,8 @@ def main(args=None, stdout=sys.stdout):
 
     if options.provisioner and not options.jobStore:
         raise NoSuchJobStoreException(
-            'Please specify a jobstore with the --jobStore option when specifying a provisioner.')
+            'Please specify a jobstore with the --jobStore option when '
+            'specifying a provisioner.')
 
     use_container = not options.no_container
 
@@ -1158,7 +1220,6 @@ def main(args=None, stdout=sys.stdout):
 
     outdir = os.path.abspath(options.outdir)
     tmp_outdir_prefix = os.path.abspath(options.tmp_outdir_prefix)
-    tmpdir_prefix = os.path.abspath(options.tmpdir_prefix)
 
     fileindex = {}
     existing = {}
@@ -1179,6 +1240,13 @@ def main(args=None, stdout=sys.stdout):
     runtime_context.rm_tmpdir = False
     loading_context = cwltool.context.LoadingContext(vars(options))
 
+    if options.provenance:
+        research_obj = cwltool.provenance.ResearchObject(
+            temp_prefix_ro=options.tmp_outdir_prefix, orcid=options.orcid,
+            full_name=options.cwl_full_name,
+            fsaccess=runtime_context.make_fs_access(''))
+        runtime_context.research_obj = research_obj
+
     with Toil(options) as toil:
         if options.restart:
             outobj = toil.restart()
@@ -1204,9 +1272,24 @@ def main(args=None, stdout=sys.stdout):
                 cwltool.main.load_job_order(
                     options, sys.stdin, loading_context.fetcher_constructor,
                     loading_context.overrides_list, tool_file_uri)
+
             loading_context, workflowobj, uri = cwltool.load_tool.fetch_document(uri, loading_context)
             loading_context, uri = cwltool.load_tool.resolve_and_validate_document(loading_context, workflowobj, uri)
             loading_context.overrides_list.extend(loading_context.metadata.get("cwltool:overrides", []))
+
+            document_loader = loading_context.loader
+            metadata = loading_context.metadata
+            processobj = document_loader.idx
+
+            if options.provenance and runtime_context.research_obj:
+                processobj['id'] = metadata['id']
+                processobj, metadata = loading_context.loader.resolve_ref(uri)
+                runtime_context.research_obj.packed_workflow(
+                    cwltool.main.print_pack(document_loader, processobj, uri, metadata))
+
+            loading_context.overrides_list.extend(
+                metadata.get("cwltool:overrides", []))
+
             try:
                 tool = cwltool.load_tool.make_tool(uri, loading_context)
             except cwltool.process.UnsupportedRequirement as err:
@@ -1258,7 +1341,6 @@ def main(args=None, stdout=sys.stdout):
 
             try:
                 runtime_context.use_container = use_container
-                runtime_context.tmpdir = os.path.realpath(tmpdir_prefix)
                 runtime_context.tmp_outdir_prefix = os.path.realpath(
                     tmp_outdir_prefix)
                 runtime_context.job_script_provider = job_script_provider
@@ -1271,11 +1353,16 @@ def main(args=None, stdout=sys.stdout):
                 return 33
 
             wf1.cwljob = initialized_job_order
-            if wf1 is CWLJob:  # Clean up temporary directories only created with CWLJobs.
-                wf1.addFollowOnFn(cleanTempDirs, wf1)
-            outobj = toil.start(wf1)
+
+            result = toil.start(wf1)
+            if isinstance(result, tuple):
+                outobj, metadata = result
+            else:
+                outobj = result
 
         outobj = resolve_indirect(outobj)
+        if isinstance(outobj, tuple):
+            outobj, metadata = outobj
 
         # Stage files. Specify destination bucket if specified in CLI
         # options. If destination bucket not passed in,
@@ -1289,6 +1376,29 @@ def main(args=None, stdout=sys.stdout):
             export=True,
             destBucket=options.destBucket)
 
+        if runtime_context.research_obj is not None:
+            runtime_context.research_obj.create_job(outobj, None, True)
+
+            def remove_at_id(doc):
+                if isinstance(doc, MutableMapping):
+                    for key in list(doc.keys()):
+                        if key == '@id':
+                            del doc[key]
+                        else:
+                            value = doc[key]
+                            if isinstance(value, MutableMapping):
+                                remove_at_id(value)
+                            if isinstance(value, MutableSequence):
+                                for entry in value:
+                                    if isinstance(value, MutableMapping):
+                                        remove_at_id(entry)
+            remove_at_id(outobj)
+            visit_class(outobj, ("File",), functools.partial(
+                add_sizes, runtime_context.make_fs_access('')))
+            prov_dependencies = cwltool.main.prov_deps(workflowobj, document_loader, uri)
+            runtime_context.research_obj.generate_snapshot(prov_dependencies)
+            runtime_context.research_obj.close(options.provenance)
+
         if not options.destBucket:
             visit_class(outobj, ("File",), functools.partial(
                 compute_checksums, cwltool.stdfsaccess.StdFsAccess("")))
