@@ -48,7 +48,8 @@ from toil.lib.expando import Expando
 from toil.lib.humanize import human2bytes
 
 from toil.common import Toil, addOptions, safeUnpickleFromStream
-from toil.fileStore import DeferredFunction
+from toil.deferred import DeferredFunction
+from toil.fileStores import FileID
 from toil.lib.bioio import (setLoggingFromOptions,
                             getTotalCpuTimeAndMemoryUsage,
                             getTotalCpuTime)
@@ -274,11 +275,11 @@ class Job(BaseJob):
             exhausting all their retries, remove any successor jobs and rerun this job to restart the
             subtree. Job must be a leaf vertex in the job graph when initially defined, see
             :func:`toil.job.Job.checkNewCheckpointsAreCutVertices`.
-        :type cores: int or string convertable by toil.lib.humanize.human2bytes to an int
-        :type disk: int or string convertable by toil.lib.humanize.human2bytes to an int
+        :type cores: int or string convertible by toil.lib.humanize.human2bytes to an int
+        :type disk: int or string convertible by toil.lib.humanize.human2bytes to an int
         :type preemptable: bool
-        :type cache: int or string convertable by toil.lib.humanize.human2bytes to an int
-        :type memory: int or string convertable by toil.lib.humanize.human2bytes to an int
+        :type cache: int or string convertible by toil.lib.humanize.human2bytes to an int
+        :type memory: int or string convertible by toil.lib.humanize.human2bytes to an int
         """
         requirements = {'memory': memory, 'cores': cores, 'disk': disk,
                         'preemptable': preemptable}
@@ -309,6 +310,7 @@ class Job(BaseJob):
         self._rvs = collections.defaultdict(list)
         self._promiseJobStore = None
         self._fileStore = None
+        self._defer = None
         self._tempDir = None
         self._succeeded = True
 
@@ -316,8 +318,9 @@ class Job(BaseJob):
         """
         Override this function to perform work and dynamically create successor jobs.
 
-        :param toil.fileStore.FileStore fileStore: Used to create local and globally
-               sharable temporary files and to send log messages to the leader process.
+        :param toil.fileStores.abstractFileStore.AbstractFileStore fileStore: Used to create local and 
+               globally sharable temporary files and to send log messages to the leader
+               process.
 
         :return: The return value of the function can be passed to other jobs by means of
                  :func:`toil.job.Job.rv`.
@@ -569,6 +572,7 @@ class Job(BaseJob):
         if self._promiseJobStore is None:
             raise RuntimeError('Trying to pass a promise from a promising job that is not a ' +
                                'predecessor of the job receiving the promise')
+        # TODO: can we guarantee self.jobStoreID is populated and so pass that here?
         with self._promiseJobStore.writeFileStream() as (fileHandle, jobStoreFileID):
             promise = UnfulfilledPromiseSentinel(str(self), False)
             pickle.dump(promise, fileHandle, pickle.HIGHEST_PROTOCOL)
@@ -718,9 +722,9 @@ class Job(BaseJob):
 
         :param dict kwargs: The keyword arguments to the function
         """
-        if self._fileStore is None:
+        if self._defer is None:
             raise Exception('A deferred function may only be registered with a job while that job is running.')
-        self._fileStore._registerDeferredFunction(DeferredFunction.create(function, *args, **kwargs))
+        self._defer(DeferredFunction.create(function, *args, **kwargs))
 
 
     ####################################################
@@ -954,7 +958,7 @@ class Job(BaseJob):
         :param fileHandle: An open, binary-mode file handle.
         :returns:
         """
-        
+
         def filter_main(module_name, class_name):
             try:
                 if module_name == '__main__':
@@ -976,13 +980,13 @@ class Job(BaseJob):
             # In Python 3 find_global isn't real and we are supposed to
             # subclass unpickler and override find_class. We can't just replace
             # it. But with cPickle in Pyhton 2 we can't subclass Unpickler.
-            
+
             class FilteredUnpickler(pickle.Unpickler):
                 def find_class(self, module, name):
                     return filter_main(module, name)
-                    
+
             unpickler = FilteredUnpickler(fileHandle)
-            
+
         runnable = unpickler.load()
         assert isinstance(runnable, BaseJob)
         runnable._config = config
@@ -1147,7 +1151,7 @@ class Job(BaseJob):
         # The pickled job is "run" as the command of the job, see worker
         # for the mechanism which unpickles the job and executes the Job.run
         # method.
-        with jobStore.writeFileStream(rootJobGraph.jobStoreID) as (fileHandle, fileStoreID):
+        with jobStore.writeFileStream(rootJobGraph.jobStoreID, cleanup=True) as (fileHandle, fileStoreID):
             pickle.dump(self, fileHandle, pickle.HIGHEST_PROTOCOL)
         # Note that getUserScript() may have been overridden. This is intended. If we used
         # self.userModule directly, we'd be getting a reference to job.py if the job was
@@ -1176,7 +1180,9 @@ class Job(BaseJob):
             # Make a job wrapper
             serviceJobGraph = serviceJob._createEmptyJobGraphForJob(jobStore, predecessorNumber=1)
 
-            # Create the start and terminate flags
+            # Create the start and terminate flags.
+            # We can't associate these with the job they belong to because
+            # that job hasn't necessarily been saved yet.
             serviceJobGraph.startJobStoreID = jobStore.getEmptyFileStoreID()
             serviceJobGraph.terminateJobStoreID = jobStore.getEmptyFileStoreID()
             serviceJobGraph.errorJobStoreID = jobStore.getEmptyFileStoreID()
@@ -1340,13 +1346,15 @@ class Job(BaseJob):
         # to the list of jobStoreFileIDs to delete
         if not self.checkpoint:
             for jobStoreFileID in Promise.filesToDelete:
-                fileStore.deleteGlobalFile(jobStoreFileID)
+                # Make sure to wrap the job store ID in a FileID object so the file store will accept it
+                # TODO: talk directly to the job sotre here instead.
+                fileStore.deleteGlobalFile(FileID(jobStoreFileID, 0))
         else:
             # Else copy them to the job wrapper to delete later
             jobGraph.checkpointFilesToDelete = list(Promise.filesToDelete)
         Promise.filesToDelete.clear()
         # Now indicate the asynchronous update of the job can happen
-        fileStore._updateJobWhenDone()
+        fileStore.startCommit(jobState=True)
         # Change dir back to cwd dir, if changed by job (this is a safety issue)
         if os.getcwd() != baseDir:
             os.chdir(baseDir)
@@ -1362,23 +1370,36 @@ class Job(BaseJob):
                 )
             )
 
-    def _runner(self, jobGraph, jobStore, fileStore):
+    def _runner(self, jobGraph, jobStore, fileStore, defer):
         """
         This method actually runs the job, and serialises the next jobs.
 
         :param class jobGraph: Instance of a jobGraph object
         :param class jobStore: Instance of the job store
-        :param toil.fileStore.FileStore fileStore: Instance of a Cached on uncached
-               filestore
+        :param toil.fileStores.abstractFileStore.AbstractFileStore fileStore: Instance of a cached
+               or uncached filestore
+        :param defer: Function yielded by open() context
+               manager of :class:`toil.DeferredFunctionManager`, which is called to
+               register deferred functions.
         :return:
         """
+
+        # Make deferred function registration available during run().
+        self._defer = defer
         # Make fileStore available as an attribute during run() ...
         self._fileStore = fileStore
         # ... but also pass it to run() as an argument for backwards compatibility.
         returnValues = self._run(jobGraph, fileStore)
+        # Clean up state changes made for run()
+        self._defer = None
+        self._fileStore = None
+
+
         # Serialize the new jobs defined by the run method to the jobStore
         self._serialiseExistingJob(jobGraph, jobStore, returnValues)
 
+        
+
     def _jobName(self):
         """
         :rtype : string, used as identifier of the job class in the stats report.
@@ -1485,9 +1506,10 @@ class JobFunctionWrappingJob(FunctionWrappingJob):
     add successor jobs for the function and perform all the functions the
     :class:`.Job` class provides.
 
-    To enable the job function to get access to the :class:`toil.fileStore.FileStore`
-    instance (see :func:`toil.job.Job.run`), it is made a variable of the wrapping job
-    called fileStore.
+    To enable the job function to get access to the
+    :class:`toil.fileStores.abstractFileStore.AbstractFileStore` instance (see
+    :func:`toil.job.Job.run`), it is made a variable of the wrapping job called
+    fileStore.
 
     To specify a job's resource requirements the following default keyword arguments
     can be specified:
