@@ -35,7 +35,7 @@ import six.moves.urllib.parse as urlparse
 from toil.lib.retry import retry_http
 
 from toil.common import safeUnpickleFromStream
-from toil.fileStore import FileID
+from toil.fileStores import FileID
 from toil.job import JobException
 from toil.lib.memoize import memoize
 from toil.lib.objects import abstractclassmethod
@@ -93,11 +93,11 @@ class NoSuchFileException(Exception):
             message = "File '%s' does not exist." % jobStoreFileID
         else:
             message = "File '%s' (%s) does not exist." % (customName, jobStoreFileID)
-        
+
         if extra:
             # Append extra data.
             message += " Extra info: " + " ".join((str(x) for x in extra))
-        
+
         super().__init__(message)
 
 
@@ -235,7 +235,6 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
         :rtype: list[AbstractJobStore]
         """
         jobStoreClassNames = (
-            "toil.jobStores.azureJobStore.AzureJobStore",
             "toil.jobStores.fileJobStore.FileJobStore",
             "toil.jobStores.googleJobStore.GoogleJobStore",
             "toil.jobStores.aws.jobStore.AWSJobStore",
@@ -279,9 +278,6 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
             - 's3' for objects in Amazon S3
                 e.g. s3://bucket/key
 
-            - 'wasb' for blobs in Azure Blob Storage
-                e.g. wasb://container/blob
-
             - 'file' for local files
                 e.g. file:///local/file/path
 
@@ -292,12 +288,12 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
                 e.g. gs://bucket/file
 
         :param str srcUrl: URL that points to a file or object in the storage mechanism of a
-                supported URL scheme e.g. a blob in an Azure Blob Storage container.
+                supported URL scheme e.g. a blob in an AWS s3 bucket.
 
         :param str sharedFileName: Optional name to assign to the imported file within the job store
 
         :return: The jobStoreFileId of the imported file or None if sharedFileName was given
-        :rtype: toil.fileStore.FileID or None
+        :rtype: toil.fileStores.FileID or None
         """
         # Note that the helper method _importFile is used to read from the source and write to
         # destination (which is the current job store in this case). To implement any
@@ -322,7 +318,7 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
         :param str sharedFileName: Optional name to assign to the imported file within the job store
 
         :return The jobStoreFileId of imported file or None if sharedFileName was given
-        :rtype: toil.fileStore.FileID or None
+        :rtype: toil.fileStores.FileID or None
         """
         if sharedFileName is None:
             with self.writeFileStream() as (writable, jobStoreFileID):
@@ -346,7 +342,7 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
 
         :param str jobStoreFileID: The id of the file in the job store that should be exported.
         :param str dstUrl: URL that points to a file or object in the storage mechanism of a
-                supported URL scheme e.g. a blob in an Azure Blob Storage container.
+                supported URL scheme e.g. a blob in an AWS s3 bucket.
         """
         dstUrl = urlparse.urlparse(dstUrl)
         otherCls = self._findJobStoreForUrl(dstUrl, export=True)
@@ -374,7 +370,7 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
         returns the size in bytes of the file at the given URL
 
         :param urlparse.ParseResult url: URL that points to a file or object in the storage
-               mechanism of a supported URL scheme e.g. a blob in an Azure Blob Storage container.
+               mechanism of a supported URL scheme e.g. a blob in an AWS s3 bucket.
         """
         raise NotImplementedError
 
@@ -387,7 +383,7 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
         Refer to :func:`~AbstractJobStore.importFile` documentation for currently supported URL schemes.
 
         :param urlparse.ParseResult url: URL that points to a file or object in the storage
-               mechanism of a supported URL scheme e.g. a blob in an Azure Blob Storage container.
+               mechanism of a supported URL scheme e.g. a blob in an AWS s3 bucket.
 
         :param writable: a writable stream
         """
@@ -402,7 +398,7 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
         Refer to AbstractJobStore.importFile documentation for currently supported URL schemes.
 
         :param urlparse.ParseResult url: URL that points to a file or object in the storage
-               mechanism of a supported URL scheme e.g. a blob in an Azure Blob Storage container.
+               mechanism of a supported URL scheme e.g. a blob in an AWS s3 bucket.
 
         :param readable: a readable stream
         """
@@ -468,7 +464,7 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
                 try:
                     return jobCache[jobId]
                 except KeyError:
-                    self.load(jobId)
+                    return self.load(jobId)
             else:
                 return self.load(jobId)
 
@@ -582,7 +578,7 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
                     return jobStoreFileID
 
                 # Make a new flag
-                newFlag = self.getEmptyFileStoreID()
+                newFlag = self.getEmptyFileStoreID(jobStoreID, cleanup=False)
 
                 # Load the jobGraph for the service and initialise the link
                 serviceJobGraph = getJob(jobStoreID)
@@ -767,7 +763,7 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
     def jobs(self):
         """
         Best effort attempt to return iterator on all jobs in the store. The iterator may not
-        return all jobs and may also contain orphaned jobs that have already finished succesfully
+        return all jobs and may also contain orphaned jobs that have already finished successfully
         and should not be rerun. To guarantee you get any and all jobs that can be run instead
         construct a more expensive ToilState object
 
@@ -783,17 +779,20 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
     ##########################################
 
     @abstractmethod
-    def writeFile(self, localFilePath, jobStoreID=None):
+    def writeFile(self, localFilePath, jobStoreID=None, cleanup=False):
         """
         Takes a file (as a path) and places it in this job store. Returns an ID that can be used
-        to retrieve the file at a later time.
+        to retrieve the file at a later time.  The file is written in a atomic manner.  It will
+        not appear in the jobStore until the write has successfully completed.
 
         :param str localFilePath: the path to the local file that will be uploaded to the job store.
 
-        :param jobStoreID: If specified the file will be associated with that job and when
-               jobStore.delete(job) is called all files written with the given job.jobStoreID will
-               be removed from the job store.
-        :type jobStoreID: str or None
+        :param str jobStoreID: the id of a job, or None. If specified, the may be associated
+               with that job in a job-store-specific way. This may influence the returned ID.
+
+        :param bool cleanup: Whether to attempt to delete the file when the job
+               whose jobStoreID was given as jobStoreID is deleted with
+               jobStore.delete(job). If jobStoreID was not given, does nothing.
 
         :raise ConcurrentFileModificationException: if the file was modified concurrently during
                an invocation of this method
@@ -810,16 +809,21 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
 
     @abstractmethod
     @contextmanager
-    def writeFileStream(self, jobStoreID=None):
+    def writeFileStream(self, jobStoreID=None, cleanup=False):
         """
         Similar to writeFile, but returns a context manager yielding a tuple of
         1) a file handle which can be written to and 2) the ID of the resulting
         file in the job store. The yielded file handle does not need to and
-        should not be closed explicitly.
+        should not be closed explicitly.  The file is written in a atomic manner.
+        It will not appear in the jobStore until the write has successfully
+        completed.
 
-        :param str jobStoreID: the id of a job, or None. If specified, the file will be associated
-               with that job and when when jobStore.delete(job) is called all files written with the
-               given job.jobStoreID will be removed from the job store.
+        :param str jobStoreID: the id of a job, or None. If specified, the may be associated
+               with that job in a job-store-specific way. This may influence the returned ID.
+
+        :param bool cleanup: Whether to attempt to delete the file when the job
+               whose jobStoreID was given as jobStoreID is deleted with
+               jobStore.delete(job). If jobStoreID was not given, does nothing.
 
         :raise ConcurrentFileModificationException: if the file was modified concurrently during
                an invocation of this method
@@ -835,14 +839,17 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
         raise NotImplementedError()
 
     @abstractmethod
-    def getEmptyFileStoreID(self, jobStoreID=None):
+    def getEmptyFileStoreID(self, jobStoreID=None, cleanup=False):
         """
         Creates an empty file in the job store and returns its ID.
         Call to fileExists(getEmptyFileStoreID(jobStoreID)) will return True.
 
-        :param str jobStoreID: the id of a job, or None. If specified, the file will be associated with
-               that job and when jobStore.delete(job) is called a best effort attempt is made to delete
-               all files written with the given job.jobStoreID
+        :param str jobStoreID: the id of a job, or None. If specified, the may be associated
+               with that job in a job-store-specific way. This may influence the returned ID.
+
+        :param bool cleanup: Whether to attempt to delete the file when the job
+               whose jobStoreID was given as jobStoreID is deleted with
+               jobStore.delete(job). If jobStoreID was not given, does nothing.
 
         :return: a jobStoreFileID that references the newly created file and can be used to reference the
                  file in the future.
@@ -858,7 +865,8 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
         the file written/updated. If the file in the job store is later
         modified via updateFile or updateFileStream, it is
         implementation-defined whether those writes will be visible at
-        localFilePath.
+        localFilePath.  The file is copied in an atomic manner.  It will not
+        appear in the local file system until the copy has completed.
 
         The file at the given local path may not be modified after this method returns!
 
@@ -904,6 +912,22 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
         """
         raise NotImplementedError()
 
+    @abstractmethod
+    def getFileSize(self, jobStoreFileID):
+        """
+        Get the size of the given file in bytes, or 0 if it does not exist when queried.
+
+        Note that job stores which encrypt files might return overestimates of
+        file sizes, since the encrypted file may have been padded to the
+        nearest block, augmented with an initialization vector, etc.
+
+        :param str jobStoreFileID: an ID referencing the file to be checked
+
+        :rtype: int
+        """
+        raise NotImplementedError()
+
+
     @abstractmethod
     def updateFile(self, jobStoreFileID, localFilePath):
         """
@@ -917,7 +941,6 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
 
         :raise ConcurrentFileModificationException: if the file was modified concurrently during
                an invocation of this method
-
         :raise NoSuchFileException: if the specified file does not exist
         """
         raise NotImplementedError()
@@ -952,7 +975,7 @@ class AbstractJobStore(with_metaclass(ABCMeta, object)):
     def writeSharedFileStream(self, sharedFileName, isProtected=None):
         """
         Returns a context manager yielding a writable file handle to the global file referenced
-        by the given name.
+        by the given name.  File will be created in an atomic manner.
 
         :param str sharedFileName: A file name matching AbstractJobStore.fileNameRegex, unique within
                this job store
