@@ -27,15 +27,15 @@ import tempfile
 import stat
 import errno
 import time
-import traceback
 try:
     import cPickle as pickle
 except ImportError:
     import pickle
 
 # toil dependencies
-from toil.fileStore import FileID
+from toil.fileStores import FileID
 from toil.lib.bioio import absSymPath
+from toil.lib.misc import mkdir_p, robust_rmtree, AtomicFileCreate, atomic_copy, atomic_copyobj
 from toil.jobStores.abstractJobStore import (AbstractJobStore,
                                              NoSuchJobException,
                                              NoSuchFileException,
@@ -52,25 +52,49 @@ class FileJobStore(AbstractJobStore):
     distributed batch systems, that file system must be shared by all worker nodes.
     """
 
-    # Valid chars for the creation of temporary directories
+    # Valid chars for the creation of temporary "spray" directories.
+    # Note that on case-insensitive filesystems we're twice as likely to use
+    # letter directories as number directories.
     validDirs = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
-    # Depth of temporary subdirectories created
-    levels = 2
+    validDirsSet = set(validDirs)
+
+    # What prefix should be on the per-job job directories, to distinguish them
+    # from the spray directories?
+    JOB_DIR_PREFIX = 'instance'
+
+    # What prefix do we put on the per-job-name directories we sort jobs into?
+    JOB_NAME_DIR_PREFIX = 'kind-'
 
     # 10Mb RAM chunks when reading/writing files
     BUFFER_SIZE = 10485760 # 10Mb
 
-    def __init__(self, path):
+    def __init__(self, path, fanOut=1000):
         """
         :param str path: Path to directory holding the job store
+        :param int fanOut: Number of items to have in a directory before making
+                           subdirectories
         """
         super(FileJobStore, self).__init__()
         self.jobStoreDir = absSymPath(path)
         logger.debug("Path to job store directory is '%s'.", self.jobStoreDir)
-        # Directory where temporary files go
-        self.tempFilesDir = os.path.join(self.jobStoreDir, 'tmp')
+
+        # Directory where actual job files go, and their job-associated temp files
+        self.jobsDir = os.path.join(self.jobStoreDir, 'jobs')
+        # Directory where stats files go
+        self.statsDir = os.path.join(self.jobStoreDir, 'stats')
+        # Directory where non-job-associated files for the file store go
+        self.filesDir = os.path.join(self.jobStoreDir, 'files/no-job')
+        # Directory where job-associated files for the file store go.
+        # Each per-job directory in here will have separate directories for
+        # files to clean up and files to not clean up when the job is deleted.
+        self.jobFilesDir = os.path.join(self.jobStoreDir, 'files/for-job')
+        # Directory where shared files go
+        self.sharedFilesDir = os.path.join(self.jobStoreDir, 'files/shared')
+
+        self.fanOut = fanOut
+
         self.linkImports = None
-        
+
     def __repr__(self):
         return 'FileJobStore({})'.format(self.jobStoreDir)
 
@@ -82,7 +106,11 @@ class FileJobStore(AbstractJobStore):
                 raise JobStoreExistsException(self.jobStoreDir)
             else:
                 raise
-        os.mkdir(self.tempFilesDir)
+        mkdir_p(self.jobsDir)
+        mkdir_p(self.statsDir)
+        mkdir_p(self.filesDir)
+        mkdir_p(self.jobFilesDir)
+        mkdir_p(self.sharedFilesDir)
         self.linkImports = config.linkImports
         super(FileJobStore, self).initialize(config)
 
@@ -91,34 +119,9 @@ class FileJobStore(AbstractJobStore):
             raise NoSuchJobStoreException(self.jobStoreDir)
         super(FileJobStore, self).resume()
 
-    def robust_rmtree(self, path, max_retries=3):
-        """Robustly tries to delete paths.
-
-        Retries several times (with increasing delays) if an OSError
-        occurs.  If the final attempt fails, the Exception is propagated
-        to the caller.
-
-        Borrowing patterns from:
-        https://github.com/hashdist/hashdist
-        """
-        
-        delay = 1
-        for _ in range(max_retries):
-            try:
-                shutil.rmtree(path)
-                break
-            except OSError:
-                logger.debug('Unable to remove path: {}.  Retrying in {} seconds.'.format(path, delay))
-                time.sleep(delay)
-                delay *= 2
-
-        if os.path.exists(path):
-            # Final attempt, pass any Exceptions up to caller.
-            shutil.rmtree(path)
-        
     def destroy(self):
         if os.path.exists(self.jobStoreDir):
-            self.robust_rmtree(self.jobStoreDir)
+            robust_rmtree(self.jobStoreDir)
 
     ##########################################
     # The following methods deal with creating/loading/updating/writing/checking for the
@@ -126,16 +129,23 @@ class FileJobStore(AbstractJobStore):
     ##########################################
 
     def create(self, jobNode):
-        # The absolute path to the job directory.
-        absJobDir = tempfile.mkdtemp(prefix="job", dir=self._getTempSharedDir())
-        # Sub directory to put temporary files associated with the job in
-        os.mkdir(os.path.join(absJobDir, "g"))
-        # Make the job
-        job = JobGraph.fromJobNode(jobNode, jobStoreID=self._getRelativePath(absJobDir),
+        # Get the job's name. We want to group jobs with the same name together.
+        # This will be e.g. the function name for wrapped-function jobs.
+        # Make sure to render it filename-safe
+        usefulFilename = self._makeStringFilenameSafe(jobNode.jobName)
+
+        # Make a unique temp directory under a directory for this job name,
+        # possibly sprayed across multiple levels of subdirectories.
+        absJobDir = tempfile.mkdtemp(prefix=self.JOB_DIR_PREFIX,
+                                     dir=self._getArbitraryJobsDirForName(usefulFilename))
+        # Make the job to save
+        job = JobGraph.fromJobNode(jobNode, jobStoreID=self._getJobIdFromDir(absJobDir),
                                    tryCount=self._defaultTryCount())
         if hasattr(self, "_batchedJobGraphs") and self._batchedJobGraphs is not None:
+            # Save it later
             self._batchedJobGraphs.append(job)
         else:
+            # Save it now
             self.update(job)
         return job
 
@@ -149,15 +159,15 @@ class FileJobStore(AbstractJobStore):
 
     def waitForExists(self, jobStoreID, maxTries=35, sleepTime=1):
         """Spin-wait and block for a file to appear before returning False if it does not.
-        
+
         The total max wait time is maxTries * sleepTime. The current default is
         tuned to match Linux NFS defaults where the client's cache of the directory
         listing on the server is supposed to become coherent within 30 sec.
         Delayes beyond that would probably indicate a pathologically slow file system
         that just should not be used for the jobStore.
-        
+
         The warning will be sent to the log only on the first retry.
-        
+
         In practice, the need for retries happens rarely, but it does happen
         over the course of large workflows with a jobStore on a busy NFS."""
         for iTry in range(1,maxTries+1):
@@ -174,17 +184,17 @@ class FileJobStore(AbstractJobStore):
 
     def exists(self, jobStoreID):
         return os.path.exists(self._getJobFileName(jobStoreID))
-    
+
     def getPublicUrl(self, jobStoreFileID):
         self._checkJobStoreFileID(jobStoreFileID)
-        jobStorePath = self._getAbsPath(jobStoreFileID)
+        jobStorePath = self._getFilePathFromId(jobStoreFileID)
         if os.path.exists(jobStorePath):
             return 'file:' + jobStorePath
         else:
             raise NoSuchFileException(jobStoreFileID)
 
     def getSharedPublicUrl(self, sharedFileName):
-        jobStorePath = self.jobStoreDir + '/' + sharedFileName
+        jobStorePath = os.path.join(self.sharedFilesDir, sharedFileName)
         if os.path.exists(jobStorePath):
             return 'file:' + jobStorePath
         else:
@@ -218,15 +228,24 @@ class FileJobStore(AbstractJobStore):
         # The jobStoreID is the relative path to the directory containing the job,
         # removing this directory deletes the job.
         if self.exists(jobStoreID):
-            self.robust_rmtree(self._getAbsPath(jobStoreID))
+            # Remove the job-associated files in need of cleanup, which may or
+            # may not live under the job's directory.
+            robust_rmtree(self._getJobFilesCleanupDir(jobStoreID))
+            # Remove the job's directory itself.
+            robust_rmtree(self._getJobDirFromId(jobStoreID))
 
     def jobs(self):
-        # Walk through list of temporary directories searching for jobs
-        for tempDir in self._tempDirectories():
+        # Walk through list of temporary directories searching for jobs.
+        # Jobs are files that start with 'job'.
+        # Note that this also catches jobWhatever.new which exists if an update
+        # is in progress.
+        for tempDir in self._jobDirectories():
             for i in os.listdir(tempDir):
-                if i.startswith('job'):
+                logger.warning('Job Dir: %s' % i)
+                if i.startswith(self.JOB_DIR_PREFIX):
+                    # This is a job instance directory
                     try:
-                        yield self.load(self._getRelativePath(os.path.join(tempDir, i)))
+                        yield self.load(self._getJobIdFromDir(os.path.join(tempDir, i)))
                     except NoSuchJobException:
                         # An orphaned job may leave an empty or incomplete job file which we can safely ignore
                         pass
@@ -250,17 +269,17 @@ class FileJobStore(AbstractJobStore):
         if self.linkImports:
             os.symlink(os.path.realpath(srcPath), destPath)
         else:
-            shutil.copyfile(srcPath, destPath)
+            atomic_copy(srcPath, destPath)
 
     def _importFile(self, otherCls, url, sharedFileName=None, hardlink=False):
         if issubclass(otherCls, FileJobStore):
             if sharedFileName is None:
-                absPath = self._getUniqueName(url.path)  # use this to get a valid path to write to in job store
+                absPath = self._getUniqueFilePath(url.path)  # use this to get a valid path to write to in job store
                 with self.optionalHardCopy(hardlink):
                     self._copyOrLink(url, absPath)
                 # TODO: os.stat(absPath).st_size consistently gives values lower than
                 # getDirSizeRecursively()
-                return FileID(self._getRelativePath(absPath), os.stat(absPath).st_size)
+                return FileID(self._getFileIdFromPath(absPath), os.stat(absPath).st_size)
             else:
                 self._requireValidSharedFileName(sharedFileName)
                 path = self._getSharedFilePath(sharedFileName)
@@ -273,7 +292,7 @@ class FileJobStore(AbstractJobStore):
 
     def _exportFile(self, otherCls, jobStoreFileID, url):
         if issubclass(otherCls, FileJobStore):
-            shutil.copyfile(self._getAbsPath(jobStoreFileID), self._extractPathFromUrl(url))
+            atomic_copy(self._getFilePathFromId(jobStoreFileID), self._extractPathFromUrl(url))
         else:
             super(FileJobStore, self)._exportFile(otherCls, jobStoreFileID, url)
 
@@ -304,8 +323,7 @@ class FileJobStore(AbstractJobStore):
         :param object readable: An open file object to read from.
         """
         # we use a ~10Mb buffer to improve speed
-        with open(cls._extractPathFromUrl(url), 'wb') as writable:
-            shutil.copyfileobj(readable, writable, length=cls.BUFFER_SIZE)
+        atomic_copyobj(readable, cls._extractPathFromUrl(url), length=cls.BUFFER_SIZE)
 
     @staticmethod
     def _extractPathFromUrl(url):
@@ -320,61 +338,65 @@ class FileJobStore(AbstractJobStore):
     def _supportsUrl(cls, url, export=False):
         return url.scheme.lower() == 'file'
 
-    def _getUserCodeFunctionName(self):
+    def _makeStringFilenameSafe(self, arbitraryString):
         """
-        Get the name of the function 4 levels up the stack (above this
-        function, our caller, and whatever Toil code delegated to the JobStore
-        implementation). Returns a string usable in a filename, and returns a
-        placeholder string if the function name is unsuitable or can't be
-        gotten.
-        """
-        
-        # Record the name of the job/function writing the file in the file name
-        try:
-            # It ought to be fourth-to-last on the stack, above us, the write
-            # function, and the FileStore or context manager. Probably.
-            sourceFunctionName = traceback.extract_stack()[-4][2]
-        except:
-            sourceFunctionName = "UNKNOWNJOB"
-            # make sure the function name fetched has no spaces or oddities
-        if not re.match("^[A-Za-z0-9_-]*$", sourceFunctionName):
-            sourceFunctionName = "ODDLYNAMEDJOB"
-            
-        return sourceFunctionName
+        Given an arbitrary string, produce a filename-safe though not
+        necessarily unique string based on it.
 
-    def writeFile(self, localFilePath, jobStoreID=None):
-        absPath = self._getUniqueName(localFilePath, jobStoreID, self._getUserCodeFunctionName())
-        relPath = self._getRelativePath(absPath)
-        shutil.copyfile(localFilePath, absPath)
+        The input string may be discarded altogether and replaced with any
+        other nonempty filename-safe string.
+
+        :param str arbitraryString: An arbitrary string
+
+        :return: A filename-safe string
+        """
+
+        # We will fill this in with the filename-safe parts we find.
+        parts = []
+
+        for substring in re.findall("[A-Za-z0-9._-]+", arbitraryString):
+            # Collect all the matching substrings
+            parts.append(substring)
+
+        if len(parts) == 0:
+            parts.append("UNPRINTABLE")
+
+        # Glue it all together
+        return '_'.join(parts)
+
+    def writeFile(self, localFilePath, jobStoreID=None, cleanup=False):
+        absPath = self._getUniqueFilePath(localFilePath, jobStoreID, cleanup)
+        relPath = self._getFileIdFromPath(absPath)
+        atomic_copy(localFilePath, absPath)
         return relPath
 
     @contextmanager
-    def writeFileStream(self, jobStoreID=None):
-        absPath = self._getUniqueName('stream', jobStoreID, self._getUserCodeFunctionName())
-        relPath = self._getRelativePath(absPath)
+    def writeFileStream(self, jobStoreID=None, cleanup=False):
+        absPath = self._getUniqueFilePath('stream', jobStoreID, cleanup)
+        relPath = self._getFileIdFromPath(absPath)
         with open(absPath, 'wb') as f:
             # Don't yield while holding an open file descriptor to the temp
             # file. That can result in temp files still being open when we try
             # to clean ourselves up, somehow, for certain workloads.
             yield f, relPath
 
-    def getEmptyFileStoreID(self, jobStoreID=None):
-        with self.writeFileStream(jobStoreID) as (fileHandle, jobStoreFileID):
+    def getEmptyFileStoreID(self, jobStoreID=None, cleanup=False):
+        with self.writeFileStream(jobStoreID, cleanup) as (fileHandle, jobStoreFileID):
             return jobStoreFileID
 
     def updateFile(self, jobStoreFileID, localFilePath):
         self._checkJobStoreFileID(jobStoreFileID)
-        jobStoreFilePath = self._getAbsPath(jobStoreFileID)
+        jobStoreFilePath = self._getFilePathFromId(jobStoreFileID)
 
         if os.path.samefile(jobStoreFilePath, localFilePath):
             # The files are already the same file. We can't copy on eover the other.
             return
 
-        shutil.copyfile(localFilePath, jobStoreFilePath)
+        atomic_copy(localFilePath, jobStoreFilePath)
 
     def readFile(self, jobStoreFileID, localFilePath, symlink=False):
         self._checkJobStoreFileID(jobStoreFileID)
-        jobStoreFilePath = self._getAbsPath(jobStoreFileID)
+        jobStoreFilePath = self._getFilePathFromId(jobStoreFileID)
         localDirPath = os.path.dirname(localFilePath)
 
         if not symlink and os.path.islink(localFilePath):
@@ -439,15 +461,22 @@ class FileJobStore(AbstractJobStore):
 
         # If we get here, neither a symlink nor a hardlink will work.
         # Make a complete copy.
-        shutil.copyfile(jobStoreFilePath, localFilePath)
+        atomic_copy(jobStoreFilePath, localFilePath)
 
     def deleteFile(self, jobStoreFileID):
         if not self.fileExists(jobStoreFileID):
             return
-        os.remove(self._getAbsPath(jobStoreFileID))
+        os.remove(self._getFilePathFromId(jobStoreFileID))
 
     def fileExists(self, jobStoreFileID):
-        absPath = self._getAbsPath(jobStoreFileID)
+        absPath = self._getFilePathFromId(jobStoreFileID)
+
+        if (not absPath.startswith(self.jobsDir) and
+            not absPath.startswith(self.filesDir) and
+            not absPath.startswith(self.jobFilesDir)):
+            # Don't even look for it, it is out of bounds.
+            raise NoSuchFileException(jobStoreFileID)
+
         try:
             st = os.stat(absPath)
         except os.error:
@@ -456,19 +485,35 @@ class FileJobStore(AbstractJobStore):
             raise NoSuchFileException(jobStoreFileID)
         return True
 
+    def getFileSize(self, jobStoreFileID):
+        # Duplicate a bunch of fileExists to save on stat calls
+        absPath = self._getFilePathFromId(jobStoreFileID)
+
+        if (not absPath.startswith(self.jobsDir) and
+            not absPath.startswith(self.filesDir) and
+            not absPath.startswith(self.jobFilesDir)):
+            # Don't even look for it, it is out of bounds.
+            raise NoSuchFileException(jobStoreFileID)
+
+        try:
+            st = os.stat(absPath)
+        except os.error:
+            return 0
+        return st.st_size
+
     @contextmanager
     def updateFileStream(self, jobStoreFileID):
         self._checkJobStoreFileID(jobStoreFileID)
         # File objects are context managers (CM) so we could simply return what open returns.
         # However, it is better to wrap it in another CM so as to prevent users from accessing
         # the file object directly, without a with statement.
-        with open(self._getAbsPath(jobStoreFileID), 'wb') as f:
+        with open(self._getFilePathFromId(jobStoreFileID), 'wb') as f:
             yield f
 
     @contextmanager
     def readFileStream(self, jobStoreFileID):
         self._checkJobStoreFileID(jobStoreFileID)
-        with open(self._getAbsPath(jobStoreFileID), 'rb') as f:
+        with open(self._getFilePathFromId(jobStoreFileID), 'rb') as f:
             yield f
 
     ##########################################
@@ -477,20 +522,21 @@ class FileJobStore(AbstractJobStore):
     ##########################################
 
     def _getSharedFilePath(self, sharedFileName):
-        return os.path.join(self.jobStoreDir, sharedFileName)
+        return os.path.join(self.sharedFilesDir, sharedFileName)
 
     @contextmanager
     def writeSharedFileStream(self, sharedFileName, isProtected=None):
         # the isProtected parameter has no effect on the fileStore
-        assert self._validateSharedFileName( sharedFileName )
-        with open(self._getSharedFilePath(sharedFileName), 'wb') as f:
-            yield f
+        self._requireValidSharedFileName(sharedFileName)
+        with AtomicFileCreate(self._getSharedFilePath(sharedFileName)) as tmpSharedFilePath:
+            with open(tmpSharedFilePath, 'wb') as f:
+                yield f
 
     @contextmanager
     def readSharedFileStream(self, sharedFileName):
-        assert self._validateSharedFileName( sharedFileName )
+        self._requireValidSharedFileName(sharedFileName)
         try:
-            with open(os.path.join(self.jobStoreDir, sharedFileName), 'rb') as f:
+            with open(self._getSharedFilePath(sharedFileName), 'rb') as f:
                 yield f
         except IOError as e:
             if e.errno == errno.ENOENT:
@@ -499,8 +545,8 @@ class FileJobStore(AbstractJobStore):
                 raise
 
     def writeStatsAndLogging(self, statsAndLoggingString):
-        # Temporary files are placed in the set of temporary files/directories
-        fd, tempStatsFile = tempfile.mkstemp(prefix="stats", suffix=".new", dir=self._getTempSharedDir())
+        # Temporary files are placed in the stats directory tree
+        fd, tempStatsFile = tempfile.mkstemp(prefix="stats", suffix=".new", dir=self._getArbitraryStatsDir())
         writeFormat = 'w' if isinstance(statsAndLoggingString, str) else 'wb'
         with open(tempStatsFile, writeFormat) as f:
             f.write(statsAndLoggingString)
@@ -509,41 +555,41 @@ class FileJobStore(AbstractJobStore):
 
     def readStatsAndLogging(self, callback, readAll=False):
         numberOfFilesProcessed = 0
-        for tempDir in self._tempDirectories():
+        for tempDir in self._statsDirectories():
             for tempFile in os.listdir(tempDir):
                 if tempFile.startswith('stats'):
                     absTempFile = os.path.join(tempDir, tempFile)
-                    if readAll or not tempFile.endswith('.new'):
-                        with open(absTempFile, 'rb') as fH:
-                            callback(fH)
-                        numberOfFilesProcessed += 1
-                        newName = tempFile.rsplit('.', 1)[0] + '.new'
-                        newAbsTempFile = os.path.join(tempDir, newName)
-                        # Mark this item as read
-                        os.rename(absTempFile, newAbsTempFile)
+                    if os.path.isfile(absTempFile):
+                        if readAll or not tempFile.endswith('.new'):
+                            with open(absTempFile, 'rb') as fH:
+                                callback(fH)
+                            numberOfFilesProcessed += 1
+                            newName = tempFile.rsplit('.', 1)[0] + '.new'
+                            newAbsTempFile = os.path.join(tempDir, newName)
+                            # Mark this item as read
+                            os.rename(absTempFile, newAbsTempFile)
         return numberOfFilesProcessed
 
     ##########################################
     # Private methods
     ##########################################
 
-    def _getAbsPath(self, relativePath):
+    def _getJobDirFromId(self, jobStoreID):
         """
-        :param str relativePath: path relative to self.tempFilesDir.
-        :rtype : string, string is the absolute path to a file path relative
-        to the self.tempFilesDir.
-        """
-        return os.path.join(self.tempFilesDir, relativePath)
 
-    def _getRelativePath(self, absPath):
-        """
-        absPath  is the absolute path to a file in the store,.
-
-        :rtype : string, string is the path to the absPath file relative to the
-        self.tempFilesDir
+        Find the directory for a job, which holds its job file.
 
+        :param str jobStoreID: ID of a job, which is a relative to self.jobsDir.
+        :rtype : string, string is the absolute path to a job directory inside self.jobsDir.
         """
-        return absPath[len(self.tempFilesDir)+1:]
+        return os.path.join(self.jobsDir, jobStoreID)
+
+    def _getJobIdFromDir(self, absPath):
+        """
+        :param str absPath: The absolute path to a job directory under self.jobsDir which represents a job.
+        :rtype : string, string is the job ID, which is a path relative to self.jobsDir
+        """
+        return absPath[len(self.jobsDir)+1:]
 
     def _getJobFileName(self, jobStoreID):
         """
@@ -552,91 +598,294 @@ class FileJobStore(AbstractJobStore):
 
         :rtype: str
         """
-        return os.path.join(self._getAbsPath(jobStoreID), "job")
+        return os.path.join(self._getJobDirFromId(jobStoreID), "job")
+
+    def _getJobFilesDir(self, jobStoreID):
+        """
+        Return the path to the directory that should hold files made by the
+        given job that should survive its deletion.
+
+        This directory will only be created if files are to be put in it.
+
+        :rtype : string, string is the absolute path to the job's files
+                 directory
+        """
+
+        return os.path.join(self.jobFilesDir, jobStoreID)
+
+    def _getJobFilesCleanupDir(self, jobStoreID):
+        """
+        Return the path to the directory that should hold files made by the
+        given job that will be deleted when the job is deleted.
+
+        This directory will only be created if files are to be put in it.
+
+        It may or may not be a subdirectory of the job's own directory.
+
+        :rtype : string, string is the absolute path to the job's cleanup
+                 files directory
+        """
+
+        return os.path.join(self.jobFilesDir, jobStoreID, "cleanup")
 
     def _checkJobStoreId(self, jobStoreID):
         """
-        Raises a NoSuchJobException if the jobStoreID does not exist.
+        Raises a NoSuchJobException if the job with ID jobStoreID does not exist.
         """
         if not self.waitForExists(jobStoreID,30):
             raise NoSuchJobException(jobStoreID)
 
+    def _getFilePathFromId(self, jobStoreFileID):
+        """
+        :param str jobStoreFileID: The ID of a file
+
+        :rtype : string, string is the absolute path that that file should
+                 appear at on disk, under either self.jobsDir if it is to be
+                 cleaned up with a job, or self.filesDir otherwise.
+        """
+
+        # We just make the file IDs paths under the job store overall.
+        absPath = os.path.join(self.jobStoreDir, jobStoreFileID)
+
+        # Don't validate here, we are called by the validation logic
+
+        return absPath
+
+    def _getFileIdFromPath(self, absPath):
+        """
+        :param str absPath: The absolute path of a file.
+
+        :rtype : string, string is the file ID.
+        """
+
+        return absPath[len(self.jobStoreDir)+1:]
+
     def _checkJobStoreFileID(self, jobStoreFileID):
         """
-        :raise NoSuchFileException: if the jobStoreFileID does not exist or is not a file
+        :raise NoSuchFileException: if the file with ID jobStoreFileID does
+                                    not exist or is not a file
         """
         if not self.fileExists(jobStoreFileID):
             raise NoSuchFileException(jobStoreFileID)
 
-    def _getTempSharedDir(self):
+    def _getArbitraryJobsDirForName(self, jobNameSlug):
         """
-        Gets a temporary directory in the hierarchy of directories in self.tempFilesDir.
-        This directory may contain multiple shared jobs/files.
+        Gets a temporary directory in a multi-level hierarchy in self.jobsDir.
+        The directory is not unique and may already have other jobs' directories in it.
+        We organize them at the top level by job name, to be user-inspectable.
+
+        We make sure to prepend a string so that job names can't collide with
+        spray directory names.
+
+        :param str jobNameSlug: A partial filename derived from the job name.
+                                Used as the first level of the directory hierarchy.
 
         :rtype : string, path to temporary directory in which to place files/directories.
+
+
         """
-        tempDir = self.tempFilesDir
-        for i in range(self.levels):
+
+
+        if len(os.listdir(self.jobsDir)) > self.fanOut:
+            # Make sure that we don't over-fill the root with too many unique job names.
+            # Go in a subdirectory tree, and then go by job name and make another tree.
+            return self._getDynamicSprayDir(os.path.join(self._getDynamicSprayDir(self.jobsDir),
+                                                                                  self.JOB_NAME_DIR_PREFIX + jobNameSlug))
+        else:
+            # Just go in the root
+            return self._getDynamicSprayDir(os.path.join(self.jobsDir, self.JOB_NAME_DIR_PREFIX + jobNameSlug))
+
+    def _getArbitraryStatsDir(self):
+        """
+        Gets a temporary directory in a multi-level hierarchy in self.statsDir.
+        The directory is not unique and may already have other stats files in it.
+
+        :rtype : string, path to temporary directory in which to place files/directories.
+
+
+        """
+
+        return self._getDynamicSprayDir(self.statsDir)
+
+    def _getArbitraryFilesDir(self):
+        """
+        Gets a temporary directory in a multi-level hierarchy in self.filesDir.
+        The directory is not unique and may already have other user files in it.
+
+        :rtype : string, path to temporary directory in which to place files/directories.
+
+
+        """
+
+        return self._getDynamicSprayDir(self.filesDir)
+
+    def _getDynamicSprayDir(self, root):
+        """
+        Gets a temporary directory in a possibly multi-level hierarchy of
+        directories under the given root.
+
+        Each time a directory in the hierarchy starts to fill up, additional
+        hierarchy levels are created under it, and we randomly "spray" further
+        files and directories across them.
+
+        We can't actually enforce that we never go over our internal limit for
+        files in a directory, because any number of calls to this function can
+        be happening simultaneously. But we can enforce that, once too many
+        files are visible on disk, only subdirectories will be created.
+
+        The returned directory will exist, and may contain other data already.
+
+        The caller may not create any files or directories in the returned
+        directory with single-character names that are in self.validDirs.
+
+        :param str root : directory to put the hierarchy under, which will
+                          fill first.
+
+        :rtype : string, path to temporary directory in which to place
+                 files/directories.
+        """
+        tempDir = root
+
+        # Make sure the root exists
+        mkdir_p(tempDir)
+
+        while len(os.listdir(tempDir)) >= self.fanOut:
+            # We need to use a layer of directories under here to avoid over-packing the directory
             tempDir = os.path.join(tempDir, random.choice(self.validDirs))
-            if not os.path.exists(tempDir):
-                try:
-                    os.mkdir(tempDir)
-                except os.error:
-                    if not os.path.exists(tempDir): # In the case that a collision occurs and
-                        # it is created while we wait then we ignore
-                        raise
+            mkdir_p(tempDir)
+
+        # When we get here, we found a sufficiently empty directory
         return tempDir
 
-    def _tempDirectories(self):
+    def _walkDynamicSprayDir(self, root):
         """
-        :rtype : an iterator to the temporary directories containing jobs/stats files
-        in the hierarchy of directories in self.tempFilesDir
-        """
-        def _dirs(path, levels):
-            if levels > 0:
-                for subPath in os.listdir(path):
-                    for i in _dirs(os.path.join(path, subPath), levels-1):
-                        yield i
-            else:
-                yield path
-        for tempDir in _dirs(self.tempFilesDir, self.levels):
-            yield tempDir
+        Walks over a directory tree filled in by _getDynamicSprayDir.
 
-    def _getUniqueName(self, fileName, jobStoreID=None, sourceFunctionName="x"):
+        Yields each directory _getDynamicSprayDir has ever returned, and no
+        directories it has not returned (besides the root).
+
+        If the caller looks in the directory, they must ignore subdirectories
+        with single-character names in self.validDirs.
+
+        :param str root : directory the hierarchy was put under
+
+        :rtype : an iterator over directories
+        """
+
+        # Always yield the root.
+        # The caller is responsible for dealing with it if it has gone away.
+        yield root
+
+        children = []
+
+        try:
+            # Look for children
+            children = os.listdir(root)
+        except:
+            # Don't care if they are gone
+            pass
+
+        for child in children:
+            # Go over all the children
+            if child not in self.validDirsSet:
+                # Only look at our reserved names we use for fan-out
+                continue
+
+            # We made this directory, so go look in it
+            childPath = os.path.join(root, child)
+
+            # Recurse
+            for item in self._walkDynamicSprayDir(childPath):
+                yield item
+
+    def _jobDirectories(self):
+        """
+        :rtype : an iterator to the temporary directories containing job
+                 files. They may also contain directories containing more
+                 job files.
+        """
+
+        # Walking the job directories is more complicated.
+        # We have one layer of spray (which is sometimes bypassed, but that's OK), then a job name, then another layer.
+        # We can tell the job name directories from the spray directories because they start with self.JOB_NAME_DIR_PREFIX.
+        # We never look at the directories containing the job name directories,
+        # so they aren't mistaken for the leaf-level per-job job directories.
+
+        for jobHoldingDir in self._walkDynamicSprayDir(self.jobsDir):
+            # For every directory in the first spray, look at children
+            children = []
+
+            try:
+                children = os.listdir(jobHoldingDir)
+            except:
+                pass
+
+            for jobNameDir in children:
+                if not jobNameDir.startswith(self.JOB_NAME_DIR_PREFIX):
+                    continue
+
+                # Now we have only the directories that are named after jobs. Look inside them.
+                for inner in self._walkDynamicSprayDir(os.path.join(jobHoldingDir, jobNameDir)):
+                    yield inner
+
+
+    def _statsDirectories(self):
+        """
+        :rtype : an iterator to the temporary directories containing stats
+                 files. They may also contain directories containing more
+                 stats files.
+        """
+
+        return self._walkDynamicSprayDir(self.statsDir)
+
+    def _getUniqueFilePath(self, fileName, jobStoreID=None, cleanup=False):
         """
         Create unique file name within a jobStore directory or tmp directory.
 
         :param fileName: A file name, which can be a full path as only the
         basename will be used.
-        :param jobStoreID: If given, the path returned will be in the jobStore directory.
-        Otherwise, the tmp directory will be used.
-        :param sourceFunctionName: This name is the name of the function that
-            generated this file.  Defaults to x if that name was not a normal
-            name.  Used for tracking files.
+        :param jobStoreID: If given, the path returned will be in a directory including the job's ID as part of its path.
+        :param bool cleanup: If True and jobStoreID is set, the path will be in
+            a place such that it gets deleted when the job is deleted.
         :return: The full path with a unique file name.
         """
-        fd, absPath = self._getTempFile(jobStoreID)
-        os.close(fd)
-        os.unlink(absPath)
-        # remove the .tmp extension and add the file name
-        (noExt,ext) = os.path.splitext(absPath)
-        uniquePath = noExt + '-' + sourceFunctionName + '-' + os.path.basename(fileName)
-        if os.path.exists(absPath):
-            return absPath  # give up, just return temp name to avoid conflicts
+
+        # Give the file a unique directory that either will be cleaned up with a job or won't.
+        directory = self._getFileDirectory(jobStoreID, cleanup)
+        # And then a path under it
+        uniquePath = os.path.join(directory, os.path.basename(fileName))
+        # No need to check if it exists already; it is in a unique directory.
         return uniquePath
 
-    def _getTempFile(self, jobStoreID=None):
+    def _getFileDirectory(self, jobStoreID=None, cleanup=False):
         """
-        :rtype : file-descriptor, string, string is the absolute path to a temporary file within
-        the given job's (referenced by jobStoreID's) temporary file directory. The file-descriptor
-        is integer pointing to open operating system file handle. Should be closed using os.close()
-        after writing some material to the file.
+        Get a new empty directory path for a file to be stored at.
+
+
+        :param str jobStoreID: If the jobStoreID is not None, the file wil
+               be associated with the job with that ID.
+
+        :param bool cleanup: If cleanup is also True, this directory
+               will be cleaned up when the job is deleted.
+
+        :rtype :string, string is the absolute path to a directory to put the file in.
         """
         if jobStoreID != None:
-            # Make a temporary file within the job's directory
+            # Make a temporary file within the job's files directory
+
+            # Make sure the job is legit
             self._checkJobStoreId(jobStoreID)
-            return tempfile.mkstemp(suffix=".tmp",
-                                dir=os.path.join(self._getAbsPath(jobStoreID), "g"))
+            # Find where all its created files should live, depending on if
+            # they need to go away when the job is deleted or not.
+            jobFilesDir = self._getJobFilesDir(jobStoreID) if not cleanup \
+                else self._getJobFilesCleanupDir(jobStoreID)
+
+            # Lazily create the parent directory.
+            # We don't want our tree filled with confusingly empty directories.
+            mkdir_p(jobFilesDir)
+
+            # Then make a temp directory inside it
+            return tempfile.mkdtemp(prefix='file-', dir=jobFilesDir)
         else:
-            # Make a temporary file within the temporary file structure
-            return tempfile.mkstemp(prefix="tmp", suffix=".tmp", dir=self._getTempSharedDir())
+            # Make a temporary file within the non-job-associated files hierarchy
+            return tempfile.mkdtemp(prefix='file-', dir=self._getArbitraryFilesDir())
