@@ -25,7 +25,9 @@ import logging
 import time
 import os
 from toil import pickle
+from toil.lib.misc import AtomicFileCreate
 from toil.lib.retry import retry
+from toil.lib.compatibility import compat_bytes
 from google.cloud import storage, exceptions
 from google.api_core.exceptions import GoogleAPICallError, InternalServerError, ServiceUnavailable
 from toil.lib.misc import truncExpBackoff
@@ -109,18 +111,18 @@ class GoogleJobStore(AbstractJobStore):
         self.readStatsBaseID = self.statsReadPrefix+self.statsBaseID
 
         self.sseKey = None
-        
+
         # Determine if we have an override environment variable for our credentials.
         # We don't pull out the filename; we just see if a name is there.
         self.credentialsFromEnvironment = bool(os.getenv('GOOGLE_APPLICATION_CREDENTIALS', False))
-        
+
         if self.credentialsFromEnvironment and not os.path.exists(os.getenv('GOOGLE_APPLICATION_CREDENTIALS')):
             # If the file is missing, complain.
             # This variable holds a file name and not any sensitive data itself.
             log.warning("File '%s' from GOOGLE_APPLICATION_CREDENTIALS is unavailable! "
                         "We may not be able to authenticate!",
                         os.getenv('GOOGLE_APPLICATION_CREDENTIALS'))
-        
+
         if not self.credentialsFromEnvironment and os.path.exists(self.nodeServiceAccountJson):
             # load credentials from a particular file on GCE nodes if an override path is not set
             self.storageClient = storage.Client.from_service_account_json(self.nodeServiceAccountJson)
@@ -141,7 +143,7 @@ class GoogleJobStore(AbstractJobStore):
         # set up sever side encryption after we set up config in super
         if self.config.sseKey is not None:
             with open(self.config.sseKey) as f:
-                self.sseKey = bytes(f.read())
+                self.sseKey = compat_bytes(f.read())
                 assert len(self.sseKey) == 32
 
     @googleRetry
@@ -188,11 +190,11 @@ class GoogleJobStore(AbstractJobStore):
 
     @googleRetry
     def exists(self, jobStoreID):
-        return self.bucket.blob(bytes(jobStoreID), encryption_key=self.sseKey).exists()
+        return self.bucket.blob(compat_bytes(jobStoreID), encryption_key=self.sseKey).exists()
 
     @googleRetry
     def getPublicUrl(self, fileName):
-        blob = self.bucket.get_blob(bytes(fileName), encryption_key=self.sseKey)
+        blob = self.bucket.get_blob(compat_bytes(fileName), encryption_key=self.sseKey)
         if blob is None:
             raise NoSuchFileException(fileName)
         return blob.generate_signed_url(self.publicUrlExpiration)
@@ -215,22 +217,22 @@ class GoogleJobStore(AbstractJobStore):
         self._delete(jobStoreID)
 
         # best effort delete associated files
-        for blob in self.bucket.list_blobs(prefix=bytes(jobStoreID)):
+        for blob in self.bucket.list_blobs(prefix=compat_bytes(jobStoreID)):
             self._delete(blob.name)
-            
+
     def getEnv(self):
         """
         Return a dict of environment variables to send out to the workers
         so they can load the job store.
         """
-        
+
         env = {}
-        
+
         if self.credentialsFromEnvironment:
             # Send along the environment variable that points to the credentials file.
             # It must be available in the same place on all nodes.
             env['GOOGLE_APPLICATION_CREDENTIALS'] = os.getenv('GOOGLE_APPLICATION_CREDENTIALS')
-            
+
         return env
 
     @googleRetry
@@ -240,20 +242,20 @@ class GoogleJobStore(AbstractJobStore):
             if len(jobStoreID) == 39:  # 'job' + uuid length
                 yield self.load(jobStoreID)
 
-    def writeFile(self, localFilePath, jobStoreID=None):
-        fileID = self._newID(isFile=True, jobStoreID=jobStoreID)
+    def writeFile(self, localFilePath, jobStoreID=None, cleanup=False):
+        fileID = self._newID(isFile=True, jobStoreID=jobStoreID if cleanup else None)
         with open(localFilePath) as f:
             self._writeFile(fileID, f)
         return fileID
 
     @contextmanager
-    def writeFileStream(self, jobStoreID=None):
-        fileID = self._newID(isFile=True, jobStoreID=jobStoreID)
+    def writeFileStream(self, jobStoreID=None, cleanup=False):
+        fileID = self._newID(isFile=True, jobStoreID=jobStoreID if cleanup else None)
         with self._uploadStream(fileID, update=False) as writable:
             yield writable, fileID
 
-    def getEmptyFileStoreID(self, jobStoreID=None):
-        fileID = self._newID(isFile=True, jobStoreID=jobStoreID)
+    def getEmptyFileStoreID(self, jobStoreID=None, cleanup=False):
+        fileID = self._newID(isFile=True, jobStoreID=jobStoreID if cleanup else None)
         self._writeFile(fileID, StringIO(""))
         return fileID
 
@@ -263,9 +265,10 @@ class GoogleJobStore(AbstractJobStore):
         # checking for JobStoreID existence
         if not self.fileExists(jobStoreFileID):
             raise NoSuchFileException(jobStoreFileID)
-        with open(localFilePath, 'w') as writeable:
-            blob = self.bucket.get_blob(bytes(jobStoreFileID), encryption_key=self.sseKey)
-            blob.download_to_file(writeable)
+        with AtomicFileCreate(localFilePath) as tmpPath:
+            with open(tmpPath, 'w') as writeable:
+                blob = self.bucket.get_blob(compat_bytes(jobStoreFileID), encryption_key=self.sseKey)
+                blob.download_to_file(writeable)
 
     @contextmanager
     def readFileStream(self, jobStoreFileID):
@@ -277,7 +280,13 @@ class GoogleJobStore(AbstractJobStore):
 
     @googleRetry
     def fileExists(self, jobStoreFileID):
-        return self.bucket.blob(bytes(jobStoreFileID), encryption_key=self.sseKey).exists()
+        return self.bucket.blob(compat_bytes(jobStoreFileID), encryption_key=self.sseKey).exists()
+
+    @googleRetry
+    def getFileSize(self, jobStoreFileID):
+        if not self.fileExists(jobStoreFileID):
+            return 0
+        return self.bucket.get_blob(compat_bytes(jobStoreFileID), encryption_key=self.sseKey).size
 
     def updateFile(self, jobStoreFileID, localFilePath):
         with open(localFilePath) as f:
@@ -323,7 +332,7 @@ class GoogleJobStore(AbstractJobStore):
 
         storageClient = storage.Client()
         bucket = storageClient.get_bucket(bucketName)
-        blob = bucket.blob(bytes(fileName))
+        blob = bucket.blob(compat_bytes(fileName))
 
         if exists:
             if not blob.exists():
@@ -365,7 +374,7 @@ class GoogleJobStore(AbstractJobStore):
         while True:
             filesReadThisLoop = 0
             # prefix seems broken
-            for blob in self.bucket.list_blobs(prefix=bytes(prefix)):
+            for blob in self.bucket.list_blobs(prefix=compat_bytes(prefix)):
                 try:
                     with self.readSharedFileStream(blob.name) as readable:
                         log.debug("Reading stats file: %s", blob.name)
@@ -376,7 +385,7 @@ class GoogleJobStore(AbstractJobStore):
                         # rereading it
                         newID = self.readStatsBaseID + blob.name[len(self.statsBaseID):]
                         # NOTE: just copies then deletes old.
-                        self.bucket.rename_blob(blob, bytes(newID))
+                        self.bucket.rename_blob(blob, compat_bytes(newID))
                 except NoSuchFileException:
                     log.debug("Stats file not found: %s", blob.name)
             if readAll:
@@ -413,7 +422,7 @@ class GoogleJobStore(AbstractJobStore):
     @googleRetry
     def _delete(self, jobStoreFileID):
         if self.fileExists(jobStoreFileID):
-            self.bucket.get_blob(bytes(jobStoreFileID)).delete()
+            self.bucket.get_blob(compat_bytes(jobStoreFileID)).delete()
         # remember, this is supposed to be idempotent, so we don't do anything
         # if the file doesn't exist
 
@@ -426,14 +435,14 @@ class GoogleJobStore(AbstractJobStore):
         :return: contents of the job file
         :rtype: string
         """
-        job = self.bucket.get_blob(bytes(jobStoreID), encryption_key=self.sseKey)
+        job = self.bucket.get_blob(compat_bytes(jobStoreID), encryption_key=self.sseKey)
         if job is None:
             raise NoSuchJobException(jobStoreID)
         return job.download_as_string()
 
     @googleRetry
     def _writeFile(self, jobStoreID, fileObj, update=False, encrypt=True):
-        blob = self.bucket.blob(bytes(jobStoreID), encryption_key=self.sseKey if encrypt else None)
+        blob = self.bucket.blob(compat_bytes(jobStoreID), encryption_key=self.sseKey if encrypt else None)
         if not update:
             # TODO: should probably raise a special exception and be added to all jobStores
             assert not blob.exists()
@@ -464,7 +473,7 @@ class GoogleJobStore(AbstractJobStore):
         :return: an instance of WritablePipe.
         :rtype: :class:`~toil.jobStores.utils.writablePipe`
         """
-        blob = self.bucket.blob(bytes(fileName), encryption_key=self.sseKey if encrypt else None)
+        blob = self.bucket.blob(compat_bytes(fileName), encryption_key=self.sseKey if encrypt else None)
 
         class UploadPipe(WritablePipe):
             def readFrom(self, readable):
@@ -489,7 +498,7 @@ class GoogleJobStore(AbstractJobStore):
         :return: an instance of ReadablePipe.
         :rtype: :class:`~toil.jobStores.utils.ReadablePipe`
         """
-        blob = self.bucket.get_blob(bytes(fileName), encryption_key=self.sseKey if encrypt else None)
+        blob = self.bucket.get_blob(compat_bytes(fileName), encryption_key=self.sseKey if encrypt else None)
         if blob is None:
             raise NoSuchFileException(fileName)
 
