@@ -1,33 +1,22 @@
 from __future__ import absolute_import
-from past.builtins import map
-import logging
-import os
+
 try:
     # In Python 3 we have this quote
     from shlex import quote
 except ImportError:
     # But in 2.7 we have this deprecated one
     from pipes import quote
-from toil import subprocess
+
 import docker
 import base64
-import time
 import requests
-
+import logging
+import os
+import sys
 from docker.errors import create_api_error_from_http_exception
 from docker.errors import ContainerError
 from docker.errors import ImageNotFound
-from docker.errors import APIError
 from docker.errors import NotFound
-from docker.errors import DockerException
-from docker.utils.types import LogConfig
-from docker.api.container import ContainerApiMixin
-
-from toil.lib.retry import retry
-from docker import client
-from pwd import getpwuid
-
-from toil.test import timeLimit
 
 logger = logging.getLogger(__name__)
 
@@ -35,169 +24,21 @@ FORGO = 0
 STOP = 1
 RM = 2
 
-def dockerPredicate(e):
-    """
-    Used to ensure Docker exceptions are retried if appropriate
-
-    :param e: Exception
-    :return: True if e retriable, else False
-    """
-    if not isinstance(e, subprocess.CalledProcessError):
-        return False
-    if e.returncode == 125:
-        return True
 
 def dockerCheckOutput(*args, **kwargs):
-    """
-    Deprecated.  Runs subprocessDockerCall() using 'subprocess.check_output()'.
+    raise RuntimeError("dockerCheckOutput() using subprocess.check_output() has been removed, "
+                       "please switch to apiDockerCall().")
 
-    Provided for backwards compatibility with a previous implementation that
-    used 'subprocess.check_output()'.  This has since been supplanted and
-    apiDockerCall() is recommended.
-    """
-    logger.warn("WARNING: dockerCheckOutput() using subprocess.check_output() "
-                "is deprecated, please switch to apiDockerCall().")
-    return subprocessDockerCall(*args, checkOutput=True, **kwargs)
 
 def dockerCall(*args, **kwargs):
-    """
-    Deprecated.  Runs subprocessDockerCall() using 'subprocess.check_output()'.
+    raise RuntimeError("dockerCall() using subprocess.check_output() has been removed, "
+                       "please switch to apiDockerCall().")
 
-    Provided for backwards compatibility with a previous implementation that
-    used 'subprocess.check_call()'.  This has since been supplanted and
-    apiDockerCall() is recommended.
-    """
-    logger.warn("WARNING: dockerCall() using subprocess.check_output() "
-                "is deprecated, please switch to apiDockerCall().")
-    return subprocessDockerCall(*args, checkOutput=False, **kwargs)
 
-def subprocessDockerCall(job,
-                         tool,
-                         parameters=None,
-                         workDir=None,
-                         dockerParameters=None,
-                         checkOutput=True,
-                         outfile=None,
-                         errfile=None,
-                         defer=None):
-    """
-    Deprecated.  Calls Docker using subprocess.check_output().
+def subprocessDockerCall(*args, **kwargs):
+    raise RuntimeError("subprocessDockerCall() has been removed, "
+                       "please switch to apiDockerCall().")
 
-    Assumes `docker` is on the PATH.  Uses Toil's defer functionality to ensure
-    containers are shutdown even in case of job or pipeline failure
-
-    Example of using dockerCall in toil to index a FASTA file with SAMtools:
-        def toil_job(job):
-            work_dir = job.fileStore.getLocalTempDir()
-            path = job.fileStore.readGlobalFile(ref_id, os.path.join(
-                                                          work_dir, 'ref.fasta')
-            parameters = ['faidx', path]
-            dockerCall(job, tool='quay.io/ucgc_cgl/samtools:latest',
-                       work_dir=work_dir, parameters=parameters)
-
-    :param toil.Job.job job: The Job instance for the calling function.
-    :param str tool: Name of the Docker image to be used
-                     (e.g. quay.io/ucsc_cgl/samtools).
-    :param list[str] parameters: Command line arguments to be passed.
-           If list of lists: list[list[str]], then treat as successive commands
-           chained with pipe.
-    :param str workDir: Directory to mount into the container via `-v`.
-                        Destination convention is '/data'.
-    :param list[str] dockerParameters: Parameters to pass to Docker.
-             Default parameters are `--rm`, `--log-driver none`, and the
-             mountpoint `-v work_dir:/data` where /data is the destination
-             convention.
-             These defaults are removed if docker_parmaters is passed,
-             so be sure to pass them if they are desired.
-    :param file outfile: Pipe output of Docker call to file handle
-    :param file errfile: Pipe standard error of Docker call to file handle
-    :param int defer: What action should be taken on the container upon job
-                      completion?
-           FORGO (0) will leave the container untouched.
-           STOP (1) will attempt to stop the container with `docker stop`
-           (useful for debugging).
-           RM (2) will stop the container and then forcefully remove it from the
-           system using `docker rm -f`. This is the default behavior if defer is
-           set to None.
-    """
-    if parameters is None:
-        parameters = []
-    if workDir is None:
-        workDir = os.getcwd()
-
-    # Setup the outgoing subprocess call for docker
-    baseDockerCall = ['docker', 'run']
-    if dockerParameters:
-        baseDockerCall += dockerParameters
-    else:
-        baseDockerCall += ['--rm', '--log-driver', 'none', '-v',
-                           os.path.abspath(workDir) + ':/data']
-
-    # Ensure the user has passed a valid value for defer
-    assert defer in (None, FORGO, STOP, RM)
-
-    # Get container name which is needed for _dockerKill
-    try:
-        if any('--name' in x for x in baseDockerCall):
-            if any('--name=' in x for x in baseDockerCall):
-                containerName = [x.split('=')[1]
-                                 for x in baseDockerCall if '--name' in x][0]
-            else:
-                containerName = baseDockerCall[
-                    baseDockerCall.index('--name') + 1]
-        else:
-            containerName = getContainerName(job)
-            baseDockerCall.extend(['--name', containerName])
-    except ValueError:
-        containerName = getContainerName(job)
-        baseDockerCall.extend(['--name', containerName])
-    except IndexError:
-        raise RuntimeError(
-            "Couldn't parse Docker's `--name=` option, check parameters: " +
-            str(dockerParameters))
-
-    # Defer the container on-exit action
-    if '--rm' in baseDockerCall and defer is None:
-        defer = RM
-    if '--rm' in baseDockerCall and defer is not RM:
-        logger.warn('--rm being passed to docker call but defer not set to '
-                    'dockerCall.RM, defer set to: ' + str(defer))
-    job.defer(_dockerKill, containerName, action=defer)
-    # Defer the permission fixing function which will run after this job.
-    # We call this explicitly later on in this function,
-    # but we defer it as well to handle unexpected job failure.
-    job.defer(_fixPermissions, tool, workDir)
-
-    # Make subprocess call
-
-    # If parameters is list of lists, treat each list as separate command and chain with pipes
-    if len(parameters) > 0 and type(parameters[0]) is list:
-        # When piping, all arguments now get merged into a single string to bash -c.
-        # We try to support spaces in paths by wrapping them all in quotes first.
-        chain_params = [' '.join(p) for p in [list(map(quote, q)) for q in parameters]]
-        # Use bash's set -eo pipefail to detect and abort on a failure in any command in the chain
-        call = baseDockerCall + ['--entrypoint', '/bin/bash',  tool, '-c',
-                                 'set -eo pipefail && {}'.format(' | '.join(chain_params))]
-    else:
-        call = baseDockerCall + [tool] + parameters
-    logger.info("Calling docker with " + repr(call))
-
-    params = {}
-    if outfile:
-        params['stdout'] = outfile
-    if errfile:
-        params['stderr'] = errfile
-    if checkOutput:
-        callMethod = subprocess.check_output
-    else:
-        callMethod = subprocess.check_call
-
-    for attempt in retry(predicate=dockerPredicate):
-        with attempt:
-            out = callMethod(call, **params)
-
-    _fixPermissions(tool=tool, workDir=workDir)
-    return out
 
 def apiDockerCall(job,
                   image,
@@ -216,6 +57,7 @@ def apiDockerCall(job,
                   stdout=None,
                   stderr=False,
                   streamfile=None,
+                  timeout=365 * 24 * 60 * 60,
                   **kwargs):
     """
     A toil wrapper for the python docker API.
@@ -287,6 +129,11 @@ def apiDockerCall(job,
                      (some other user : some other user group).
     :param environment: Allows one to set environment variables inside of the
                         container, such as:
+    :param int timeout: Use the given timeout in seconds for interactions with
+                        the Docker daemon. Note that the underlying docker module is
+                        not always able to abort ongoing reads and writes in order
+                        to respect the timeout. Defaults to 1 year (i.e. wait
+                        essentially indefinitely).
     :param kwargs: Additional keyword arguments supplied to the docker API's
                    run command.  The list is 75 keywords total, for examples
                    and full documentation see:
@@ -351,18 +198,18 @@ def apiDockerCall(job,
     assert deferParam in (None, FORGO, STOP, RM), \
         'Please provide a valid value for deferParam.'
 
-    client = docker.from_env(version='auto')
+    client = docker.from_env(version='auto', timeout=timeout)
 
     if deferParam == STOP:
-        job.defer(dockerStop, containerName, client)
+        job.defer(dockerStop, containerName)
 
     if deferParam == FORGO:
         remove = False
     elif deferParam == RM:
         remove = True
-        job.defer(dockerKill, containerName, client)
+        job.defer(dockerKill, containerName)
     elif remove is True:
-        job.defer(dockerKill, containerName, client)
+        job.defer(dockerKill, containerName)
 
     if auto_remove is None:
         auto_remove = remove
@@ -389,7 +236,6 @@ def apiDockerCall(job,
                                         user=user,
                                         environment=environment,
                                         **kwargs)
-
             return out
         else:
             if (stdout or stderr) and log_config is None:
@@ -433,8 +279,7 @@ def apiDockerCall(job,
             return container
             
     except ContainerError:
-        logger.error("Docker had non-zero exit.  Check your command: " + \
-                      repr(command))
+        logger.error("Docker had non-zero exit.  Check your command: " + repr(command))
         raise
     except ImageNotFound:
         logger.error("Docker image not found.")
@@ -442,16 +287,21 @@ def apiDockerCall(job,
     except requests.exceptions.HTTPError as e:
         logger.error("The server returned an error.")
         raise create_api_error_from_http_exception(e)
-    except:
-        raise
 
-def dockerKill(container_name, client, gentleKill=False):
+
+def dockerKill(container_name, gentleKill=False, timeout=365 * 24 * 60 * 60):
     """
     Immediately kills a container.  Equivalent to "docker kill":
     https://docs.docker.com/engine/reference/commandline/kill/
     :param container_name: Name of the container being killed.
     :param client: The docker API client object to call.
+    :param int timeout: Use the given timeout in seconds for interactions with
+                        the Docker daemon. Note that the underlying docker module is
+                        not always able to abort ongoing reads and writes in order
+                        to respect the timeout. Defaults to 1 year (i.e. wait
+                        essentially indefinitely).
     """
+    client = docker.from_env(version='auto', timeout=timeout)
     try:
         this_container = client.containers.get(container_name)
         while this_container.status == 'running':
@@ -468,23 +318,30 @@ def dockerKill(container_name, client, gentleKill=False):
                       container_name)
         raise create_api_error_from_http_exception(e)
 
-def dockerStop(container_name, client):
+
+def dockerStop(container_name):
     """
     Gracefully kills a container.  Equivalent to "docker stop":
     https://docs.docker.com/engine/reference/commandline/stop/
     :param container_name: Name of the container being stopped.
     :param client: The docker API client object to call.
     """
-    dockerKill(container_name, client, gentleKill=True)
+    dockerKill(container_name, gentleKill=True)
 
-def containerIsRunning(container_name):
+
+def containerIsRunning(container_name, timeout=365 * 24 * 60 * 60):
     """
     Checks whether the container is running or not.
     :param container_name: Name of the container being checked.
     :returns: True if status is 'running', False if status is anything else,
     and None if the container does not exist.
+    :param int timeout: Use the given timeout in seconds for interactions with
+                        the Docker daemon. Note that the underlying docker module is
+                        not always able to abort ongoing reads and writes in order
+                        to respect the timeout. Defaults to 1 year (i.e. wait
+                        essentially indefinitely).
     """
-    client = docker.from_env(version='auto')
+    client = docker.from_env(version='auto', timeout=timeout)
     try:
         this_container = client.containers.get(container_name)
         if this_container.status == 'running':
@@ -499,81 +356,9 @@ def containerIsRunning(container_name):
                       container_name)
         raise create_api_error_from_http_exception(e)
 
+
 def getContainerName(job):
     """Create a random string including the job name, and return it."""
     return '--'.join([str(job),
                       base64.b64encode(os.urandom(9), b'-_').decode('utf-8')])\
                       .replace("'", '').replace('"', '').replace('_', '')
-
-
-def _dockerKill(containerName, action):
-    """
-    Deprecated.  Kills the specified container.
-    :param str containerName: The name of the container created by docker_call
-    :param int action: What action should be taken on the container?
-    """
-    running = containerIsRunning(containerName)
-    if running is None:
-        # This means that the container doesn't exist.  We will see this if the
-        # container was run with --rm and has already exited before this call.
-        logger.debug('The container with name "%s" appears to have already been '
-                    'removed.  Nothing to '
-                  'do.', containerName)
-    else:
-        if action in (None, FORGO):
-            logger.debug('The container with name %s continues to exist as we '
-                        'were asked to forgo a '
-                      'post-job action on it.', containerName)
-        else:
-            logger.debug('The container with name %s exists. Running '
-                        'user-specified defer functions.',
-                         containerName)
-            if running and action >= STOP:
-                logger.debug('Stopping container "%s".', containerName)
-                for attempt in retry(predicate=dockerPredicate):
-                    with attempt:
-                        subprocess.check_call(['docker', 'stop', containerName])
-            else:
-                logger.debug('The container "%s" was not found to be running.',
-                            containerName)
-            if action >= RM:
-                # If the container was run with --rm, then stop will most likely
-                # remove the container.  We first check if it is running then
-                # remove it.
-                running = containerIsRunning(containerName)
-                if running is not None:
-                    logger.debug('Removing container "%s".', containerName)
-                    for attempt in retry(predicate=dockerPredicate):
-                        with attempt:
-                            subprocess.check_call(['docker', 'rm', '-f',
-                                                   containerName])
-                else:
-                    logger.debug('Container "%s" was not found on the system.'
-                                'Nothing to remove.',
-                                 containerName)
-
-def _fixPermissions(tool, workDir):
-    """
-    Deprecated.
-
-    Fix permission of a mounted Docker directory by reusing the tool to change
-    ownership.  Docker natively runs as a root inside the container, and files
-    written to the mounted directory are implicitly owned by root.
-    :param list baseDockerCall: Docker run parameters
-    :param str tool: Name of tool
-    :param str workDir: Path of work directory to recursively chown
-    """
-    if os.geteuid() == 0:
-        # we're running as root so this chown is redundant
-        return
-
-    baseDockerCall = ['docker', 'run', '--log-driver=none',
-                      '-v', os.path.abspath(workDir) + ':/data', '--rm',
-                      '--entrypoint=chown']
-    stat = os.stat(workDir)
-    command = baseDockerCall + [tool] + ['-R', '{}:{}'.format(stat.st_uid,
-                                                              stat.st_gid),
-                                         '/data']
-    for attempt in retry(predicate=dockerPredicate):
-        with attempt:
-            subprocess.check_call(command)
