@@ -20,7 +20,8 @@ import datetime
 import logging
 import textwrap
 
-logger = logging.getLogger( __name__ )
+logger = logging.getLogger(__name__)
+dirname = os.path.dirname(__file__)
 
 
 EC2Regions = {'us-west-1': 'US West (N. California)',
@@ -107,6 +108,7 @@ def parseStorage(storageData):
         "EBS only"
         "1 x 410"
         "8 x 1.9 NVMe SSD"
+        "900 GB NVMe SSD"
 
     :param str storageData: EC2 JSON storage param string.
     :return: Two floats representing: (# of disks), and (disk_capacity in GiB of each disk).
@@ -117,6 +119,8 @@ def parseStorage(storageData):
         specs = storageData.strip().split()
         if isNumber(specs[0]) and specs[1] == 'x' and isNumber(specs[2]):
             return float(specs[0].replace(',', '')), float(specs[2].replace(',', ''))
+        elif isNumber(specs[0]) and specs[1] == 'GB' and specs[2] == 'NVMe' and specs[3] == 'SSD':
+            return 1, float(specs[0].replace(',', ''))
         else:
             raise RuntimeError('EC2 JSON format has likely changed.  Error parsing disk specs.')
 
@@ -139,9 +143,8 @@ def parseMemory(memAttribute):
         raise RuntimeError('EC2 JSON format has likely changed.  Error parsing memory.')
 
 
-def fetchEC2InstanceDict(regionNickname=None):
-    """
-    Fetches EC2 instances types by region programmatically using the AWS pricing API.
+def fetchEC2Index(filename):
+    """Downloads and writes the AWS Billing JSON to a file using the AWS pricing API.
 
     See: https://aws.amazon.com/blogs/aws/new-aws-price-list-api/
 
@@ -149,60 +152,55 @@ def fetchEC2InstanceDict(regionNickname=None):
              aws instance name (example: 't2.micro'), and the value is an
              InstanceType object representing that aws instance name.
     """
-    ec2Source = 'https://pricing.us-east-1.amazonaws.com/offers/v1.0/aws/AmazonEC2/current/index.json'
-    if regionNickname is None:
-        regionNickname = 'us-west-2'
-    region = EC2Regions[regionNickname]  # JSON uses verbose region names as keys
+    print('Downloading ~1Gb AWS billing file to parse for information.\n')
 
-    ec2InstanceList = []
-
-    # summon the API to grab the latest instance types/prices/specs
-    response = requests.get(ec2Source)
+    response = requests.get('https://pricing.us-east-1.amazonaws.com/offers/v1.0/aws/AmazonEC2/current/index.json')
     if response.ok:
-        ec2InstanceList = parseEC2Json2List(jsontext=response.text, region=region)
-
-    if ec2InstanceList:
-        return dict((_.name, _) for _ in ec2InstanceList)
+        with open(filename, 'w') as f:
+            f.write(str(json.dumps(json.loads(response.text), indent=4)))
+            print('Download completed successfully!\n')
     else:
-        from toil.lib import generatedEC2Lists as defaultEC2
-        return dict((_.name, _) for _ in defaultEC2.ec2InstancesByRegion[regionNickname])
+        raise RuntimeError('Error: ' + str(response) + ' :: ' + str(response.text))
 
 
-def parseEC2Json2List(jsontext, region):
+def fetchEC2InstanceDict(awsBillingJson, region):
     """
     Takes a JSON and returns a list of InstanceType objects representing EC2 instance params.
 
-    :param jsontext:
     :param region:
     :return:
     """
-    currentList = json.loads(jsontext)
     ec2InstanceList = []
-    for k, v in iteritems(currentList["products"]):
-        if "location" in v["attributes"] and v["attributes"]["location"] == region:
-            # 3 tenant types: 'Host' (always $0.00; just a template?)
-            #                 'Dedicated' (toil does not support; these are pricier)
-            #                 'Shared' (AWS default and what toil uses)
-            if "tenancy" in v["attributes"] and v["attributes"]["tenancy"] == "Shared":
-                if v["attributes"]["operatingSystem"] == "Linux":
-                    # The same instance can appear with multiple "operation"
-                    # values; "RunInstances" is normal, and
-                    # "RunInstances:<code>" is e.g. Linux with MS SQL Server
-                    # installed.
-                    if v["attributes"]["operation"] == "RunInstances":
-                        disks, disk_capacity = parseStorage(v["attributes"]["storage"])
-                        memory = parseMemory(v["attributes"]["memory"])
-                        instance = InstanceType(name=v["attributes"]["instanceType"],
-                                                cores=v["attributes"]["vcpu"],
-                                                memory=memory,
-                                                disks=disks,
-                                                disk_capacity=disk_capacity)
-                        if instance not in ec2InstanceList:
-                            ec2InstanceList.append(instance)
-                        else:
-                            raise RuntimeError('EC2 JSON format has likely changed.  '
-                                               'Duplicate instance {} found.'.format(instance))
-    return ec2InstanceList
+    for k, v in iteritems(awsBillingJson['products']):
+        i = v['attributes']
+        # NOTES:
+        #
+        # 3 tenant types: 'Host' (always $0.00; just a template?)
+        #                 'Dedicated' (toil does not support; these are pricier)
+        #                 'Shared' (AWS default and what toil uses)
+        #
+        # The same instance can appear with multiple "operation" values;
+        # "RunInstances" is normal
+        # "RunInstances:<code>" is e.g. Linux with MS SQL Server installed.
+        if (i.get('location') == region and
+            i.get('tenancy') == 'Shared' and
+            i.get('operatingSystem') == 'Linux' and
+            i.get('operation') == 'RunInstances'):
+
+            normal_use = i.get('usagetype').endswith('BoxUsage:' + i['instanceType'])  # not reserved or unused
+            if normal_use:
+                disks, disk_capacity = parseStorage(v["attributes"]["storage"])
+                instance = InstanceType(name=i["instanceType"],
+                                        cores=i["vcpu"],
+                                        memory=parseMemory(i["memory"]),
+                                        disks=disks,
+                                        disk_capacity=disk_capacity)
+                if instance in ec2InstanceList:
+                    raise RuntimeError('EC2 JSON format has likely changed.  '
+                                       'Duplicate instance {} found.'.format(instance))
+                ec2InstanceList.append(instance)
+    print('Finished for ' + str(region) + '.  ' + str(len(ec2InstanceList)) + ' added.')
+    return dict((_.name, _) for _ in ec2InstanceList)
 
 
 def updateStaticEC2Instances():
@@ -213,20 +211,39 @@ def updateStaticEC2Instances():
 
     :return: Nothing.  Writes a new 'generatedEC2Lists.py' file.
     """
-    logger.info("Updating Toil's EC2 lists to the most current version from AWS's bulk API.  "
-                "This may take a while, depending on your internet connection.")
+    print("Updating Toil's EC2 lists to the most current version from AWS's bulk API.\n"
+          "This may take a while, depending on your internet connection (~1Gb file).\n")
 
-    dirname = os.path.dirname(__file__)
-    # the file Toil uses to get info about EC2 instance types
-    origFile = os.path.join(dirname, 'generatedEC2Lists.py')
+    origFile = os.path.join(dirname, 'generatedEC2Lists.py')  # original
     assert os.path.exists(origFile)
     # use a temporary file until all info is fetched
-    genFile = os.path.join(dirname, 'generatedEC2Lists_tmp.py')
-    assert not os.path.exists(genFile)
-    # will be used to save a copy of the original when finished
-    oldFile = os.path.join(dirname, 'generatedEC2Lists_old.py')
+    genFile = os.path.join(dirname, 'generatedEC2Lists_tmp.py')  # temp
+    if os.path.exists(genFile):
+        os.remove(genFile)
 
-    # provenance note, copyright and imports
+    # filepath to store the aws json request (will be cleaned up)
+    # this is done because AWS changes their json format from time to time
+    # and debugging is faster with the file stored locally
+    awsJsonIndex = os.path.join(dirname, 'index.json')
+
+    if not os.path.exists(awsJsonIndex):
+        fetchEC2Index(filename=awsJsonIndex)
+    else:
+        print('Reusing previously downloaded json @: ' + awsJsonIndex)
+
+    with open(awsJsonIndex, 'r') as f:
+        awsProductDict = json.loads(f.read())
+
+    currentEC2List = []
+    instancesByRegion = {}
+    for regionNickname in EC2Regions:
+        currentEC2Dict = fetchEC2InstanceDict(awsProductDict, region=EC2Regions[regionNickname])
+        for instanceName, instanceTypeObj in iteritems(currentEC2Dict):
+            if instanceTypeObj not in currentEC2List:
+                currentEC2List.append(instanceTypeObj)
+            instancesByRegion.setdefault(regionNickname, []).append(instanceName)
+
+    # write provenance note, copyright and imports
     with open(genFile, 'w') as f:
         f.write(textwrap.dedent('''
         # !!! AUTOGENERATED FILE !!!
@@ -248,15 +265,6 @@ def updateStaticEC2Instances():
         from six import iteritems
         from toil.lib.ec2nodes import InstanceType\n\n\n''').format(year=datetime.date.today().strftime("%Y"))[1:])
 
-    currentEC2List = []
-    instancesByRegion = {}
-    for regionNickname, _ in iteritems(EC2Regions):
-        currentEC2Dict = fetchEC2InstanceDict(regionNickname=regionNickname)
-        for instanceName, instanceTypeObj in iteritems(currentEC2Dict):
-            if instanceTypeObj not in currentEC2List:
-                currentEC2List.append(instanceTypeObj)
-            instancesByRegion.setdefault(regionNickname, []).append(instanceName)
-
     # write header of total EC2 instance type list
     genString = "# {num} Instance Types.  Generated {date}.\n".format(
                 num=str(len(currentEC2List)), date=str(datetime.datetime.now()))
@@ -289,11 +297,11 @@ def updateStaticEC2Instances():
 
     with open(genFile, 'a+') as f:
         f.write(regionKey)
-    # preserve the original file unless it already exists
-    if not os.path.exists(oldFile):
-        os.rename(origFile, oldFile)
-    # delete the original file if it's still there
+    # delete the original file
     if os.path.exists(origFile):
         os.remove(origFile)
     # replace the instance list with a current list
     os.rename(genFile, origFile)
+    # delete the aws billing json file
+    if os.path.exists(awsJsonIndex):
+        os.remove(awsJsonIndex)
