@@ -19,14 +19,31 @@ from operator import attrgetter
 import datetime
 from toil.lib.misc import std_dev, mean
 from six import string_types
-
-from toil.test import runningOnEC2
+from six.moves.urllib.request import urlopen
+from six.moves.urllib.error import URLError
 
 logger = logging.getLogger(__name__)
 
 ZoneTuple = namedtuple('ZoneTuple', ['name', 'price_deviation'])
 
 
+def runningOnEC2():
+    def file_begins_with(path, prefix):
+        with open(path) as f:
+            return f.read(len(prefix)) == prefix
+
+    hv_uuid_path = '/sys/hypervisor/uuid'
+    if os.path.exists(hv_uuid_path) and file_begins_with(hv_uuid_path, 'ec2'):
+        return True
+    # Some instances do not have the /sys/hypervisor/uuid file, so check the identity document instead.
+    # See https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-identity-documents.html
+    try:
+        urlopen('http://169.254.169.254/latest/dynamic/instance-identity/document', timeout=1)
+        return True
+    except URLError:
+        return False
+
+
 def zoneToRegion(zone):
     """Get a region (e.g. us-west-2) from a zone (e.g. us-west-1c)."""
     from toil.lib.context import Context
@@ -201,12 +218,13 @@ iamFullPolicy = dict(Version="2012-10-17", Statement=[
 
 def checkValidNodeTypes(provisioner, nodeTypes):
     """
-    Raises if an invalid nodeType is specified for aws, azure, or gce.
+    Raises if an invalid nodeType is specified for aws or gce.
 
-    :param str provisioner: 'aws', 'gce', or 'azure' to specify which cloud provisioner used.
+    :param str provisioner: 'aws' or 'gce' to specify which cloud provisioner used.
     :param nodeTypes: A list of node types.  Example: ['t2.micro', 't2.medium']
     :return: Nothing.  Raises if invalid nodeType.
     """
+    # TODO: Move out of "aws.__init__.py"  >.>
     if not nodeTypes:
         return
     if not isinstance(nodeTypes, list):
@@ -235,8 +253,7 @@ def checkValidNodeTypes(provisioner, nodeTypes):
                     helpText = ''
                 raise RuntimeError('Invalid nodeType (%s) specified for AWS in region: %s.%s'
                                    '' % (nodeType, currentZone, helpText))
-    # Only checks if aws nodeType specified for gce/azure atm.
-    if provisioner == 'gce' or provisioner == 'azure':
+    elif provisioner == 'gce':
         for nodeType in nodeTypes:
             if nodeType and ':' in nodeType:
                 nodeType = nodeType.split(':')[0]
@@ -247,3 +264,6 @@ def checkValidNodeTypes(provisioner, nodeTypes):
                                    "".format(provisioner, provisioner))
             except KeyError:
                 pass
+    else:
+        raise RuntimeError("Invalid provisioner: {}".format(provisioner))
+
