@@ -340,7 +340,12 @@ class ClusterScaler(object):
             minNodes = [0 for node in self.nodeTypes]
         maxNodes = config.maxNodes
         while len(maxNodes) < len(self.nodeTypes):
+            # Pad out the max node counts if we didn't get one per type.
             maxNodes.append(maxNodes[0])
+        while len(minNodes) < len(self.nodeTypes):
+            # Pad out the min node counts with 0s, so we can have fewer than
+            # the node types without crashing.
+            minNodes.append(0)
         self.minNodes = dict(zip(self.nodeShapes, minNodes))
         self.maxNodes = dict(zip(self.nodeShapes, maxNodes))
 
@@ -671,7 +676,7 @@ class ClusterScaler(object):
         logger.debug('Terminating %i instance(s).', len(nodeToNodeInfo))
         if nodeToNodeInfo:
             for node in nodeToNodeInfo:
-                if node in self.ignoredNodes:
+                if node.privateIP in self.ignoredNodes:
                     self.ignoredNodes.remove(node.privateIP)
                     self.leader.batchSystem.unignoreNode(node.privateIP)
             self.provisioner.terminateNodes(nodeToNodeInfo)
@@ -685,7 +690,10 @@ class ClusterScaler(object):
         #Remove any nodes that have already been terminated from the list
         # of ignored nodes
         allNodeIPs = [node.privateIP for node in nodeToNodeInfo]
-        self.ignoredNodes = set([ip for ip in self.ignoredNodes if ip in allNodeIPs])
+        terminatedIPs = set([ip for ip in self.ignoredNodes if ip not in allNodeIPs])
+        for ip in terminatedIPs:
+            self.ignoredNodes.remove(ip)
+            self.leader.batchSystem.unignoreNode(ip)
 
         logger.debug("There are %i nodes being ignored by the batch system, "
                     "checking if they can be terminated" % len(self.ignoredNodes))
@@ -960,5 +968,3 @@ class ClusterStats(object):
                 threadName = 'Preemptable' if preemptable else 'Non-preemptable'
                 logger.debug('%s provisioner stats thread shut down successfully.', threadName)
                 self.stats[threadName] = stats
-        else:
-            pass
