@@ -40,6 +40,8 @@ from six.moves.urllib.request import urlopen
 from toil.lib.memoize import strict_bool
 from toil.lib.iterables import concat
 
+from toil.version import exactPython
+
 from toil import inVirtualEnv
 
 log = logging.getLogger(__name__)
@@ -287,11 +289,21 @@ class DirectoryResource(Resource):
         else:
             # This is a simple user script (with possibly a few helper files)
             rootDir = path
-        with ZipFile(file=bytesIO, mode='w') as zipFile:
-            for dirName, _, fileList in os.walk(path):
-                for fileName in fileList:
-                    fullPath = os.path.join(dirName, fileName)
-                    zipFile.write(fullPath, os.path.relpath(fullPath, rootDir))
+        skipdirList = ['/tmp', '/var', '/etc', '/bin', '/sbin', '/home', '/dev', '/sys', '/usr', '/run']
+        if path not in skipdirList:
+            with ZipFile(file=bytesIO, mode='w') as zipFile:
+                for dirName, _, fileList in os.walk(path):
+                    for fileName in fileList:
+                        try:
+                            fullPath = os.path.join(dirName, fileName)
+                            zipFile.write(fullPath, os.path.relpath(fullPath, rootDir))
+                        except IOError:
+                            log.critical('Cannot access and read the file at path: %s' % fullPath)
+                            sys.exit(1)
+        else:
+            log.critical("Couldn't package the directory at %s for hot deployment. Would recommend to create a \
+                subdirectory (ie %s/MYDIR_HERE/)" % (path, path))
+            sys.exit(1)
         bytesIO.seek(0)
         return bytesIO
 
@@ -555,8 +567,8 @@ class ModuleDescriptor(namedtuple('ModuleDescriptor', ('dirPath', 'name', 'fromV
                 raise ResourceException(
                     "Toil does not support loading a user script from a package directory. You "
                     "may want to remove %s from %s or invoke the user script as a module via "
-                    "'PYTHONPATH=\"%s\" python -m %s.%s'." %
-                    tuple(concat(initName, self.dirPath, os.path.split(self.dirPath), self.name)))
+                    "'PYTHONPATH=\"%s\" %s -m %s.%s'." %
+                    tuple(concat(initName, self.dirPath, exactPython, os.path.split(self.dirPath), self.name)))
             return self.dirPath
 
     moduleExtensions = ('.py', '.pyc', '.pyo')
