@@ -29,12 +29,14 @@ import itertools
 import tempfile
 from textwrap import dedent
 import time
-import multiprocessing
 import sys
 from toil import subprocess
 from unittest import skipIf
 
 from toil.common import Config
+# Don't import any batch systems here that depend on extras
+# in order to import properly. Import them later, in tests 
+# protected by annotations.
 from toil.batchSystems.mesos.test import MesosTestSupport
 from toil.batchSystems.parasolTestSupport import ParasolTestSupport
 from toil.batchSystems.parasol import ParasolBatchSystem
@@ -42,16 +44,21 @@ from toil.batchSystems.singleMachine import SingleMachineBatchSystem
 from toil.batchSystems.abstractBatchSystem import (InsufficientSystemResources,
                                                    BatchSystemSupport)
 from toil.job import Job, JobNode
+from toil.lib.threading import cpu_count
 from toil.test import (ToilTest,
+                       needs_aws_s3,
                        needs_lsf,
+                       needs_kubernetes,
                        needs_mesos,
                        needs_parasol,
                        needs_gridengine,
                        needs_slurm,
                        needs_torque,
                        needs_htcondor,
+                       needs_downloadable_appliance,
                        slow,
-                       tempFileContaining)
+                       tempFileContaining,
+                       travis_test)
 from future.utils import with_metaclass
 
 log = logging.getLogger(__name__)
@@ -75,7 +82,9 @@ class hidden(object):
 
     class AbstractBatchSystemTest(with_metaclass(ABCMeta, ToilTest)):
         """
-        A base test case with generic tests that every batch system should pass
+        A base test case with generic tests that every batch system should pass.
+
+        Cannot assume that the batch system actually executes commands on the local machine/filesystem.
         """
 
         @abstractmethod
@@ -126,12 +135,11 @@ class hidden(object):
         def tearDown(self):
             self.batchSystem.shutdown()
             super(hidden.AbstractBatchSystemTest, self).tearDown()
-
+        
         def testAvailableCores(self):
-            self.assertTrue(multiprocessing.cpu_count() >= numCores)
-
+            self.assertTrue(cpu_count() >= numCores)
+        
         def testRunJobs(self):
-            testPath = os.path.join(self.tempDir, "test.txt")
             jobNode1 = JobNode(command='sleep 1000', jobName='test1', unitName=None,
                                jobStoreID='1', requirements=defaultRequirements)
             jobNode2 = JobNode(command='sleep 1000', jobName='test2', unitName=None,
@@ -142,7 +150,17 @@ class hidden(object):
             issuedIDs = self._waitForJobsToIssue(2)
             self.assertEqual(set(issuedIDs), {job1, job2})
 
-            runningJobIDs = self._waitForJobsToStart(2)
+            # Now at some point we want these jobs to become running
+            # But since we may be testing against a live cluster (Kubernetes)
+            # we want to handle weird cases and high cluster load as much as we can.
+
+            # Wait a bit for any Dockers to download and for the
+            # jobs to have a chance to start.
+            # TODO: We insist on neither of these ever finishing when we test
+            # getUpdatedBatchJob, and the sleep time is longer than the time we
+            # should spend waiting for both to start, so if our cluster can
+            # only run one job at a time, we will fail the test.
+            runningJobIDs = self._waitForJobsToStart(2, tries=120)
             self.assertEqual(set(runningJobIDs), {job1, job2})
 
             # Killing the jobs instead of allowing them to complete means this test can run very
@@ -152,59 +170,60 @@ class hidden(object):
 
             # Issue a job and then allow it to finish by itself, causing it to be added to the
             # updated jobs queue.
-            self.assertFalse(os.path.exists(testPath))
-            jobNode3 = JobNode(command="touch %s" % testPath, jobName='test3', unitName=None,
+            # We would like to have this touch something on the filesystem and
+            # then check for it having happened, but we can't guarantee that
+            # the batch system will run against the same filesystem we are
+            # looking at.
+            jobNode3 = JobNode(command="mktemp -d", jobName='test3', unitName=None,
                                jobStoreID='3', requirements=defaultRequirements)
             job3 = self.batchSystem.issueBatchJob(jobNode3)
 
             jobID, exitStatus, wallTime = self.batchSystem.getUpdatedBatchJob(maxWait=1000)
+            log.info('Third job completed: {} {} {}'.format(jobID, exitStatus, wallTime))
 
             # Since the first two jobs were killed, the only job in the updated jobs queue should
             # be job 3. If the first two jobs were (incorrectly) added to the queue, this will
             # fail with jobID being equal to job1 or job2.
-            self.assertEqual(exitStatus, 0)
             self.assertEqual(jobID, job3)
+            self.assertEqual(exitStatus, 0)
             if self.supportsWallTime():
                 self.assertTrue(wallTime > 0)
             else:
                 self.assertIsNone(wallTime)
-            if not os.path.exists(testPath):
-                time.sleep(20)
-            self.assertTrue(os.path.exists(testPath))
+            # TODO: Work out a way to check if the job we asked to run actually ran.
+            # Don't just believe the batch system, but don't assume it ran on this machine either.
             self.assertFalse(self.batchSystem.getUpdatedBatchJob(0))
 
             # Make sure killBatchJobs can handle jobs that don't exist
             self.batchSystem.killBatchJobs([10])
-
+        
         def testSetEnv(self):
-            # Parasol disobeys shell rules and stupidly splits the command at the space character
-            # before exec'ing it, whether the space is quoted, escaped or not. This means that we
-            # can't have escaped or quotes spaces in the command line. So we can't use bash -c
-            #  '...' or python -c '...'. The safest thing to do here is to script the test and
-            # invoke that script rather than inline the test via -c.
-            def assertEnv():
-                import os, sys
-                sys.exit(23 if os.getenv('FOO') == 'bar' else 42)
+            # Parasol disobeys shell rules and stupidly splits the command at
+            # the space character into arguments before exec'ing it, whether
+            # the space is quoted, escaped or not.
 
-            script_body = dedent('\n'.join(getsource(assertEnv).split('\n')[1:]))
-            with tempFileContaining(script_body, suffix='.py') as script_path:
-                # First, ensure that the test fails if the variable is *not* set
-                command = sys.executable + ' ' + script_path
-                jobNode4 = JobNode(command=command, jobName='test4', unitName=None,
-                                   jobStoreID='4', requirements=defaultRequirements)
-                job4 = self.batchSystem.issueBatchJob(jobNode4)
-                jobID, exitStatus, wallTime = self.batchSystem.getUpdatedBatchJob(maxWait=1000)
-                self.assertEqual(exitStatus, 42)
-                self.assertEqual(jobID, job4)
-                # Now set the variable and ensure that it is present
-                self.batchSystem.setEnv('FOO', 'bar')
-                jobNode5 = JobNode(command=command, jobName='test5', unitName=None,
-                                   jobStoreID='5', requirements=defaultRequirements)
-                job5 = self.batchSystem.issueBatchJob(jobNode5)
-                jobID, exitStatus, wallTime = self.batchSystem.getUpdatedBatchJob(maxWait=1000)
-                self.assertEqual(exitStatus, 23)
-                self.assertEqual(jobID, job5)
+            script_shell = 'if [ "x${FOO}" == "xbar" ] ; then exit 23 ; else exit 42 ; fi'
 
+            # Escape the semicolons
+            script_protected = script_shell.replace(';', '\;') 
+             
+            # Turn into a string which convinces bash to take all args and paste them back together and run them
+            command = "bash -c \"\\${@}\" bash eval " + script_protected
+            jobNode4 = JobNode(command=command, jobName='test4', unitName=None,
+                               jobStoreID='4', requirements=defaultRequirements)
+            job4 = self.batchSystem.issueBatchJob(jobNode4)
+            jobID, exitStatus, wallTime = self.batchSystem.getUpdatedBatchJob(maxWait=1000)
+            self.assertEqual(exitStatus, 42)
+            self.assertEqual(jobID, job4)
+            # Now set the variable and ensure that it is present
+            self.batchSystem.setEnv('FOO', 'bar')
+            jobNode5 = JobNode(command=command, jobName='test5', unitName=None,
+                               jobStoreID='5', requirements=defaultRequirements)
+            job5 = self.batchSystem.issueBatchJob(jobNode5)
+            jobID, exitStatus, wallTime = self.batchSystem.getUpdatedBatchJob(maxWait=1000)
+            self.assertEqual(exitStatus, 23)
+            self.assertEqual(jobID, job5)
+        
         def testCheckResourceRequest(self):
             if isinstance(self.batchSystem, BatchSystemSupport):
                 checkResourceRequest = self.batchSystem.checkResourceRequest
@@ -221,7 +240,7 @@ class hidden(object):
                 self.assertRaises(AssertionError, checkResourceRequest,
                                   memory=10, cores=None, disk=1000)
                 checkResourceRequest(memory=10, cores=1, disk=100)
-
+       
         def testScalableBatchSystem(self):
             # If instance of scalable batch system
             pass
@@ -235,11 +254,20 @@ class hidden(object):
                 time.sleep(1)
             return issuedIDs
 
-        def _waitForJobsToStart(self, numJobs):
+        def _waitForJobsToStart(self, numJobs, tries=20):
+            """
+            Loop until the given number of distinct jobs are in the
+            running state, or until the given number of tries is exhausted
+            (with 1 second polling period).
+
+            Returns the list of IDs that are running.
+            """
             runningIDs = []
-            # prevent an endless loop, give it 20 tries
-            for it in range(20):
-                runningIDs = list(self.batchSystem.getRunningBatchJobIDs().keys())
+            # prevent an endless loop, give it a few tries
+            for it in range(tries):
+                running = self.batchSystem.getRunningBatchJobIDs()
+                log.info('Running jobs now: {}'.format(running))
+                runningIDs = list(running.keys())
                 if len(runningIDs) == numJobs:
                     break
                 time.sleep(1)
@@ -251,7 +279,7 @@ class hidden(object):
         than using the batch system directly.
         """
 
-        cpuCount = multiprocessing.cpu_count()
+        cpuCount = cpu_count()
         allocatedCores = sorted({1, 2, cpuCount})
         sleepTime = 5
 
@@ -316,6 +344,23 @@ class hidden(object):
             config.jobStore = 'file:' + self._createTempDir('jobStore')
             return config
 
+@needs_kubernetes
+@needs_aws_s3
+@needs_downloadable_appliance
+class KubernetesBatchSystemTest(hidden.AbstractBatchSystemTest):
+    """
+    Tests against the Kubernetes batch system
+    """
+
+    def supportsWallTime(self):
+        return True
+
+    def createBatchSystem(self):
+        # We know we have Kubernetes so we can import the batch system
+        from toil.batchSystems.kubernetes import KubernetesBatchSystem
+        return KubernetesBatchSystem(config=self.config,
+                                     maxCores=numCores, maxMemory=1e9, maxDisk=2001)
+
 @slow
 @needs_mesos
 class MesosBatchSystemTest(hidden.AbstractBatchSystemTest, MesosTestSupport):
@@ -336,6 +381,7 @@ class MesosBatchSystemTest(hidden.AbstractBatchSystemTest, MesosTestSupport):
         return True
 
     def createBatchSystem(self):
+        # We know we have Mesos so we can import the batch system
         from toil.batchSystems.mesos.batchSystem import MesosBatchSystem
         self._startMesos(numCores)
         return MesosBatchSystem(config=self.config,
@@ -354,11 +400,12 @@ class MesosBatchSystemTest(hidden.AbstractBatchSystemTest, MesosTestSupport):
         issuedID = self._waitForJobsToIssue(1)
         self.assertEqual(set(issuedID), {job})
 
-        runningJobIDs = self._waitForJobsToStart(1)
+        # Wait until a job starts or we go a while without that happening
+        runningJobIDs = self._waitForJobsToStart(1, tries=20)
         # Make sure job is NOT running
         self.assertEqual(set(runningJobIDs), set({}))
 
-
+@travis_test
 class SingleMachineBatchSystemTest(hidden.AbstractBatchSystemTest):
     """
     Tests against the single-machine batch system
@@ -714,6 +761,7 @@ class HTCondorBatchSystemTest(hidden.AbstractGridEngineBatchSystemTest):
     def tearDown(self):
         super(HTCondorBatchSystemTest, self).tearDown()
 
+@travis_test
 class SingleMachineBatchSystemJobTest(hidden.AbstractBatchSystemJobTest):
     """
     Tests Toil workflow against the SingleMachine batch system
