@@ -49,23 +49,33 @@ from toil.lib.exceptions import panic
 # (installed by `make prepare`)
 from mock import patch
 
+from toil.lib.compatibility import USING_PYTHON2
 from toil.common import Config, Toil
-from toil.fileStore import FileID
+from toil.fileStores import FileID
 from toil.job import Job, JobNode
 from toil.jobStores.abstractJobStore import (NoSuchJobException,
                                              NoSuchFileException)
-from toil.jobStores.googleJobStore import googleRetry
 from toil.jobStores.fileJobStore import FileJobStore
 from toil.statsAndLogging import StatsAndLogging
 from toil.test import (ToilTest,
-                       needs_aws,
-                       needs_azure,
+                       needs_aws_s3,
                        needs_encryption,
                        make_tests,
                        needs_google,
+                       travis_test,
                        slow)
 from future.utils import with_metaclass
 
+# Need googleRetry decorator even if google is not available, so make one up.
+# Unconventional use of decorator to determine if google is enabled by seeing if
+# it returns the parameter passed in.
+if needs_google(needs_google) is needs_google:
+    from toil.jobStores.googleJobStore import googleRetry
+else:
+    def googleRetry(x):
+        return x
+
+
 logger = logging.getLogger(__name__)
 
 
@@ -140,11 +150,13 @@ class AbstractJobStoreTest(object):
             self.jobstore_resumed_noconfig.destroy()
             super(AbstractJobStoreTest.Test, self).tearDown()
 
+        @travis_test
         def testInitialState(self):
             """Ensure proper handling of nonexistant files."""
             self.assertFalse(self.jobstore_initialized.exists('nonexistantFile'))
             self.assertRaises(NoSuchJobException, self.jobstore_initialized.load, 'nonexistantFile')
 
+        @travis_test
         def testJobCreation(self):
             """
             Test creation of a job.
@@ -175,6 +187,7 @@ class AbstractJobStoreTest(object):
             self.assertEqual(job.predecessorsFinished, set())
             self.assertEqual(job.logJobStoreFileID, None)
 
+        @travis_test
         def testConfigEquality(self):
             """
             Ensure that the command line configurations are successfully loaded and stored.
@@ -188,6 +201,7 @@ class AbstractJobStoreTest(object):
             self.assertEqual(newJobStore.config, self.config)
             self.assertIsNot(newJobStore.config, self.config)
 
+        @travis_test
         def testJobLoadEquality(self):
             """Tests that a job loaded into one jobstore from another can be used equivalently by another."""
 
@@ -203,6 +217,7 @@ class AbstractJobStoreTest(object):
 
             self.assertEqual(job1, job2)
 
+        @travis_test
         def testChildLoadingEquality(self):
             """Test that loading a child job operates as expected."""
             aJobNode = JobNode(command='parent1',
@@ -220,6 +235,7 @@ class AbstractJobStoreTest(object):
             self.jobstore_initialized.update(job)
             self.assertEqual(self.jobstore_initialized.load(childJob.jobStoreID), childJob)
 
+        @travis_test
         def testPersistantFilesToDelete(self):
             """
             Make sure that updating a job  carries over filesToDelete.
@@ -231,15 +247,16 @@ class AbstractJobStoreTest(object):
 
             # Create a job.
             jobNode = JobNode(command='job1',
-                               requirements=self.parentJobReqs,
-                               jobName='test1', unitName='onJS1',
-                               jobStoreID=None, predecessorNumber=0)
+                              requirements=self.parentJobReqs,
+                              jobName='test1', unitName='onJS1',
+                              jobStoreID=None, predecessorNumber=0)
 
             job = self.jobstore_initialized.create(jobNode)
             job.filesToDelete = ['1', '2']
             self.jobstore_initialized.update(job)
             self.assertEqual(self.jobstore_initialized.load(job.jobStoreID).filesToDelete, ['1', '2'])
 
+        @travis_test
         def testUpdateBehavior(self):
             """Tests the proper behavior during updating jobs."""
             jobstore1 = self.jobstore_initialized
@@ -273,7 +290,7 @@ class AbstractJobStoreTest(object):
 
             # Check equivalence between jobstore1 and jobstore2.
             # While job1 and job2 share a jobStoreID, job1 has not been "refreshed" to show the newly added child jobs.
-            self.assertNotEquals(job2, job1)
+            self.assertNotEqual(job2, job1)
 
             # Reload parent job on jobstore, "refreshing" the job.
             job1 = jobstore1.load(job1.jobStoreID)
@@ -282,9 +299,10 @@ class AbstractJobStoreTest(object):
             # Load children on jobstore and check equivalence
             self.assertEqual(jobstore1.load(childJob1.jobStoreID), childJob1)
             self.assertEqual(jobstore1.load(childJob2.jobStoreID), childJob2)
-            self.assertEqual(job1,job2)            # The jobs should both have children now...
-            self.assertIsNot(job1,job2)             # but should not be the same.
+            self.assertEqual(job1, job2)  # The jobs should both have children now...
+            self.assertIsNot(job1, job2)  # but should not be the same.
 
+        @travis_test
         def testChangingJobStoreID(self):
             """
             Tests that changing the jobStoreID makes jobs unequivalent.
@@ -308,7 +326,7 @@ class AbstractJobStoreTest(object):
             parentJob2 = jobstore2.load(parentJob1.jobStoreID)
 
             # Create an array of child jobs for each jobstore.
-            for i in range(0,5):
+            for i in range(0, 5):
                 jobNodeOnChild1 = JobNode(command='child' + str(i),
                                           requirements=self.childJobReqs1,
                                           jobName='test' + str(i), unitName='onChild1',
@@ -322,7 +340,7 @@ class AbstractJobStoreTest(object):
                 self.assertEqual(childJob, jobstore1.load(childJob.jobStoreID))
                 childJob.logJobStoreFileID = str(uuid.uuid4())
                 childJob.remainingRetryCount = 66
-                self.assertNotEquals(childJob, jobstore1.load(childJob.jobStoreID))
+                self.assertNotEqual(childJob, jobstore1.load(childJob.jobStoreID))
 
             # Update the children on the second jobstore.
             for childJob in parentJob2.stack:
@@ -333,6 +351,7 @@ class AbstractJobStoreTest(object):
                 self.assertEqual(jobstore1.load(childJob.jobStoreID), childJob)
                 self.assertEqual(jobstore2.load(childJob.jobStoreID), childJob)
 
+        @travis_test
         def testJobDeletions(self):
             """Tests the consequences of deleting jobs."""
             # A local jobstore object for testing.
@@ -392,6 +411,7 @@ class AbstractJobStoreTest(object):
             except NoSuchFileException:
                 pass
 
+        @travis_test
         def testSharedFiles(self):
             """Tests the sharing of files."""
             jobstore1 = self.jobstore_initialized
@@ -415,6 +435,7 @@ class AbstractJobStoreTest(object):
             self.assertUrl(jobstore1.getSharedPublicUrl('nonEncrypted'))
             self.assertRaises(NoSuchFileException, jobstore1.getSharedPublicUrl, 'missing')
 
+        @travis_test
         def testPerJobFiles(self):
             """Tests the behavior of files on jobs."""
             jobstore1 = self.jobstore_initialized
@@ -428,7 +449,7 @@ class AbstractJobStoreTest(object):
 
             # First recreate job
             jobOnJobStore1 = jobstore1.create(jobNodeOnJobStore1)
-            fileOne = jobstore2.getEmptyFileStoreID(jobOnJobStore1.jobStoreID)
+            fileOne = jobstore2.getEmptyFileStoreID(jobOnJobStore1.jobStoreID, cleanup=True)
             # Check file exists
             self.assertTrue(jobstore2.fileExists(fileOne))
             self.assertTrue(jobstore1.fileExists(fileOne))
@@ -463,7 +484,7 @@ class AbstractJobStoreTest(object):
                     f.truncate(0)
                     f.write(two)
                 # ... and create a second file from the local file.
-                fileTwo = jobstore1.writeFile(path, jobOnJobStore1.jobStoreID)
+                fileTwo = jobstore1.writeFile(path, jobOnJobStore1.jobStoreID, cleanup=True)
                 with jobstore2.readFileStream(fileTwo) as f:
                     self.assertEqual(f.read(), two)
                 # Now update the first file from the local file ...
@@ -473,7 +494,7 @@ class AbstractJobStoreTest(object):
             finally:
                 os.unlink(path)
             # Create a third file to test the last remaining method.
-            with jobstore2.writeFileStream(jobOnJobStore1.jobStoreID) as (f, fileThree):
+            with jobstore2.writeFileStream(jobOnJobStore1.jobStoreID, cleanup=True) as (f, fileThree):
                 f.write(three)
             with jobstore1.readFileStream(fileThree) as f:
                 self.assertEqual(f.read(), three)
@@ -492,6 +513,7 @@ class AbstractJobStoreTest(object):
                 except NoSuchFileException:
                     pass
 
+        @travis_test
         def testStatsAndLogging(self):
             """Tests behavior of reading and writting stats and logging."""
             jobstore1 = self.jobstore_initialized
@@ -506,13 +528,8 @@ class AbstractJobStoreTest(object):
 
             # Test stats and logging
             stats = None
-            one = 'one'
-            two = 'two'
-            three = 'three'
-            if sys.version_info >= (3, 0):
-                one = b'one'
-                two = b'two'
-                three = b'three'
+            one = b'one' if not USING_PYTHON2 else 'one'
+            two = b'two' if not USING_PYTHON2 else 'two'
 
             # Allows stats to be read/written to/from in read/writeStatsAndLogging.
             def callback(f2):
@@ -529,7 +546,7 @@ class AbstractJobStoreTest(object):
             jobstore2.writeStatsAndLogging(one)
             self.assertEqual(1, jobstore1.readStatsAndLogging(callback))
             self.assertEqual({one}, stats)
-            self.assertEqual(0, jobstore1.readStatsAndLogging(callback))   # readStatsAndLogging purges saved stats etc
+            self.assertEqual(0, jobstore1.readStatsAndLogging(callback))  # readStatsAndLogging purges saved stats etc
 
             jobstore2.writeStatsAndLogging(one)
             jobstore2.writeStatsAndLogging(two)
@@ -551,6 +568,7 @@ class AbstractJobStoreTest(object):
             self.assertFalse(jobstore1.exists(jobOnJobStore1.jobStoreID))
             # TODO: Who deletes the shared files?
 
+        @travis_test
         def testWriteLogFiles(self):
             """Test writing log files."""
             jobNames = ['testStatsAndLogging_writeLogFiles']
@@ -565,6 +583,7 @@ class AbstractJobStoreTest(object):
                 self.assertEqual(f.read(), 'string\nbytes\n\nnewline\n')
             os.remove(jobLogFile)
 
+        @travis_test
         def testBatchCreate(self):
             """Test creation of many jobs."""
             jobstore = self.jobstore_initialized
@@ -580,6 +599,7 @@ class AbstractJobStoreTest(object):
             for jobGraph in jobGraphs:
                 self.assertTrue(jobstore.exists(jobGraph.jobStoreID))
 
+        @travis_test
         def testGrowingAndShrinkingJob(self):
             """Make sure jobs update correctly if they grow/shrink."""
             # Make some very large data, large enough to trigger
@@ -656,7 +676,7 @@ class AbstractJobStoreTest(object):
         @classmethod
         def makeImportExportTests(cls):
 
-            testClasses = [FileJobStoreTest, AWSJobStoreTest, AzureJobStoreTest, GoogleJobStoreTest]
+            testClasses = [FileJobStoreTest, AWSJobStoreTest, GoogleJobStoreTest]
 
             activeTestClassesByName = {testCls.__name__: testCls
                                        for testCls in testClasses
@@ -726,6 +746,7 @@ class AbstractJobStoreTest(object):
                        cls,
                        otherCls=activeTestClassesByName)
 
+        @travis_test
         def testImportHttpFile(self):
             '''Test importing a file over HTTP.'''
             http = socketserver.TCPServer(('', 0), StubHttpRequestHandler)
@@ -735,7 +756,8 @@ class AbstractJobStoreTest(object):
                 try:
                     assignedPort = http.server_address[1]
                     url = 'http://localhost:%d' % assignedPort
-                    with self.jobstore_initialized.readFileStream(self.jobstore_initialized.importFile(url)) as readable:
+                    with self.jobstore_initialized.readFileStream(
+                            self.jobstore_initialized.importFile(url)) as readable:
                         f1 = readable.read()
                         f2 = StubHttpRequestHandler.fileContents
                         if isinstance(f1, bytes) and not isinstance(f2, bytes):
@@ -749,6 +771,7 @@ class AbstractJobStoreTest(object):
             finally:
                 http.server_close()
 
+        @travis_test
         def testImportFtpFile(self):
             '''Test importing a file over FTP'''
             ftpfile = {'name': 'foo', 'content': 'foo bar baz qux'}
@@ -777,7 +800,8 @@ class AbstractJobStoreTest(object):
             n = self._batchDeletionSize()
             for numFiles in (1, n - 1, n, n + 1, 2 * n):
                 job = self.jobstore_initialized.create(self.arbitraryJob)
-                fileIDs = [self.jobstore_initialized.getEmptyFileStoreID(job.jobStoreID) for _ in range(0, numFiles)]
+                fileIDs = [self.jobstore_initialized.getEmptyFileStoreID(job.jobStoreID, cleanup=True) for _ in
+                           range(0, numFiles)]
                 self.jobstore_initialized.delete(job.jobStoreID)
                 for fileID in fileIDs:
                     # NB: the fooStream() methods return context managers
@@ -789,8 +813,6 @@ class AbstractJobStoreTest(object):
             This test is meant to cover multi-part uploads in the AWSJobStore but it doesn't hurt
             running it against the other job stores as well.
             """
-            # Should not block. On Linux, /dev/random blocks when it's running low on entropy
-            random_device = '/dev/urandom'
             # http://unix.stackexchange.com/questions/11946/how-big-is-the-pipe-buffer
             bufSize = 65536
             partSize = self._partSize()
@@ -815,8 +837,10 @@ class AbstractJobStoreTest(object):
                 checksumThread = Thread(target=checksumThreadFn)
                 checksumThread.start()
                 try:
-                    with open(random_device, 'rb') as readable:
-                        with self.jobstore_initialized.writeFileStream(job.jobStoreID) as (writable, fileId):
+                    # Should not block. On Linux, /dev/random blocks when it's running low on entropy
+                    with open('/dev/urandom', 'rb') as readable:
+                        with self.jobstore_initialized.writeFileStream(job.jobStoreID, cleanup=True) as (
+                        writable, fileId):
                             for i in range(int(partSize * partsPerFile / bufSize)):
                                 buf = readable.read(bufSize)
                                 checksumQueue.put(buf)
@@ -842,12 +866,12 @@ class AbstractJobStoreTest(object):
                 fh, path = tempfile.mkstemp()
                 try:
                     with os.fdopen(fh, 'wb+') as writable:
-                        with open(random_device, 'rb') as readable:
+                        with open('/dev/urandom', 'rb') as readable:
                             for i in range(int(partSize * partsPerFile / bufSize)):
                                 buf = readable.read(bufSize)
                                 writable.write(buf)
                                 checksum.update(buf)
-                    fileId = self.jobstore_initialized.writeFile(path, job.jobStoreID)
+                    fileId = self.jobstore_initialized.writeFile(path, job.jobStoreID, cleanup=True)
                 finally:
                     os.unlink(path)
                 before = checksum.hexdigest()
@@ -864,13 +888,14 @@ class AbstractJobStoreTest(object):
                 self.assertEqual(before, after)
             self.jobstore_initialized.delete(job.jobStoreID)
 
+        @travis_test
         def testZeroLengthFiles(self):
             '''Test reading and writing of empty files.'''
             job = self.jobstore_initialized.create(self.arbitraryJob)
-            nullFile = self.jobstore_initialized.writeFile('/dev/null', job.jobStoreID)
+            nullFile = self.jobstore_initialized.writeFile('/dev/null', job.jobStoreID, cleanup=True)
             with self.jobstore_initialized.readFileStream(nullFile) as f:
                 assert not f.read()
-            with self.jobstore_initialized.writeFileStream(job.jobStoreID) as (f, nullStream):
+            with self.jobstore_initialized.writeFileStream(job.jobStoreID, cleanup=True) as (f, nullStream):
                 pass
             with self.jobstore_initialized.readFileStream(nullStream) as f:
                 assert not f.read()
@@ -891,7 +916,7 @@ class AbstractJobStoreTest(object):
 
             # Load the file into a jobstore.
             job = self.jobstore_initialized.create(self.arbitraryJob)
-            jobStoreFileID = self.jobstore_initialized.writeFile(filePath, job.jobStoreID)
+            jobStoreFileID = self.jobstore_initialized.writeFile(filePath, job.jobStoreID, cleanup=True)
 
             # Remove the local file.
             os.unlink(filePath)
@@ -940,6 +965,9 @@ class AbstractJobStoreTest(object):
 
             noCacheTime = noCacheEnd - noCacheStart
 
+            # Make sure we have all the jobs: root and children.
+            self.assertEqual(len(list(jobstore.jobs())), 101)
+
             # See how long it takes to clean with cache
             jobCache = {jobGraph.jobStoreID: jobGraph
                         for jobGraph in jobstore.jobs()}
@@ -954,37 +982,15 @@ class AbstractJobStoreTest(object):
             # Running with the cache should be faster.
             self.assertTrue(cacheTime <= noCacheTime)
 
-        @skip("too slow")  # This takes a long time on the remote JobStores
-        def testManyJobs(self):
-            # Make sure we can store large numbers of jobs
-
-            # Make a bunch of jobs
-            jobstore = self.jobstore_initialized
-
-            # Create parent job
-            rootJob = jobstore.createRootJob(self.arbitraryJob)
-
-            # Create a bunch of child jobs
-            for i in range(3000):
-                child = jobstore.create(self.arbitraryJob)
-                rootJob.stack.append(child)
-            jobstore.update(rootJob)
-
-            # Pull them all back out again
-            allJobs = list(jobstore.jobs())
-
-            # Make sure we have the right number of jobs. Cannot be precise because of limitations
-            # on the jobs iterator for certain cloud providers
-            self.assertTrue(len(allJobs) <= 3001)
-
         # NB: the 'thread' method seems to be needed here to actually
         # ensure the timeout is raised, probably because the only
         # "live" thread doesn't hold the GIL.
+        @travis_test
         @pytest.mark.timeout(45, method='thread')
         def testPartialReadFromStream(self):
             """Test whether readFileStream will deadlock on a partial read."""
             job = self.jobstore_initialized.create(self.arbitraryJob)
-            with self.jobstore_initialized.writeFileStream(job.jobStoreID) as (f, fileID):
+            with self.jobstore_initialized.writeFileStream(job.jobStoreID, cleanup=True) as (f, fileID):
                 # Write enough data to make sure the writer thread
                 # will get blocked on the write. Technically anything
                 # greater than the pipe buffer size plus the libc
@@ -1014,6 +1020,7 @@ class AbstractJobStoreTest(object):
             jobstore.destroy()
             # Note that self.jobstore_initialized.destroy() is done as part of shutdown
 
+        @travis_test
         def testDestructionIdempotence(self):
             # Jobstore is fully initialized
             self.jobstore_initialized.destroy()
@@ -1026,6 +1033,7 @@ class AbstractJobStoreTest(object):
             cleaner = self._createJobStore()
             cleaner.destroy()
 
+        @travis_test
         def testEmptyFileStoreIDIsReadable(self):
             """Simply creates an empty fileStoreID and attempts to read from it."""
             id = self.jobstore_initialized.getEmptyFileStoreID()
@@ -1059,14 +1067,11 @@ class AbstractEncryptedJobStoreTest(object):
         def setUp(self):
             # noinspection PyAttributeOutsideInit
             self.sseKeyDir = tempfile.mkdtemp()
-            # noinspection PyAttributeOutsideInit
-            self.cseKeyDir = tempfile.mkdtemp()
             super(AbstractEncryptedJobStoreTest.Test, self).setUp()
 
         def tearDown(self):
             super(AbstractEncryptedJobStoreTest.Test, self).tearDown()
             shutil.rmtree(self.sseKeyDir)
-            shutil.rmtree(self.cseKeyDir)
 
         def _createConfig(self):
             config = super(AbstractEncryptedJobStoreTest.Test, self)._createConfig()
@@ -1075,11 +1080,6 @@ class AbstractEncryptedJobStoreTest(object):
                 f.write('01234567890123456789012345678901')
             config.sseKey = sseKeyFile
             # config.attrib['sse_key'] = sseKeyFile
-
-            cseKeyFile = os.path.join(self.cseKeyDir, 'keyFile')
-            with open(cseKeyFile, 'w') as f:
-                f.write("i am a fake key, so don't use me")
-            config.cseKey = cseKeyFile
             return config
 
         def testEncrypted(self):
@@ -1087,28 +1087,29 @@ class AbstractEncryptedJobStoreTest(object):
             Create an encrypted file. Read it in encrypted mode then try with encryption off
             to ensure that it fails.
             """
-            phrase = 'This file is encrypted.'
+            phrase = 'This file is encrypted.'.encode('utf-8')
             fileName = 'foo'
             with self.jobstore_initialized.writeSharedFileStream(fileName, isProtected=True) as f:
                 f.write(phrase)
             with self.jobstore_initialized.readSharedFileStream(fileName) as f:
                 self.assertEqual(phrase, f.read())
 
-            #disable encryption
+            # disable encryption
             self.jobstore_initialized.config.sseKey = None
-            self.jobstore_initialized.config.cseKey = None
             try:
                 with self.jobstore_initialized.readSharedFileStream(fileName) as f:
                     self.assertEqual(phrase, f.read())
             except AssertionError as e:
-                self.assertEqual("Content is encrypted but no key was provided.", e.message)
+                self.assertEqual("Content is encrypted but no key was provided.", e.args[0])
             else:
                 self.fail("Read encryption content with encryption off.")
 
 
 class FileJobStoreTest(AbstractJobStoreTest.Test):
     def _createJobStore(self):
-        return FileJobStore(self.namePrefix)
+        # Make a FileJobStore with an artificially low fan out threshold, to
+        # make sure to test fan out logic
+        return FileJobStore(self.namePrefix, fanOut=2)
 
     def _corruptJobStore(self):
         assert isinstance(self.jobstore_initialized, FileJobStore)  # type hint
@@ -1139,18 +1140,18 @@ class FileJobStoreTest(AbstractJobStoreTest.Test):
     def _cleanUpExternalStore(self, dirPath):
         shutil.rmtree(dirPath)
 
+    @travis_test
     def testPreserveFileName(self):
         "Check that the fileID ends with the given file name."
         fh, path = tempfile.mkstemp()
         try:
             os.close(fh)
             job = self.jobstore_initialized.create(self.arbitraryJob)
-            fileID = self.jobstore_initialized.writeFile(path, job.jobStoreID)
+            fileID = self.jobstore_initialized.writeFile(path, job.jobStoreID, cleanup=True)
             self.assertTrue(fileID.endswith(os.path.basename(path)))
         finally:
             os.unlink(path)
 
-
 @needs_google
 class GoogleJobStoreTest(AbstractJobStoreTest.Test):
     projectID = os.getenv('TOIL_GOOGLE_PROJECTID')
@@ -1171,8 +1172,12 @@ class GoogleJobStoreTest(AbstractJobStoreTest.Test):
         url = 'gs://%s/%s' % (bucket.name, fileName)
         if size is None:
             return url
-        with open('/dev/urandom', 'r') as readable:
-            contents = readable.read(size)
+        read_type = 'r' if USING_PYTHON2 else 'rb'
+        with open('/dev/urandom', read_type) as readable:
+            if USING_PYTHON2:
+                contents = readable.read(size)
+            else:
+                contents = str(readable.read(size))
         GoogleJobStore._writeToUrl(StringIO(contents), urlparse.urlparse(url))
         return url, hashlib.md5(contents).hexdigest()
 
@@ -1199,9 +1204,8 @@ class GoogleJobStoreTest(AbstractJobStoreTest.Test):
             bucket.delete()
 
 
-@needs_aws
+@needs_aws_s3
 class AWSJobStoreTest(AbstractJobStoreTest.Test):
-
     def _createJobStore(self):
         from toil.jobStores.aws.jobStore import AWSJobStore
         partSize = self._partSize()
@@ -1210,7 +1214,7 @@ class AWSJobStoreTest(AbstractJobStoreTest.Test):
     def _corruptJobStore(self):
         from toil.jobStores.aws.jobStore import AWSJobStore
         assert isinstance(self.jobstore_initialized, AWSJobStore)  # type hinting
-        self.jobstore_initialized.filesBucket.delete()
+        self.jobstore_initialized.destroy()
 
     def testSDBDomainsDeletedOnFailedJobstoreBucketCreation(self):
         """
@@ -1220,6 +1224,7 @@ class AWSJobStoreTest(AbstractJobStoreTest.Test):
         """
         from boto.sdb import connect_to_region
         from boto.s3.connection import Location, S3Connection
+        from boto.exception import S3ResponseError
         from toil.jobStores.aws.jobStore import BucketLocationConflictException
         from toil.jobStores.aws.utils import retry_s3
         externalAWSLocation = Location.USWest
@@ -1231,7 +1236,7 @@ class AWSJobStoreTest(AbstractJobStoreTest.Test):
             testJobStoreUUID = str(uuid.uuid4())
             # Create the bucket at the external region
             s3 = S3Connection()
-            for attempt in retry_s3(delays=(2,5,10,30,60), timeout=600):
+            for attempt in retry_s3(delays=(2, 5, 10, 30, 60), timeout=600):
                 with attempt:
                     bucket = s3.create_bucket('domain-test-' + testJobStoreUUID + '--files',
                                               location=externalAWSLocation)
@@ -1257,9 +1262,19 @@ class AWSJobStoreTest(AbstractJobStoreTest.Test):
             else:
                 self.fail()
             finally:
-                for attempt in retry_s3():
-                    with attempt:
-                        s3.delete_bucket(bucket=bucket)
+                try:
+                    for attempt in retry_s3():
+                        with attempt:
+                            s3.delete_bucket(bucket=bucket)
+                except S3ResponseError as e:
+                    # The actual HTTP code of the error is in status.
+                    # See https://github.com/boto/boto/blob/91ba037e54ef521c379263b0ac769c66182527d7/boto/exception.py#L77-L80
+                    # See also: https://github.com/boto/boto/blob/91ba037e54ef521c379263b0ac769c66182527d7/boto/exception.py#L154-L156
+                    if e.status == 404:
+                        # The bucket doesn't exist; maybe a failed delete actually succeeded.
+                        pass
+                    else:
+                        raise
 
     @slow
     def testInlinedFiles(self):
@@ -1275,23 +1290,21 @@ class AWSJobStoreTest(AbstractJobStoreTest.Test):
                 with jobstore.readSharedFileStream('foo') as f:
                     self.assertEqual(s, f.read())
 
-    def testInaccessableLocation(self):
-        url = 's3://toil-no-location-bucket-dont-delete/README'
-        with patch('toil.jobStores.aws.jobStore.log') as mock_log:
-            jobStoreID = self.jobstore_initialized.importFile(url)
-            self.assertTrue(self.jobstore_initialized.fileExists(jobStoreID))
-
     def testOverlargeJob(self):
         jobstore = self.jobstore_initialized
         jobRequirements = dict(memory=12, cores=34, disk=35, preemptable=True)
         overlargeJobNode = JobNode(command='overlarge',
-                                    requirements=jobRequirements,
-                                    jobName='test-overlarge', unitName='onJobStore',
-                                    jobStoreID=None, predecessorNumber=0)
+                                   requirements=jobRequirements,
+                                   jobName='test-overlarge', unitName='onJobStore',
+                                   jobStoreID=None, predecessorNumber=0)
 
-        #Make the pickled size of the job larger than 256K
-        with open("/dev/urandom", "r") as random:
-            overlargeJobNode.jobName = random.read(512 * 1024)
+        # Make the pickled size of the job larger than 256K
+        read_type = 'r' if USING_PYTHON2 else 'rb'
+        with open("/dev/urandom", read_type) as random:
+            if USING_PYTHON2:
+                overlargeJobNode.jobName = random.read(512 * 1024)
+            else:
+                overlargeJobNode.jobName = str(random.read(512 * 1024))
         overlargeJob = jobstore.create(overlargeJobNode)
         self.assertTrue(jobstore.exists(overlargeJob.jobStoreID))
         overlargeJobDownloaded = jobstore.load(overlargeJob.jobStoreID)
@@ -1304,8 +1317,12 @@ class AWSJobStoreTest(AbstractJobStoreTest.Test):
         url = 's3://%s/%s' % (bucket.name, fileName)
         if size is None:
             return url
-        with open('/dev/urandom', 'r') as readable:
-            bucket.new_key(fileName).set_contents_from_string(readable.read(size))
+        read_type = 'r' if USING_PYTHON2 else 'rb'
+        with open('/dev/urandom', read_type) as readable:
+            if USING_PYTHON2:
+                bucket.new_key(fileName).set_contents_from_string(readable.read(size))
+            else:
+                bucket.new_key(fileName).set_contents_from_string(str(readable.read(size)))
         return url, hashlib.md5(bucket.get_key(fileName).get_contents_as_string()).hexdigest()
 
     def _hashTestFile(self, url):
@@ -1346,7 +1363,7 @@ class AWSJobStoreTest(AbstractJobStoreTest.Test):
         return AWSJobStore.itemsPerBatchDelete
 
 
-@needs_aws
+@needs_aws_s3
 class InvalidAWSJobStoreTest(ToilTest):
     def testInvalidJobStoreName(self):
         from toil.jobStores.aws.jobStore import AWSJobStore
@@ -1361,121 +1378,16 @@ class InvalidAWSJobStoreTest(ToilTest):
                           'us-west-2:a_b')
 
 
-@needs_azure
-class AzureJobStoreTest(AbstractJobStoreTest.Test):
-    accountName = os.getenv('TOIL_AZURE_KEYNAME')
-
-    def _createJobStore(self):
-        from toil.jobStores.azureJobStore import AzureJobStore
-        return AzureJobStore(self.accountName + ':' + self.namePrefix)
-
-    def _corruptJobStore(self):
-        from toil.jobStores.azureJobStore import AzureJobStore
-        assert isinstance(self.jobstore_initialized, AzureJobStore)  # type hinting
-        self.jobstore_initialized.tableService.delete_table(self.jobstore_initialized.jobFileIDs)
-
-    def _partSize(self):
-        from toil.jobStores.azureJobStore import AzureJobStore
-        return AzureJobStore._maxAzureBlockBytes
-
-    def testLargeJob(self):
-        from toil.jobStores.azureJobStore import maxAzureTablePropertySize
-        command = os.urandom(maxAzureTablePropertySize * 2)
-        jobNode1 = self.arbitraryJob
-        jobNode1.command=command
-        job1 = self.jobstore_initialized.create(jobNode1)
-        self.assertEqual(job1.command, command)
-        job2 = self.jobstore_initialized.load(job1.jobStoreID)
-        self.assertIsNot(job1, job2)
-        self.assertEqual(job2.command, command)
-
-    def testJobStoreExists(self):
-        from toil.jobStores.azureJobStore import AzureJobStore
-        assert isinstance(self.jobstore_initialized, AzureJobStore)  # mostly for type hinting
-        self.assertTrue(self.jobstore_initialized._jobStoreExists())
-        self.jobstore_initialized.destroy()
-        self.assertFalse(self.jobstore_initialized._jobStoreExists())
-
-    def _prepareTestFile(self, containerName, size=None):
-        from toil.jobStores.azureJobStore import _fetchAzureAccountKey
-        from azure.storage.blob.blockblobservice import BlockBlobService
-
-        fileName = 'testfile_%s' % uuid.uuid4()
-        url = 'wasb://%s@%s.blob.core.windows.net/%s' % (containerName, self.accountName, fileName)
-        if size is None:
-            return url
-        blobService = BlockBlobService(account_key=_fetchAzureAccountKey(self.accountName),
-                                       account_name=self.accountName)
-        content = os.urandom(size)
-        blobService.create_blob_from_text(containerName, fileName, content)
-        return url, hashlib.md5(content).hexdigest()
-
-    def _hashTestFile(self, url):
-        from toil.jobStores.azureJobStore import AzureJobStore, retry_azure
-        url = urlparse.urlparse(url)
-        blob = AzureJobStore._parseWasbUrl(url)
-        for attempt in retry_azure():
-            with attempt:
-                blob = blob.service.get_blob_to_bytes(blob.container, blob.name)
-                return hashlib.md5(blob.content).hexdigest()
-
-    def _createExternalStore(self):
-        from toil.jobStores.azureJobStore import _fetchAzureAccountKey
-        from azure.storage.blob.blockblobservice import BlockBlobService
-
-        blobService = BlockBlobService(account_key=_fetchAzureAccountKey(self.accountName),
-                                       account_name=self.accountName)
-        containerName = 'import-export-test-%s' % uuid.uuid4()
-        blobService.create_container(containerName)
-        return containerName
-
-    def _cleanUpExternalStore(self, containerName):
-        from toil.jobStores.azureJobStore import _fetchAzureAccountKey
-        from azure.storage.blob.blockblobservice import BlockBlobService
-        blobService = BlockBlobService(account_key=_fetchAzureAccountKey(self.accountName),
-                                       account_name=self.accountName)
-        blobService.delete_container(containerName)
-
-
-@needs_azure
-class InvalidAzureJobStoreTest(ToilTest):
-    def testInvalidJobStoreName(self):
-        from toil.jobStores.azureJobStore import AzureJobStore
-        self.assertRaises(ValueError,
-                          AzureJobStore,
-                          'toiltest:a--b')
-        self.assertRaises(ValueError,
-                          AzureJobStore,
-                          'toiltest:' + ('a' * 100))
-        self.assertRaises(ValueError,
-                          AzureJobStore,
-                          'toiltest:a_b')
-
-@needs_aws
+@needs_aws_s3
 @needs_encryption
 @slow
 class EncryptedAWSJobStoreTest(AWSJobStoreTest, AbstractEncryptedJobStoreTest.Test):
     pass
 
 
-@needs_azure
-@needs_encryption
-@slow
-class EncryptedAzureJobStoreTest(AzureJobStoreTest, AbstractEncryptedJobStoreTest.Test):
-    pass
-
-# TODO: Has this ever actually run?  Duplicates the AzureJobstore Testing needlessly while
-# claiming to be the google jobstore.
-
-# @needs_google
-# @needs_encryption
-# @slow
-# class EncryptedGoogleJobStoreTest(AzureJobStoreTest, AbstractEncryptedJobStoreTest.Test):
-#     pass
-
-
 class StubHttpRequestHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):
     fileContents = 'A good programmer looks both ways before crossing a one-way street'
+
     def do_GET(self):
         self.send_response(200)
         self.send_header("Content-type", "text/plain")
