@@ -35,7 +35,7 @@ from six import iteritems
 
 from toil.job import JobNode, Job
 from toil.lib.humanize import human2bytes as h2b
-from toil.test import ToilTest, slow
+from toil.test import ToilTest, slow, travis_test
 from toil.batchSystems.abstractBatchSystem import (AbstractScalableBatchSystem,
                                                    NodeInfo,
                                                    AbstractBatchSystem)
@@ -78,7 +78,8 @@ class BinPackingTest(ToilTest):
     def setUp(self):
         self.nodeShapes = [c4_8xlarge_preemptable, r3_8xlarge]
         self.bpf = BinPackedFit(self.nodeShapes)
-
+    
+    @travis_test
     def testPackingOneShape(self):
         """Pack one shape and check that the resulting reservations look sane."""
         self.bpf.nodeReservations[c4_8xlarge_preemptable] = [NodeReservation(c4_8xlarge_preemptable)]
@@ -99,7 +100,8 @@ class BinPackingTest(ToilTest):
                                  cores=36,
                                  disk=h2b('100G'),
                                  preemptable=True)]])
-
+    
+    @travis_test
     def testSorting(self):
         """
         Test that sorting is correct: preemptable, then memory, then cores, then disk,
@@ -112,7 +114,8 @@ class BinPackingTest(ToilTest):
                              t2_micro, t2_micro, t2_micro,
                              c4_8xlarge, c4_8xlarge, c4_8xlarge,
                              r3_8xlarge, r3_8xlarge, r3_8xlarge]
-
+    
+    @travis_test
     def testAddingInitialNode(self):
         """Pack one shape when no nodes are available and confirm that we fit one node properly."""
         self.bpf.addJobShape(Shape(wallTime=1000,
@@ -131,7 +134,8 @@ class BinPackingTest(ToilTest):
                                  cores=36,
                                  disk=h2b('100G'),
                                  preemptable=True)]])
-
+    
+    @travis_test
     def testLowTargetTime(self):
         """
         Test that a low targetTime (0) parallelizes jobs aggressively (1000 queued jobs require
@@ -152,7 +156,8 @@ class BinPackingTest(ToilTest):
                                               jobTime=300,
                                               globalTargetTime=0)
         self.assertEqual(allocation, {t2_micro: 1000})
-
+    
+    @travis_test
     def testHighTargetTime(self):
         """
         Test that a high targetTime (3600 seconds) maximizes packing within the targetTime.
@@ -172,7 +177,8 @@ class BinPackingTest(ToilTest):
                                               jobTime=300,
                                               globalTargetTime=3600)
         self.assertEqual(allocation, {t2_micro: 84})
-
+    
+    @travis_test
     def testZeroResourceJobs(self):
         """
         Test that jobs requiring zero cpu/disk/mem pack first, regardless of targetTime.
@@ -190,7 +196,8 @@ class BinPackingTest(ToilTest):
                                               jobTime=300,
                                               globalTargetTime=0)
         self.assertEqual(allocation, {t2_micro: 1})
-
+    
+    @travis_test
     def testLongRunningJobs(self):
         """
         Test that jobs with long run times (especially service jobs) are aggressively parallelized.
@@ -223,7 +230,8 @@ class BinPackingTest(ToilTest):
                                    disk=jobDisk,
                                    preemptable=False))
         return bpf.getRequiredNodes()
-
+    
+    @travis_test
     def testPathologicalCase(self):
         """Test a pathological case where only one node can be requested to fit months' worth of jobs.
 
@@ -248,7 +256,8 @@ class BinPackingTest(ToilTest):
                                        preemptable=False))
         # Hopefully we didn't assign just one node to cover all those jobs.
         self.assertNotEqual(self.bpf.getRequiredNodes(), {r3_8xlarge: 1, c4_8xlarge_preemptable: 0})
-
+    
+    @travis_test
     def testJobTooLargeForAllNodes(self):
         """
         If a job is too large for all node types, the scaler should print a
@@ -274,8 +283,12 @@ class ClusterScalerTest(ToilTest):
             # In Python 3 we can use a SimpleNamespace as a mock provisioner
             self.provisioner = types.SimpleNamespace()
         except:
-            # In Python 2 we can just tack fields onto an object
-            self.provisioner = object()
+            # In Python 2 we should just be able to tack fields onto an object.
+            # But this has been known to produce:
+            # AttributeError: 'newobject' object has no attribute 'nodeTypes'
+            # So we use an Argparse Namespace instead.
+            import argparse
+            self.provisioner = argparse.Namespace()
         setattr(self.provisioner, 'nodeTypes', ['r3.8xlarge', 'c4.8xlarge'])
         setattr(self.provisioner, 'nodeShapes', [r3_8xlarge,
                                                  c4_8xlarge_preemptable])
@@ -283,7 +296,8 @@ class ClusterScalerTest(ToilTest):
         setattr(self.provisioner, 'retryPredicate', lambda _: False)
 
         self.leader = MockBatchSystemAndProvisioner(self.config, 1)
-        
+    
+    @travis_test
     def testRounding(self):
         """
         Test to make sure the ClusterScaler's rounding rounds properly.
@@ -315,7 +329,8 @@ class ClusterScalerTest(ToilTest):
         self.assertEqual(scaler._round(15.5), 16)
         self.assertEqual(scaler._round(-15.5), -16)
         self.assertEqual(scaler._round(123456789101112.5), 123456789101113)
-
+    
+    @travis_test
     def testMaxNodes(self):
         """
         Set the scaler to be very aggressive, give it a ton of jobs, and
@@ -338,7 +353,8 @@ class ClusterScalerTest(ToilTest):
         estimatedNodeCounts = scaler.getEstimatedNodeCounts(jobShapes, defaultdict(int))
         self.assertEqual(estimatedNodeCounts[r3_8xlarge], 2)
         self.assertEqual(estimatedNodeCounts[c4_8xlarge_preemptable], 3)
-
+    
+    @travis_test
     def testMinNodes(self):
         """
         Without any jobs queued, the scaler should still estimate "minNodes" nodes.
@@ -350,7 +366,8 @@ class ClusterScalerTest(ToilTest):
         estimatedNodeCounts = scaler.getEstimatedNodeCounts(jobShapes, defaultdict(int))
         self.assertEqual(estimatedNodeCounts[r3_8xlarge], 2)
         self.assertEqual(estimatedNodeCounts[c4_8xlarge_preemptable], 3)
-
+    
+    @travis_test
     def testPreemptableDeficitResponse(self):
         """
         When a preemptable deficit was detected by a previous run of the
@@ -388,7 +405,8 @@ class ClusterScalerTest(ToilTest):
         # nodes. All we want to know is if we responded to the deficit
         # properly: 0.5 * 5 (preemptableCompensation * the deficit) = 3 (rounded up).
         self.assertEqual(estimatedNodeCounts[self.provisioner.nodeShapes[1]], 3)
-
+    
+    @travis_test
     def testPreemptableDeficitIsSet(self):
         """
         Make sure that updateClusterSize sets the preemptable deficit if
@@ -420,7 +438,8 @@ class ClusterScalerTest(ToilTest):
         self.provisioner.addNodes = MagicMock(return_value=5)
         scaler.updateClusterSize(estimatedNodeCounts)
         self.assertEqual(scaler.preemptableNodeDeficit['c4.8xlarge'], 0)
-
+    
+    @travis_test
     def testNoLaunchingIfDeltaAlreadyMet(self):
         """
         Check that the scaler doesn't try to launch "0" more instances if
@@ -444,7 +463,8 @@ class ClusterScalerTest(ToilTest):
         self.assertEqual(len(scaler.ignoredNodes), 0,
                          "The scaler didn't unignore an ignored node when "
                          "scaling up")
-
+    
+    @travis_test
     def testBetaInertia(self):
         # This is really high, but makes things easy to calculate.
         self.config.betaInertia = 0.5
