@@ -16,7 +16,7 @@ import os
 
 from toil.lib.bioio import getTempFile
 from toil.job import Job as T
-from toil.test import ToilTest
+from toil.test import ToilTest, travis_test
 from toil.test.src.jobTest import fn1Test as f
 
 
@@ -24,7 +24,8 @@ class JobEncapsulationTest(ToilTest):
     """
     Tests testing the EncapsulationJob class
     """
-
+    
+    @travis_test
     def testEncapsulation(self):
         """
         Tests the Job.encapsulation method, which uses the EncapsulationJob
@@ -50,7 +51,8 @@ class JobEncapsulationTest(ToilTest):
             self.assertEqual(open(outFile, 'r').readline(), "ABCDE")
         finally:
             os.remove(outFile)
-
+    
+    @travis_test
     def testAddChildEncapsulate(self):
         """
         Make sure that the encapsulate child does not have two parents
