@@ -14,14 +14,15 @@
 from __future__ import absolute_import
 from builtins import range
 from toil.job import Job
-from toil.test import ToilTest
+from toil.test import ToilTest, travis_test
 
 
 class CachedUnpicklingJobStoreTest(ToilTest):
     """
     https://github.com/BD2KGenomics/toil/issues/817
     """
-
+    
+    @travis_test
     def test(self):
         """
         Runs two identical Toil workflows with different job store paths
@@ -45,7 +46,8 @@ class ChainedIndexedPromisesTest(ToilTest):
     """
     https://github.com/BD2KGenomics/toil/issues/1021
     """
-
+    
+    @travis_test
     def test(self):
         options = Job.Runner.getDefaultOptions(self._getTestJobStorePath())
         options.logLevel = 'INFO'
@@ -70,7 +72,8 @@ class PathIndexingPromiseTest(ToilTest):
     Test support for indexing promises of arbitrarily nested data structures of lists, dicts and
     tuples, or any other object supporting the __getitem__() protocol.
     """
-
+    
+    @travis_test
     def test(self):
         options = Job.Runner.getDefaultOptions(self._getTestJobStorePath())
         options.logLevel = 'INFO'
