@@ -18,7 +18,7 @@ import sys
 import os
 
 from toil import subprocess
-from toil.test import ToilTest, slow
+from toil.test import ToilTest, slow, travis_test
 from toil.test.mesos import helloWorld
 
 logging.basicConfig(level=logging.DEBUG)
@@ -61,7 +61,8 @@ class RegularLogTest(ToilTest):
                                               '--clean=always',
                                               '--logLevel=info'], stderr=subprocess.STDOUT)
         assert helloWorld.childMessage in toilOutput.decode('utf-8')
-
+    
+    @travis_test
     def testWriteLogs(self):
         subprocess.check_call([sys.executable,
                                '-m', helloWorld.__name__,
@@ -89,7 +90,8 @@ class RegularLogTest(ToilTest):
                                               '--clean=always',
                                               '--logLevel=info'], stderr=subprocess.STDOUT)
         assert helloWorld.parentMessage in toilOutput.decode('utf-8')
-
+    
+    @travis_test
     def testRegularLog(self):
         toilOutput = subprocess.check_output([sys.executable,
                                               '-m', helloWorld.__name__,
