@@ -29,6 +29,7 @@ from toil import subprocess
 from toil import inVirtualEnv
 from toil.resource import ModuleDescriptor, Resource, ResourceException
 from toil.test import ToilTest, tempFileContaining, travis_test
+from toil.version import exactPython
 
 
 class ResourceTest(ToilTest):
@@ -72,8 +73,10 @@ class ResourceTest(ToilTest):
         if virtualenv:
             self.assertTrue(inVirtualEnv())
             # --never-download prevents silent upgrades to pip, wheel and setuptools
-            subprocess.check_call(['virtualenv', '--never-download', dirPath])
-            sitePackages = os.path.join(dirPath, 'lib', 'python2.7', 'site-packages')
+            subprocess.check_call(['virtualenv', '--never-download', '--python', exactPython, dirPath])
+            sitePackages = os.path.join(dirPath, 'lib',
+                                        exactPython,
+                                        'site-packages')
             # tuple assignment is necessary to make this line immediately precede the try:
             oldPrefix, sys.prefix, dirPath = sys.prefix, dirPath, sitePackages
         else:
@@ -180,7 +183,7 @@ class ResourceTest(ToilTest):
             # The name should be equal between original and localized resource ...
             self.assertEqual(module.name, localModule.name)
             # ... but the directory should be different.
-            self.assertNotEquals(module.dirPath, localModule.dirPath)
+            self.assertNotEqual(module.dirPath, localModule.dirPath)
             # Show that we can 'undo' localization. This is necessary when the user script's jobs
             #  are invoked on the worker where they generate more child jobs.
             self.assertEqual(localModule.globalize(), module)
@@ -222,5 +225,5 @@ class ResourceTest(ToilTest):
             process = subprocess.Popen([scriptPath, jobStorePath], stderr=subprocess.PIPE)
             stdout, stderr = process.communicate()
             self.assertTrue('The name of a user script/module must end in .py or .pyc.' in stderr.decode('utf-8'))
-            self.assertNotEquals(0, process.returncode)
+            self.assertNotEqual(0, process.returncode)
             self.assertFalse(os.path.exists(jobStorePath))
