@@ -34,11 +34,12 @@ from toil import resolveEntryPoint
 from toil.job import Job
 from toil.utils.toilStatus import ToilStatus
 from toil.lib.bioio import getTempFile, system
-from toil.test import ToilTest, needs_aws, needs_rsync3, integrative, slow, needs_cwl, needs_docker
+from toil.test import ToilTest, needs_aws_ec2, needs_rsync3, integrative, slow, needs_cwl, needs_docker, travis_test
 from toil.test.sort.sortTest import makeFileToSort
 from toil.utils.toilStats import getStats, processData
 from toil.common import Toil, Config
 from toil.provisioners import clusterFactory
+from toil.version import python
 
 logger = logging.getLogger(__name__)
 
@@ -65,7 +66,7 @@ class UtilsTest(ToilTest):
             self.correctSort = fileHandle.readlines()
             self.correctSort.sort()
 
-        self.sort_workflow_cmd = ['python', '-m', 'toil.test.sort.sort',
+        self.sort_workflow_cmd = [python, '-m', 'toil.test.sort.sort',
                                   'file:' + self.toilDir,
                                   '--clean=never',
                                   '--numLines=1', '--lineLength=1']
@@ -102,7 +103,7 @@ class UtilsTest(ToilTest):
 
     @needs_rsync3
     @pytest.mark.timeout(1200)
-    @needs_aws
+    @needs_aws_ec2
     @integrative
     @slow
     def testAWSProvisionerUtils(self):
@@ -168,7 +169,7 @@ class UtilsTest(ToilTest):
             for test in testStrings:
                 logger.debug('Testing SSH with special string: %s', test)
                 compareTo = "import sys; assert sys.argv[1]==%r" % test
-                leader.sshAppliance('python', '-', test, input=compareTo)
+                leader.sshAppliance(python, '-', test, input=compareTo)
 
             try:
                 leader.sshAppliance('nonsenseShouldFail')
@@ -177,7 +178,7 @@ class UtilsTest(ToilTest):
             else:
                 self.fail('The remote command failed silently where it should have raised an error')
 
-            leader.sshAppliance('python', '-c', "import os; assert os.environ['TOIL_WORKDIR']=='/var/lib/toil'")
+            leader.sshAppliance(python, '-c', "import os; assert os.environ['TOIL_WORKDIR']=='/var/lib/toil'")
 
             # `toil rsync-cluster`
             # Testing special characters - string.punctuation
@@ -303,7 +304,8 @@ class UtilsTest(ToilTest):
 
         # Delete output file
         os.remove(self.outputFile)
-
+    
+    @travis_test
     def testUnicodeSupport(self):
         options = Job.Runner.getDefaultOptions(self._getTestJobStorePath())
         options.clean = 'always'
@@ -334,7 +336,8 @@ class UtilsTest(ToilTest):
             if i > seconds:
                 s = status_fn(self.toilDir)
                 self.assertEqual(s, status, 'Status took longer than 10 seconds to fetch:  %s' % s)
-
+    
+    @travis_test
     def testGetPIDStatus(self):
         """Test that ToilStatus.getPIDStatus() behaves as expected."""
         wf = subprocess.Popen(self.sort_workflow_cmd)
@@ -342,9 +345,12 @@ class UtilsTest(ToilTest):
         wf.wait()
         self.check_status('COMPLETED', status_fn=ToilStatus.getPIDStatus)
 
-        os.remove(os.path.join(self.toilDir, 'pid.log'))
+        # TODO: we need to reach into the FileJobStore's files and
+        # delete this shared file. We assume we know its internal layout.
+        os.remove(os.path.join(self.toilDir, 'files/shared/pid.log'))
         self.check_status('QUEUED', status_fn=ToilStatus.getPIDStatus)
-
+    
+    @travis_test
     def testGetStatusFailedToilWF(self):
         """
         Test that ToilStatus.getStatus() behaves as expected with a failing Toil workflow.
