@@ -302,7 +302,7 @@ def sprintTag(key, tag, options, columnWidths=None):
             (tag.max_memory, columnWidths.getWidth("memory", "max")),
             (tag.total_memory, columnWidths.getWidth("memory", "total")),
             ]:
-            tag_str += reportMemory(t, options, field=width, isBytes=True)
+            tag_str += reportMemory(t, options, field=width)
     out_str += header + "\n"
     out_str += sub_header + "\n"
     out_str += tag_str + "\n"
@@ -452,9 +452,11 @@ def buildElement(element, items, itemName):
     itemClocks = []
     itemMemory = []
     for item in items:
-        itemTimes.append(assertNonnegative(float(item["time"]), "time"))
-        itemClocks.append(assertNonnegative(float(item["clock"]), "clock"))
-        itemMemory.append(assertNonnegative(float(item["memory"]), "memory"))
+        # If something lacks an entry, assume it used none of that thing.
+        # This avoids crashing when jobs e.g. aren't done.
+        itemTimes.append(assertNonnegative(float(item.get("time", 0)), "time"))
+        itemClocks.append(assertNonnegative(float(item.get("clock", 0)), "clock"))
+        itemMemory.append(assertNonnegative(float(item.get("memory", 0)), "memory"))
     assert len(itemClocks) == len(itemTimes) == len(itemMemory)
 
     itemWaits=[]
@@ -532,14 +534,16 @@ def getStats(jobStore):
 
 
 def processData(config, stats):
-    ##########################################
-    # Collate the stats and report
-    ##########################################
-    if stats.get("total_time", None) is None:  # Hack to allow unfinished toils.
-        stats.total_time = {"total_time": "0.0", "total_clock": "0.0"}
-    else:
-        stats.total_time = sum([float(number) for number in stats.total_time])
-        stats.total_clock = sum([float(number) for number in stats.total_clock])
+    """
+    Collate the stats and report
+    """
+    if 'total_time' not in stats or 'total_clock' not in stats:
+        # toil job not finished yet
+        stats.total_time = [0.0]
+        stats.total_clock = [0.0]
+
+    stats.total_time = sum([float(number) for number in stats.total_time])
+    stats.total_clock = sum([float(number) for number in stats.total_clock])
 
     collatedStatsTag = Expando(total_run_time=stats.total_time,
                                total_clock=stats.total_clock,
@@ -550,8 +554,8 @@ def processData(config, stats):
                                )
 
     # Add worker info
-    worker = [_f for _f in stats.workers if _f]
-    jobs = [_f for _f in stats.jobs if _f]
+    worker = [_f for _f in getattr(stats, 'workers', []) if _f]
+    jobs = [_f for _f in getattr(stats, 'jobs', []) if _f]
     jobs = [item for sublist in jobs for item in sublist]
 
     def fn4(job):
@@ -562,7 +566,7 @@ def processData(config, stats):
 
     buildElement(collatedStatsTag, worker, "worker")
     createSummary(buildElement(collatedStatsTag, jobs, "jobs"),
-                  stats.workers, "worker", fn4)
+                  getattr(stats, 'workers', []), "worker", fn4)
     # Get info for each job
     jobNames = set()
     for job in jobs:
