@@ -507,10 +507,21 @@ def combine_dicts(dict1, dict2):
     return combineddict
 
 
+def basename(path, suffix=None):
+    """https://software.broadinstitute.org/wdl/documentation/article?id=10554"""
+    path = path.strip()
+    if suffix:
+        suffix = suffix.strip()
+        if path.endswith(suffix):
+            path = path[:-len(suffix)]
+    return os.path.basename(path)
+
+
 def heredoc_wdl(template, dictionary={}, indent=''):
     template = textwrap.dedent(template).format(**dictionary)
     return template.replace('\n', '\n' + indent) + '\n'
 
+
 def read_tsv(f, delimiter="\t"):
     '''
     Take a tsv filepath and return an array; e.g. [[],[],[]].
@@ -533,6 +544,7 @@ def read_tsv(f, delimiter="\t"):
             tsv_array.append(line)
     return (tsv_array)
 
+
 def read_csv(f):
     '''
     Take a csv filepath and return an array; e.g. [[],[],[]].
