@@ -25,20 +25,27 @@ import json
 import tempfile
 import traceback
 import time
+import signal
 import socket
 import logging
 import shutil
-from threading import Thread
 
+from toil.lib.compatibility import USING_PYTHON2
 from toil.lib.expando import MagicExpando
 from toil.common import Toil, safeUnpickleFromStream
-from toil.fileStore import FileStore
+from toil.fileStores.abstractFileStore import AbstractFileStore
 from toil import logProcessContext
 from toil.job import Job
 from toil.lib.bioio import setLogLevel
 from toil.lib.bioio import getTotalCpuTime
 from toil.lib.bioio import getTotalCpuTimeAndMemoryUsage
-import signal
+from toil.deferred import DeferredFunctionManager
+try:
+    from toil.cwl.cwltoil import CWL_INTERNAL_JOBS
+except ImportError:
+    # CWL extra not installed
+    CWL_INTERNAL_JOBS = ()
+
 
 logging.basicConfig()
 logger = logging.getLogger(__name__)
@@ -106,7 +113,6 @@ def workerScript(jobStore, config, jobName, jobStoreID, redirectOutputToLogFile=
     :param str jobName: The "job name" (a user friendly name) of the job to be run
     :param str jobStoreLocator: Specifies the job store to use
     :param str jobStoreID: The job store ID of the job to be run
-    :param bool redirectOutputToLogFile: Redirect standard out and standard error to a log file
     """
     logging.basicConfig()
     setLogLevel(config.logLevel)
@@ -116,7 +122,7 @@ def workerScript(jobStore, config, jobName, jobStoreID, redirectOutputToLogFile=
     ##########################################
 
     logFileByteReportLimit = config.maxLogFileSize
-
+    
     if config.badWorker > 0 and random.random() < config.badWorker:
         # We need to kill the process we are currently in, to simulate worker
         # failure. We don't want to just send SIGKILL, because we can't tell
@@ -147,7 +153,7 @@ def workerScript(jobStore, config, jobName, jobStoreID, redirectOutputToLogFile=
             except OSError:
                 pass
             # Exit without doing any of Toil's cleanup
-            os._exit()
+            os._exit(0)
             
         # We don't need to reap the child. Either it kills us, or we finish
         # before it does. Either way, init will have to clean it up for us.
@@ -159,17 +165,37 @@ def workerScript(jobStore, config, jobName, jobStoreID, redirectOutputToLogFile=
     #First load the environment for the jobGraph.
     with jobStore.readSharedFileStream("environment.pickle") as fileHandle:
         environment = safeUnpickleFromStream(fileHandle)
+    env_blacklist = {
+        "TMPDIR",
+        "TMP",
+        "HOSTNAME",
+        "HOSTTYPE",
+        "HOME",
+        "LOGNAME",
+        "USER",
+        "DISPLAY",
+        "JAVA_HOME"
+    }
     for i in environment:
-        if i not in ("TMPDIR", "TMP", "HOSTNAME", "HOSTTYPE"):
+        if i == "PATH":
+            # Handle path specially. Sometimes e.g. leader may not include
+            # /bin, but the Toil appliance needs it.
+            if i in os.environ and os.environ[i] != '':
+                # Use the provided PATH and then the local system's PATH
+                os.environ[i] = environment[i] + ':' + os.environ[i]
+            else:
+                # Use the provided PATH only
+                os.environ[i] = environment[i]
+        elif i not in env_blacklist:
             os.environ[i] = environment[i]
     # sys.path is used by __import__ to find modules
     if "PYTHONPATH" in environment:
         for e in environment["PYTHONPATH"].split(':'):
             if e != '':
                 sys.path.append(e)
-
+                
     toilWorkflowDir = Toil.getWorkflowDir(config.workflowID, config.workDir)
-    
+
     ##########################################
     #Setup the temporary directories.
     ##########################################
@@ -190,6 +216,9 @@ def workerScript(jobStore, config, jobName, jobStoreID, redirectOutputToLogFile=
     
     #When we start, standard input is file descriptor 0, standard output is
     #file descriptor 1, and standard error is file descriptor 2.
+    
+    # Do we even want to redirect output? Let the config make us not do it.
+    redirectOutputToLogFile = redirectOutputToLogFile and not config.disableWorkerOutputCapture
 
     #What file do we want to point FDs 1 and 2 to?
     tempWorkerLogPath = os.path.join(localWorkerTempDir, "worker_log.txt")
@@ -243,6 +272,11 @@ def workerScript(jobStore, config, jobName, jobStoreID, redirectOutputToLogFile=
         sys.stdout.flush()
         
         logProcessContext(config)
+        
+        ##########################################
+        #Connect to the deferred function system
+        ##########################################
+        deferredFunctionManager = DeferredFunctionManager(toilWorkflowDir)
 
         ##########################################
         #Load the jobGraph
@@ -319,16 +353,20 @@ def workerScript(jobStore, config, jobName, jobStoreID, redirectOutputToLogFile=
                     jobGraph.checkpoint = jobGraph.command
 
                 # Create a fileStore object for the job
-                fileStore = FileStore.createFileStore(jobStore, jobGraph, localWorkerTempDir, blockFn,
-                                                      caching=not config.disableCaching)
+                fileStore = AbstractFileStore.createFileStore(jobStore, jobGraph, localWorkerTempDir, blockFn,
+                                                              caching=not config.disableCaching)
                 with job._executor(jobGraph=jobGraph,
                                    stats=statsDict if config.stats else None,
                                    fileStore=fileStore):
-                    with fileStore.open(job):
-                        # Get the next block function and list that will contain any messages
-                        blockFn = fileStore._blockFn
+                    with deferredFunctionManager.open() as defer:
+                        with fileStore.open(job):
+                            # Get the next block function to wait on committing this job
+                            blockFn = fileStore.waitForCommit
 
-                        job._runner(jobGraph=jobGraph, jobStore=jobStore, fileStore=fileStore)
+                            job._runner(jobGraph=jobGraph, jobStore=jobStore, fileStore=fileStore, defer=defer)
+
+                            # When the job succeeds, start committing files immediately.
+                            fileStore.startCommit(jobState=False)      
 
                 # Accumulate messages from this job & any subsequent chained jobs
                 statsDict.workers.logsToMaster += fileStore.loggingMessages
@@ -340,7 +378,7 @@ def workerScript(jobStore, config, jobName, jobStoreID, redirectOutputToLogFile=
                 logger.debug("No user job to run, so finishing")
                 break
             
-            if FileStore._terminateEvent.isSet():
+            if AbstractFileStore._terminateEvent.isSet():
                 raise RuntimeError("The termination flag is set")
 
             ##########################################
@@ -349,6 +387,10 @@ def workerScript(jobStore, config, jobName, jobStoreID, redirectOutputToLogFile=
             successorJobGraph = nextChainableJobGraph(jobGraph, jobStore)
             if successorJobGraph is None or config.disableChaining:
                 # Can't chain any more jobs.
+                # TODO: why don't we commit the last job's file store? Won't
+                # its async uploads never necessarily finish?
+                # If we do call startCommit here it messes with the job
+                # itself and Toil thinks the job needs to run again.
                 break
 
             ##########################################
@@ -380,17 +422,17 @@ def workerScript(jobStore, config, jobName, jobStoreID, redirectOutputToLogFile=
             assert jobGraph.cores >= successorJobGraph.cores
             
             #Build a fileStore to update the job
-            fileStore = FileStore.createFileStore(jobStore, jobGraph, localWorkerTempDir, blockFn,
-                                                  caching=not config.disableCaching)
+            fileStore = AbstractFileStore.createFileStore(jobStore, jobGraph, localWorkerTempDir, blockFn,
+                                                          caching=not config.disableCaching)
 
             #Update blockFn
-            blockFn = fileStore._blockFn
+            blockFn = fileStore.waitForCommit
 
             #Add successorJobGraph to those to be deleted
             fileStore.jobsToDelete.add(successorJobGraph.jobStoreID)
             
             #This will update the job once the previous job is done
-            fileStore._updateJobWhenDone()            
+            fileStore.startCommit(jobState=True)            
             
             #Clone the jobGraph and its stack again, so that updates to it do
             #not interfere with this update
@@ -419,7 +461,7 @@ def workerScript(jobStore, config, jobName, jobStoreID, redirectOutputToLogFile=
     except: #Case that something goes wrong in worker
         traceback.print_exc()
         logger.error("Exiting the worker because of a failed job on host %s", socket.gethostname())
-        FileStore._terminateEvent.set()
+        AbstractFileStore._terminateEvent.set()
     
     ##########################################
     #Wait for the asynchronous chain of writes/updates to finish
@@ -432,7 +474,7 @@ def workerScript(jobStore, config, jobName, jobStoreID, redirectOutputToLogFile=
     #so safe to test if they completed okay
     ########################################## 
     
-    if FileStore._terminateEvent.isSet():
+    if AbstractFileStore._terminateEvent.isSet():
         jobGraph = jobStore.load(jobStoreID)
         jobGraph.setupJobAfterFailure(config)
         workerFailed = True
@@ -469,33 +511,44 @@ def workerScript(jobStore, config, jobName, jobStoreID, redirectOutputToLogFile=
 
     # Now our file handles are in exactly the state they were in before.
 
-    #Copy back the log file to the global dir, if needed
+    # Copy back the log file to the global dir, if needed.
+    # Note that we work with bytes instead of characters so we can seek
+    # relative to the end (since Python won't decode Unicode backward, or even
+    # interpret seek offsets in characters for us). TODO: We may get invalid or
+    # just different Unicode by breaking up a character at the boundary!
     if workerFailed and redirectOutputToLogFile:
-        jobGraph.logJobStoreFileID = jobStore.getEmptyFileStoreID(jobGraph.jobStoreID)
+        jobGraph.logJobStoreFileID = jobStore.getEmptyFileStoreID(jobGraph.jobStoreID, cleanup=True)
         jobGraph.chainedJobs = listOfJobs
         with jobStore.updateFileStream(jobGraph.logJobStoreFileID) as w:
-            with open(tempWorkerLogPath, "r") as f:
+            with open(tempWorkerLogPath, 'rb') as f:
                 if os.path.getsize(tempWorkerLogPath) > logFileByteReportLimit !=0:
                     if logFileByteReportLimit > 0:
                         f.seek(-logFileByteReportLimit, 2)  # seek to last tooBig bytes of file
                     elif logFileByteReportLimit < 0:
                         f.seek(logFileByteReportLimit, 0)  # seek to first tooBig bytes of file
-                w.write(f.read().encode('utf-8')) # TODO load file using a buffer
+                # Dump the possibly-invalid-Unicode bytes into the log file
+                w.write(f.read()) # TODO load file using a buffer
         jobStore.update(jobGraph)
 
-    elif debugging and redirectOutputToLogFile:  # write log messages
-        with open(tempWorkerLogPath, 'r') as logFile:
+    elif ((debugging or (config.writeLogsFromAllJobs and not jobName.startswith(CWL_INTERNAL_JOBS)))
+          and redirectOutputToLogFile):  # write log messages
+        with open(tempWorkerLogPath, 'rb') as logFile:
             if os.path.getsize(tempWorkerLogPath) > logFileByteReportLimit != 0:
                 if logFileByteReportLimit > 0:
                     logFile.seek(-logFileByteReportLimit, 2)  # seek to last tooBig bytes of file
                 elif logFileByteReportLimit < 0:
                     logFile.seek(logFileByteReportLimit, 0)  # seek to first tooBig bytes of file
-            logMessages = logFile.read().splitlines()
+            # Make sure lines are Unicode so they can be JSON serialized as part of the dict.
+            # We may have damaged the Unicode text by cutting it at an arbitrary byte so we drop bad characters.
+            logMessages = [line.decode('utf-8', 'skip') for line in logFile.read().splitlines()]
         statsDict.logs.names = listOfJobs
         statsDict.logs.messages = logMessages
 
     if (debugging or config.stats or statsDict.workers.logsToMaster) and not workerFailed:  # We have stats/logging to report back
-        jobStore.writeStatsAndLogging(json.dumps(statsDict, ensure_ascii=True))
+        if USING_PYTHON2:
+            jobStore.writeStatsAndLogging(json.dumps(statsDict, ensure_ascii=True))
+        else:
+            jobStore.writeStatsAndLogging(json.dumps(statsDict, ensure_ascii=True).encode())
 
     #Remove the temp dir
     cleanUp = config.cleanWorkDir
